/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates both interrupt and polling access mode of PWC timer.
 ** The PWM output is used as measured wave for PWC timer, connect P23(TIOA2_0) 
 ** with P49(TIOB0_0) before running the program.
 **
 ** History:
 **   - 2013-05-20  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
// PWM 
#define  USER_BT_PWM             BT2
#define  PwmInitOutput()         SetPinFunc_TIOA2_0_OUT();
// PWC
#define  USER_BT_PWC             BT0
#define  USER_BT_TIMER_SIZE      PwcSize32Bit
#define  PwcInitInput()          SetPinFunc_TIOB0_0_IN();
#define  MEASURE_CNT_MAX         (10)

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static uint32_t m_u32CntIntMeasure, m_u32CntIntOverflow, m_u32CntMeasureErr;
static uint32_t m_aMeasureResult[MEASURE_CNT_MAX];

/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/
static void PwcOverflowIntHandler(void);
static void PwcMeasCmpIntHandler(void);
/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/

/*!
 ******************************************************************************
 ** \brief PWC timer example code
 **
 ** 1. Register initialization
 ** 2. Callback function initialization.
 ** 3. Enable interrupt.
 ** 4. Enable count operation.
 ******************************************************************************
 */
int32_t main(void)
{
    stc_bt_pwc_config_t stcPwcConfig;
    stc_pwc_int_sel_t stcPwcIntSel;
    stc_bt_pwm_config_t stcPwmConfig;
    stc_pwc_int_cb_t  stcPwcIntCallback;
    uint32_t u32Cnt;
    
    /*------------- PWM Generation -----------------*/
    // Set IO port 
    PwmInitOutput() ; 
    // PWM configuration
    Bt_ConfigIOMode(&USER_BT_PWM, BtIoMode0);
    Bt_SelTimerMode(&USER_BT_PWM, BtPwmMode);
    stcPwmConfig.enPres = PwmPres1Div4;
    stcPwmConfig.enMode = PwmContinuous;
    stcPwmConfig.enExtTrig = PwmExtTrigDisable;
    stcPwmConfig.enOutputMask = PwmOutputNormal;
    stcPwmConfig.enOutputPolarity = PwmPolarityLow;
    stcPwmConfig.enRestartEn = PwmRestartEnable;
    Bt_Pwm_Init(&USER_BT_PWM , &stcPwmConfig);
    Bt_Pwm_WriteCycleVal(&USER_BT_PWM, 999);
    Bt_Pwm_WriteDutyVal(&USER_BT_PWM, 499);
    // Start PWM
    Bt_Pwm_EnableCount(&USER_BT_PWM);
    Bt_Pwm_EnableSwTrig(&USER_BT_PWM);
    /*---------------------------------------------*/
    
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("PWC Example Program Start \n");
    printf("==================================================\n");
#endif
    
    // Set IO port 
    PwcInitInput();
       
    // Set IO Mode 
    Bt_ConfigIOMode(&USER_BT_PWC, BtIoMode0);
    
    // Set PWM timer mode
    Bt_SelTimerMode(&USER_BT_PWC, BtPwcMode);
    
    // PWC register initialization 
    stcPwcConfig.enPres = PwcPres1Div4;
    stcPwcConfig.enMode = PwcContinuous;
    stcPwcConfig.enMeasureEdge = PwcMeasureRisingToFalling;
    stcPwcConfig.enSize = USER_BT_TIMER_SIZE;
    Bt_Pwc_Init(&USER_BT_PWC, &stcPwcConfig);
    
    // Enable Interrupt 
    stcPwcIntSel.bPwcMeasCmpInt = 1;
    stcPwcIntSel.bPwcMeasOverflowInt = 1;
    stcPwcIntCallback.pfnPwcMeasCmpCallback = PwcMeasCmpIntHandler;
    stcPwcIntCallback.pfnPwcMeasOverflowCallback = PwcOverflowIntHandler;
    Bt_Pwc_EnableInt(&USER_BT_PWC, &stcPwcIntSel, &stcPwcIntCallback);

#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Enable Interrupt mode, Waiting for mesaurment compeleted \n");
#endif      
    
    // Enable count operatoin 
    Bt_Pwc_EnableCount(&USER_BT_PWC);

    // Waiting for mesaurment compeleted
    while(m_u32CntIntMeasure < MEASURE_CNT_MAX);
    
    Bt_Pwc_DisableInt(&USER_BT_PWC, &stcPwcIntSel);
    
#ifdef DEBUG_PRINT
    printf("Interrupt times:\n"); 
    printf("BT_INT_TYP_MEASURE: %d\n", m_u32CntIntMeasure);    
    printf("BT_INT_TYP_OVERFLOW: %d\n", m_u32CntIntOverflow);
    printf("Measure error: %d\n", m_u32CntMeasureErr); 
    for(u32Cnt=0; u32Cnt<MEASURE_CNT_MAX; u32Cnt++)
    {
        printf("Measurement result %d: %d\n", (u32Cnt+1), m_aMeasureResult[u32Cnt]);   
    }
#endif     
 
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Disable Interrupt, enable query mode, Waiting for mesaurment compeleted\n");
#endif
 
    m_u32CntIntMeasure = 0;
    m_u32CntIntOverflow = 0;
    m_u32CntMeasureErr = 0; 
    
    for(u32Cnt = 0; u32Cnt < MEASURE_CNT_MAX; u32Cnt++)
    {
        m_aMeasureResult[u32Cnt] = 0;
    } 

    // Interrupt query mode 
    do 
    {   
        if (Bt_Pwc_GetIntFlag(&USER_BT_PWC, PwcMeasureCompleteInt) == PdlSet)
        {
            if (stcPwcConfig.enSize == PwcSize16Bit)
            {
                m_aMeasureResult[m_u32CntIntMeasure] = Bt_Pwc_Get16BitMeasureData(&USER_BT_PWC) +
                    (m_u32CntIntOverflow * 0xffff); 
            }
            else //BT_32IT_TIMER
            {
                //Comment this line if testing measure error feature 
		m_aMeasureResult[m_u32CntIntMeasure] = Bt_Pwc_Get32BitMeasureData(&USER_BT_PWC);
            }
            m_u32CntIntOverflow = 0;
            m_u32CntIntMeasure++;
            
            if (Bt_Pwc_GetErrorFlag(&USER_BT_PWC) == PdlSet)
            {            
                m_u32CntMeasureErr++;
            }
        }
        else if (Bt_Pwc_GetIntFlag(&USER_BT_PWC, PwcMeasureCompleteInt) == PdlSet)
        {   
            Bt_Pwc_ClrIntFlag(&USER_BT_PWC, PwcMeasureOverflowInt);
            m_u32CntIntOverflow++;
        }
        
    } while(m_u32CntIntMeasure < MEASURE_CNT_MAX);
    
#ifdef DEBUG_PRINT
    printf("Interrupt request times:\n"); 
    printf("BT_INT_TYP_MEASURE: %d\n", m_u32CntIntMeasure);    
    printf("BT_INT_TYP_OVERFLOW: %d\n", m_u32CntIntOverflow); 
    printf("Measure error: %d\n", m_u32CntMeasureErr); 
    for(u32Cnt=0; u32Cnt<MEASURE_CNT_MAX; u32Cnt++)
    {
        printf("Measurement result %d: %d\n", (u32Cnt+1), m_aMeasureResult[u32Cnt]);   
    }   
#endif 
    
    Bt_Pwc_DisableCount(&USER_BT_PWC); 
    
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("PWC Example Program End \n");
    printf("==================================================\n");
#endif    
    
    while(1);
}

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
/*!
 ******************************************************************************
 ** \brief PWC overflow interrupt handler
 ******************************************************************************
 */
static void PwcOverflowIntHandler(void)
{
    //<<< user code here
    m_u32CntIntOverflow++;
}

/*!
 ******************************************************************************
 ** \brief PWC measure complete interrupt handler
 ******************************************************************************
 */
static void PwcMeasCmpIntHandler(void)
{
    //<<< user code here
    if (m_u32CntIntMeasure < MEASURE_CNT_MAX)
    {  
        if (USER_BT_TIMER_SIZE == PwcSize16Bit) 
        {
            m_aMeasureResult[m_u32CntIntMeasure] = Bt_Pwc_Get16BitMeasureData(&USER_BT_PWC) +
                (m_u32CntIntOverflow * 0xffff);
        }
        else //BT_32IT_TIMER
        {
            m_aMeasureResult[m_u32CntIntMeasure] = Bt_Pwc_Get32BitMeasureData(&USER_BT_PWC);
        }             
        m_u32CntIntMeasure++;
    } 
    else
    {
        if (USER_BT_TIMER_SIZE == PwcSize16Bit) 
        {
            Bt_Pwc_Get16BitMeasureData(&USER_BT_PWC);
        }
        else
        {
            Bt_Pwc_Get32BitMeasureData(&USER_BT_PWC);
        }
    }
    m_u32CntIntOverflow = 0;
}


/*****************************************************************************/
/* END OF FILE */
