/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** Main Module
 **
 ** \brief This example shows how to use the DMA for software block transfer
 **
 ** In this example the DMA channel 0 is used to transfer the contents of
 ** an array to antother array using software block transfer. After the
 ** transfer a callback function is executed, which sets a DMA finished flag.
 ** The transfer is checked in main().
 **
 ** History:
 **   - 2014-02-21  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

#define DMA_MAX_COUNT 256u

boolean_t bDmaFinished = FALSE;					// DMA finished notification flag
uint32_t  au32SourceData[DMA_MAX_COUNT];		// Source data (filled in main() )
uint32_t  au32DestinationData[DMA_MAX_COUNT];

/**
 ******************************************************************************
 ** \brief  DMA Error callback function
 ******************************************************************************/
void Main_dma_error_callback(uint8_t u8ErrorCode)
{
  // Error handling ... Should never happen.
}

/**
 ******************************************************************************
 ** \brief  DMNA finish callback function
 ******************************************************************************/
void Main_dma_finish_callback(void)
{
  bDmaFinished = TRUE;                  	// Set DMA finished notification flag
}

/**
 ******************************************************************************
 ** \brief  ADC0 initialization and single conversion start
 ******************************************************************************/
void Main_dma(void)
{
  stc_dma_config_t stcConfigDma;
  stc_dma_int_sel_t stcIntSel;
  stc_dma_int_cb_t  stcIntCb;

  // Clear local configuration structure to zero.
  PDL_ZERO_STRUCT(stcConfigDma);	
  PDL_ZERO_STRUCT(stcIntSel);	
  PDL_ZERO_STRUCT(stcIntCb);	
   
  // DMMAC0 configuration
  stcConfigDma.enDmaIdrq = Software;
  stcConfigDma.u8BlockCount = 1u;
  stcConfigDma.u16TransferCount = DMA_MAX_COUNT ;   
  stcConfigDma.enTransferMode = DmaBlockTransfer;
  stcConfigDma.enTransferWdith = Dma32Bit;
  stcConfigDma.u32SourceAddress = (uint32_t) &au32SourceData[0u];			// Source array's address
  stcConfigDma.u32DestinationAddress = (uint32_t) &au32DestinationData[0u];	// Destination array's address
  stcConfigDma.bFixedSource = FALSE;
  stcConfigDma.bFixedDestination = FALSE;
  stcConfigDma.bReloadCount = FALSE;     
  stcConfigDma.bReloadSource = FALSE;
  stcConfigDma.bReloadDestination = FALSE;
  stcConfigDma.bEnableBitMask = FALSE;
  
  if (Ok == Dma_Init_Channel(0u, &stcConfigDma))      // Initialize DMA channel 0
  {
    /* Configure interrupt */
    stcIntSel.bCompleteInt = 1;
    stcIntSel.bErrorInt = 1;
    stcIntCb.pfnCompleteCallback = Main_dma_finish_callback;
    stcIntCb.pfnErrorCallback = Main_dma_error_callback; 
    Dma_EnableInt(0u, &stcIntSel, &stcIntCb);
    
    Dma_Enable();            // Overall enable of DMA
  
    Dma_SetChannel(0u, TRUE, FALSE, TRUE);	      // Enable channel and software trigger
  }
}

/**
 ******************************************************************************
 ** \brief  Main function
 **
 ** \return int32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
  uint32_t  u8Count;
  boolean_t bError = FALSE;
  
  // Fill Source data with "random" data
  for (u8Count = 0; u8Count < DMA_MAX_COUNT; u8Count++)
  {
    au32SourceData[u8Count] = (uint32_t)u8Count ^ 0x12345678u;    
  }
  
  Main_dma();                   // Setup DMA and start transfer
  
  while(FALSE == bDmaFinished);  // Wait for DMA finished notification flag
  
  Dma_Disable();				// Disable DMA
  
  // Compare data
  for (u8Count = 0; u8Count < DMA_MAX_COUNT; u8Count++)
  {
    if (au32SourceData[u8Count] != au32DestinationData[u8Count])
    {
      bError = TRUE;
      break;
    }
  }
  
  if (TRUE == bError)			// Should never happen ...
  {
    while(1)
    {}
  }
  
  while(1)
  {}
  
}
