/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** Main Module
 ** 
 ** \brief This example shows how to use Fast GPIO macros
 **
 ** This example enables Fast IO output mode of P10 and P3E and initializes P10 
 ** and P3E to output with the default value '0' and P0F to input with no 
 ** pull-up resistor connected. 
 ** Afterwards the P10 and P3E output are set to '1', and set to '0' at the 
 ** following.
 ** Then the status of P0F is read out.
 ** The same is done with P10 and P3E but with inverted logic at the following.
 ** Also the same is done with P0F with inverted logic at the following.  
 ** At last, the Fast IO mode of Port 0 and 3 are disabled.
 **
 ** History:
 **   - 2014-01-15  1.0  Edison Zhang      First version.
 **   - 2014-02-26  1.1  Edison Zhang      Update access mothed of Fast IO R/W
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

boolean_t bReadData; 

/**
 ******************************************************************************
 ** \brief  Sample program of Fast GPIO
 **
 ** \return int32_t return value, if needed
 ** \note The optimization need to be enabled to achieve high IO polling speed
 ******************************************************************************/
int32_t main(void)
{
    // enable Fast GPIO output mode
    //    F E D C B A 9 8 7 6 5 4 3 2 1 0
    // b' 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1
    //                                  |-> P10
    FGpio_EnableOutput(FGpioPort1, 0x0001u); 
    //    F E D C B A 9 8 7 6 5 4 3 2 1 0
    // b' 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0
    //      |-> P3E
    FGpio_EnableOutput(FGpioPort3, 0x4000u);
    
    FGpio1pin_InitOut( FGPIO1PIN_P10, FGpio1pin_InitVal( 0x00u ) );  // intial Fast GPIO as output pin
    FGpio1pin_InitOut( FGPIO1PIN_P3E, FGpio1pin_InitVal( 0x00u ) );  // intial Fast GPIO as output pin
    FGpio1pin_InitIn( FGPIO1PIN_P0F, FGpio1pin_InitPullup(0) );  // intial Fast GPIO as input pin
   
    FGpio1pin_Put(FGPIO1PIN_P10, 0xFFu); // set P10 to high level
    FGpio1pin_Put(FGPIO1PIN_P10, 0x00u); // set P10 to low level
    FGpio1pin_Put(FGPIO1PIN_P3E, 0xFFu); // set P3E to high level
    FGpio1pin_Put(FGPIO1PIN_P3E, 0x00u); // set P3E to low level

    bReadData = FGpio1pin_Get(FGPIO1PIN_P0F);  // read the P0F

    FGpio1pin_Put(FGPIO1PIN_NP10, 0xFFu); // set P10 to high level with invert logic (equal to set to low)
    FGpio1pin_Put(FGPIO1PIN_NP10, 0x00u); // set P10 to low level with invert logic (equal to set to high)
    FGpio1pin_Put(FGPIO1PIN_NP3E, 0xFFu); // set P3E to high level with invert logic (equal to set to low)
    FGpio1pin_Put(FGPIO1PIN_NP3E, 0x00u); // set P3E to low level with invert logic (equal to set to high)
    
    bReadData = FGpio1pin_Get(FGPIO1PIN_NP0F); // read the P0F with invert logic
    // disable Fast GPIO mode for Port 1 and 3
    FGpio_DisableOutput(FGpioPort1); 
    FGpio_DisableOutput(FGpioPort3); 
    
    while(1);
}
