/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates RTC mode of low power consumption mode. Every 
 ** 5 second the MCU is wakeup from RTC mode and polling a LED for 1 second.
 **
 ** History:
 **   - 2014-03-06  0.0.1  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/
uint8_t u8TimerIntFlag = 0;

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/

/**
 ******************************************************************************
 ** \brief Initializatio GPIO
 **
 ******************************************************************************/
static void PortInit(void)
{
   Gpio1pin_InitOut(GPIO1PIN_P61, Gpio1pin_InitVal(1u));
}

/**
 ******************************************************************************
 ** \brief Set port for LED
 **
 ******************************************************************************/
static void SetLed(boolean_t bLed)
{
    if (0 == bLed)
    {
        Gpio1pin_Put(GPIO1PIN_P61, 0);
    }
    else
    {
        Gpio1pin_Put(GPIO1PIN_P61, 1);
    }
}

/**
 ******************************************************************************
 ** \brief One Second Interrupt
 **
 ******************************************************************************/
static void SampleRtcTimerCb(void)
{
    // Set timer interruption flag
    u8TimerIntFlag = TRUE;
}

/**
 ******************************************************************************
 ** \brief Configure and start RTC
 ******************************************************************************/
static void Main_Rtc_Config(void)
{
    stc_rtc_config_t stcRtcConfig;
    stc_rtc_int_sel_t stcInt;
    stc_rtc_int_cb_t  stcIntCb;
    
    // Clear structures
    PDL_ZERO_STRUCT(stcRtcConfig);
    PDL_ZERO_STRUCT(stcInt);
    PDL_ZERO_STRUCT(stcIntCb);
    
    // Clear seconds interrupt flag
    u8TimerIntFlag = FALSE;
    
    stcRtcConfig.enClkSel = RtcUseSubClk;
    stcRtcConfig.bEnSuboutDivider = FALSE;
    stcRtcConfig.bInitFreqCorr = FALSE;
    stcRtcConfig.enRtccoSel = RtccoOutput1Hz;
    
    // Time setting (23:59:00 1st of January 2014)
    stcRtcConfig.bInitTimeDate = TRUE;
    stcRtcConfig.stcTimeDate.u8Second = 0;                    // Second      : 00
    stcRtcConfig.stcTimeDate.u8Minute = 59;                   // Minutes     : 59
    stcRtcConfig.stcTimeDate.u8Hour   = 23;                   // Hour        : 23
    stcRtcConfig.stcTimeDate.u8Day    = 31;                   // Date        : 31th
    stcRtcConfig.stcTimeDate.u8Month  = RtcJanuary;           // Month       : January
    stcRtcConfig.stcTimeDate.u16Year   = 2014;                // Year        : 2014
    (void)Rtc_SetDayOfWeek(&stcRtcConfig.stcTimeDate);        // Set Day of the Week in stcRtcTime

    // Alarm setting 
    stcRtcConfig.bInitAlarm = FALSE;

    // Intialize RTC timer
    stcRtcConfig.bInitTimer = TRUE;
    stcRtcConfig.stcTimer.enMode = RtcTimerPeriod;
    stcRtcConfig.stcTimer.u32TimerCycle = 5; // Generate interrupt every 5s
    
    // Initialize the RTC
    if(Ok == Rtc_Init(&stcRtcConfig))
    {
        // Configure interrupt 
        stcInt.TimerInt = TRUE;
        stcIntCb.pfnTimerCallback      = SampleRtcTimerCb;
        Rtc_EnableInt(&stcInt, &stcIntCb);
        // Start RTC counting 
        Rtc_EnableFunc(RtcCount);
        // Enable timer 
        Rtc_EnableFunc(RtcTimer);
    }
}

/**
 ******************************************************************************
 ** \brief  Main function of project for MB9BF56X series (FM0P).
 **
 ** \return int32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
    PortInit();
    /* Turn off LED */
    SetLed(1);
    /* Configure and start RTC */
    Main_Rtc_Config();
          
    while(1)
    {
        /* Enter RTC mode */
        Lpm_GoToStandByMode(StbRtcMode, TRUE);
        if(u8TimerIntFlag == TRUE) // wakeup here!, RTC interrupt flag is set in the callback function
        {
            u8TimerIntFlag = FALSE;

            /* Turn on LED */
            SetLed(0); 
            Rtc_ClrStatus(RtcHalfSecondFlag);  
            while(Rtc_GetStatus(RtcHalfSecondFlag) != TRUE); //  Wait 0.5s
              
            /* Turn off LED */
            SetLed(1);  
            Rtc_ClrStatus(RtcHalfSecondFlag); 
            while(Rtc_GetStatus(RtcHalfSecondFlag) != TRUE); //  Wait 0.5s
        }
    }
}

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
