/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates the operations to Spansion SPI Flash (S25FL127S)
 ** via CSIO I/F.
 **
 ** History:
 **   - 2014-03-31  1.0  Edison Zhang        First release version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/******************************************************************************/
/* Local variable                                                             */
/******************************************************************************/
static uint8_t u8ManufactureId, u8DeviceId;
static uint8_t au8TxData[S25FL127S_DEFAULT_PAGE_SIZE]; 
static uint8_t au8RxData[S25FL127S_DEFAULT_PAGE_SIZE];

/**
 ******************************************************************************
 ** \brief  Compare each data in the input two buffers
 **
 ** \relval Ok     The data in buffer1 are same with that in buffer2
 ** \relval Error  The data in buffer1 are not same with that in buffer2 
 ******************************************************************************/
static en_result_t CompareData(uint8_t* pBuf1, uint8_t* pBuf2, uint16_t u16Length)
{
    while(u16Length--)
    {
        if(*pBuf1++ != *pBuf2++)
        {
            return Error;
        }
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Main function of PDL
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
    uint16_t u16i;
    uint8_t u8Flag = 0;
  
    S25fl127s_Init();
  
    /* Read Manufacture ID and device ID */
    S25fl127s_ReadId(&u8ManufactureId, &u8DeviceId);
    
    /* Sector erase, page program, read and varify */
    if(Ok != S25fl127s_4kSectorErase(0x000000u))
    {
        while(1);
    }
    
    for(u16i = 0; u16i<S25FL127S_DEFAULT_PAGE_SIZE; u16i++)
    {
        au8TxData[u16i] = u16i;
    }
    
    if(Ok != S25fl127s_PageProgram(0x000000u, au8TxData, S25FL127S_DEFAULT_PAGE_SIZE))
    {
        while(1);
    }
    
    if(Ok != S25fl127s_Read(0x000000u, au8RxData, S25FL127S_DEFAULT_PAGE_SIZE))
    {
        while(1);
    }
    
    if(Ok != CompareData(au8TxData, au8RxData, S25FL127S_DEFAULT_PAGE_SIZE))
    {
        while(1);
    }
    
    /* Sector erase, several bytes program, read and varify */
    if(Ok != S25fl127s_SectorErase(0x020000u))
    {
        while(1);
    }
    
    for(u16i = 0; u16i<S25FL127S_DEFAULT_PAGE_SIZE; u16i++)
    {
        au8TxData[u16i] = u16i^0xFF;
    }
    
    if(Ok != S25fl127s_PageProgram(0x020010u, au8TxData, 10))
    {
        while(1);
    }
    
    if(Ok != S25fl127s_Read(0x020010u, au8RxData, 10))
    {
        while(1);
    }
    
    if(Ok != CompareData(au8TxData, au8RxData, 10))
    {
        while(1);
    }
        
    /* Chip erase */
    if(u8Flag > 0)
    {
        if(Ok != S25fl127s_BulkErase()) /* It may take a long time... */
        {
            while(1);
        }
    }
    
    while(1)
    {}
}
