/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates communication between CSIO0 and CSIO1 with 
 ** interrupt mode using CS pin. 
 ** Make following connection before using this example:
 ** Master             Slave
 ** SOT1_1  -------    SIN3_1
 ** SCK1_1  -------    SCK3_1
 ** SIN1_1  -------    SOT3_1
 ** SCS10_1 -------    SCS30_2
 ** 
 ** History:
 **   - 2014-03-17  1.0  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/
/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/
#define SAMPLE_CSIO_SLAVE_RX_BUFFSIZE   sizeof(au8CsioMasterTxBuf)/sizeof(char)
#define SAMPLE_CSIO_MASTER_TX_BUFFSIZE  SAMPLE_CSIO_SLAVE_RX_BUFFSIZE

#define SAMPLE_CSIO_MASTER_RX_BUFFSIZE  sizeof(au8CsioSlaveTxBuf)/sizeof(char)
#define SAMPLE_CSIO_SLAVE_TX_BUFFSIZE   SAMPLE_CSIO_MASTER_RX_BUFFSIZE

#define InitCsio1Io(void)  {SetPinFunc_SIN1_1();SetPinFunc_SOT1_1();SetPinFunc_SCK1_1();SetPinFunc_SCS10_1();}
#define InitCsio3Io(void)  {SetPinFunc_SIN3_1();SetPinFunc_SOT3_1();SetPinFunc_SCK3_1();Gpio1pin_InitIn(GPIO1PIN_P3C, Gpio1pin_InitPullup(1u));SetPinFunc_SCS30_2();}   

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/
/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/
volatile stc_mfsn_csio_t* CsioCh1 = &CSIO1;
volatile stc_mfsn_csio_t* CsioCh3 = &CSIO3;
stc_csio_int_sel_t stcCsio1IntSel, stcCsio3IntSel;
stc_csio_int_cb_t stcCsio1IntCb, stcCsio3IntCb;
uint32_t SendCnt = 0, ReceiveCnt = 0;
static uint8_t au8CsioMasterTxBuf[8] = "01234567";
static uint8_t au8CsioSlaveTxBuf[8] = "abcdefgh";
static uint8_t au8CsioMasterRxBuf[SAMPLE_CSIO_MASTER_RX_BUFFSIZE];
static uint8_t au8CsioSlaveRxBuf[SAMPLE_CSIO_SLAVE_RX_BUFFSIZE];

/**
 ******************************************************************************
 ** \brief  Compare each data in the input two buffers
 **
 ** \relval Ok     The data in buffer1 are same with that in buffer2
 ** \relval Error  The data in buffer1 are not same with that in buffer2 
 ******************************************************************************/
static en_result_t CompareData(uint8_t* pBuf1, uint8_t* pBuf2, uint8_t u8Length)
{
    while(u8Length--)
    {
        if(*pBuf1++ != *pBuf2++)
        {
            return Error;
        }
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  CSIO master transfer interrupt callback function
 ******************************************************************************/
static void CsioMasterTxIntCallback(void)
{
    if(SendCnt == SAMPLE_CSIO_MASTER_TX_BUFFSIZE)
    {
        /* Disable interrupt */
        stcCsio1IntSel.bTxInt = TRUE;
        Mfs_Csio_DisableInt(CsioCh1, &stcCsio1IntSel);
        return;
    }
  
    Mfs_Csio_SendData(CsioCh1, au8CsioMasterTxBuf[SendCnt], TRUE);  
    SendCnt++;

}

/**
 ******************************************************************************
 ** \brief  CSIO slave receive interrupt callback function
 ******************************************************************************/
static void CsioSlaveRxIntCallback(void)
{
    au8CsioSlaveRxBuf[ReceiveCnt] = Mfs_Csio_ReceiveData(CsioCh3);
    ReceiveCnt++;
    
    if(ReceiveCnt == SAMPLE_CSIO_SLAVE_RX_BUFFSIZE)
    {
        /* Disable interrupt */
        stcCsio3IntSel.bRxInt = TRUE;
        Mfs_Csio_DisableInt(CsioCh3, &stcCsio3IntSel);
    }
}

/**
 ******************************************************************************
 ** \brief  CSIO Master receive interrupt callback function
 ******************************************************************************/
static void CsioMasterRxIntCallback(void)
{
    au8CsioMasterRxBuf[ReceiveCnt] = Mfs_Csio_ReceiveData(CsioCh1);
    ReceiveCnt++;
    
    if(ReceiveCnt == SAMPLE_CSIO_MASTER_RX_BUFFSIZE)
    {
        /* Disable interrupt */
        stcCsio1IntSel.bRxInt = TRUE;
        Mfs_Csio_DisableInt(CsioCh1, &stcCsio1IntSel);
        return;
    }
}

/**
 ******************************************************************************
 ** \brief  CSIO slave transfer interrupt callback function
 ******************************************************************************/
static void CsioSlaveTxIntCallback(void)
{
    if(SendCnt == SAMPLE_CSIO_SLAVE_TX_BUFFSIZE)
    {
        /* Disable interrupt */
        stcCsio3IntSel.bTxInt = TRUE;
        Mfs_Csio_DisableInt(CsioCh3, &stcCsio3IntSel);
        return;
    }
  
    Mfs_Csio_SendData(CsioCh3, au8CsioSlaveTxBuf[SendCnt], TRUE);  
    SendCnt++;
}

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{   
    stc_mfs_csio_config_t stcCsio1Config, stcCsio3Config;
    
    /* Clear configuration structure */
    PDL_ZERO_STRUCT(stcCsio1Config);
    PDL_ZERO_STRUCT(stcCsio3Config);
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    PDL_ZERO_STRUCT(stcCsio3IntSel);
    PDL_ZERO_STRUCT(stcCsio1IntCb);
    PDL_ZERO_STRUCT(stcCsio3IntCb);
    
    /* Initialize CSIO function I/O */    
    InitCsio1Io();
    InitCsio3Io();
    
    SendCnt = 0;
    ReceiveCnt = 0;
    
    /* Initialize CSIO master  */
    stcCsio1Config.enMsMode = CsioMaster;
    stcCsio1Config.enActMode = CsioActNormalMode;
    stcCsio1Config.bInvertClk = FALSE;
    stcCsio1Config.u32BaudRate = 100000;
    stcCsio1Config.enDataLength = CsioEightBits;
    stcCsio1Config.enBitDirection = CsioDataMsbFirst;
    stcCsio1Config.enSyncWaitTime = CsioSyncWaitZero;
    stcCsio1Config.bEnableFifo = FALSE;
    
    stcCsio1Config.bEnChipSelection = TRUE;
    stcCsio1Config.stcCsConfig.enCsStartPin = CsPinScs0;
    stcCsio1Config.stcCsConfig.enCsEndPin = CsPinScs0; 
    stcCsio1Config.stcCsConfig.enClkDiv = CsClkNoDiv;
    stcCsio1Config.stcCsConfig.bActiveHold = FALSE;
    stcCsio1Config.stcCsConfig.enLevel = CsLowActive;
    stcCsio1Config.stcCsConfig.u8CsSetupDelayTime = 200u;
    stcCsio1Config.stcCsConfig.u8CsHoldDelayTime = 200u;
    stcCsio1Config.stcCsConfig.u16CsDeselectTime = 2000u;
    stcCsio1Config.stcCsConfig.u8Scs0TransferByteCnt = SAMPLE_CSIO_MASTER_TX_BUFFSIZE;
    stcCsio1Config.stcCsConfig.u8Scs1TransferByteCnt = 0;
    stcCsio1Config.stcCsConfig.u8Scs2TransferByteCnt = 0;
    stcCsio1Config.stcCsConfig.u8Scs3TransferByteCnt = 0;
    stcCsio1Config.stcCsConfig.bScs0En = TRUE;
    stcCsio1Config.stcCsConfig.bScs1En = FALSE;
    stcCsio1Config.stcCsConfig.bScs2En = FALSE;
    stcCsio1Config.stcCsConfig.bScs3En = FALSE;
    
    stcCsio1Config.bEnSyncTransfer = FALSE;
    
    Mfs_Csio_Init(CsioCh1, &stcCsio1Config);
    
    /* Initialize CSIO slave  */
    stcCsio3Config.enMsMode = CsioSlave;
    stcCsio3Config.enActMode = CsioActNormalMode;
    stcCsio3Config.bInvertClk = FALSE;
    stcCsio3Config.u32BaudRate = 100000;
    stcCsio3Config.enDataLength = CsioEightBits;
    stcCsio3Config.enBitDirection = CsioDataMsbFirst;
    stcCsio3Config.bEnableFifo = FALSE;
    
    stcCsio3Config.bEnChipSelection = TRUE;
    stcCsio3Config.stcCsConfig.bScs0En = TRUE; // Only the SCS0 can be used in slave mode
    stcCsio3Config.stcCsConfig.enLevel = CsLowActive;
    
    stcCsio3Config.bEnSyncTransfer = FALSE;
    
    Mfs_Csio_Init(CsioCh3, &stcCsio3Config);
    
    /*************************************************************************/
    /*                Master sends data to slave                             */
    /*************************************************************************/
    /* Enable TX function of CSIO1   */
    Mfs_Csio_EnableFunc(CsioCh1, CsioTx);
    
    /* Enable RX function of CSIO3   */
    Mfs_Csio_EnableFunc(CsioCh3, CsioRx);
    
    /* Configure interrupt */
    stcCsio3IntSel.bRxInt = TRUE;
    stcCsio3IntCb.pfnRxIntCb = CsioSlaveRxIntCallback;
    Mfs_Csio_EnableInt(CsioCh3, &stcCsio3IntSel, &stcCsio3IntCb);
    
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
    Mfs_Csio_EnableInt(CsioCh1, &stcCsio1IntSel, &stcCsio1IntCb);
    
    /* Wait until send finish */
    while(SendCnt < SAMPLE_CSIO_MASTER_TX_BUFFSIZE)
    {
        ;
    }
    
    while(TRUE != Mfs_Csio_GetStatus(CsioCh1, CsioTxIdle)); // wait until TX bus idle
    
    /* Wait until receive finish */
    while(ReceiveCnt < SAMPLE_CSIO_SLAVE_RX_BUFFSIZE)
    {
        ;
    }
    
    /* Compare receive data with transfer data */
    if(Error == CompareData(au8CsioMasterTxBuf, au8CsioSlaveRxBuf, SAMPLE_CSIO_SLAVE_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }
    
    /* Disable TX function of CSIO0   */
    Mfs_Csio_DisableFunc(CsioCh1, CsioTx);
    /* Disable RX function of CSIO1   */
    Mfs_Csio_DisableFunc(CsioCh3, CsioRx);
    /* Stop Sync Timer   */
    Mfs_Csio_DisableFunc(CsioCh1, CsioSerialTimer);
    
    /*************************************************************************/
    /*                Master receives data from slave                        */
    /*************************************************************************/
    /* Enable TX function of CSIO1   */
    Mfs_Csio_EnableFunc(CsioCh3, CsioTx);
    
    /* Enable TX and RX function of CSIO0   */
    Mfs_Csio_EnableFunc(CsioCh1, CsioTx);
    Mfs_Csio_EnableFunc(CsioCh1, CsioRx);
   
    /* Configure interrupt */
    SendCnt = 0;
    ReceiveCnt = 0;
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    PDL_ZERO_STRUCT(stcCsio3IntSel);
    PDL_ZERO_STRUCT(stcCsio1IntCb);
    PDL_ZERO_STRUCT(stcCsio3IntCb);
    
    stcCsio1IntSel.bRxInt = TRUE;
    stcCsio1IntCb.pfnRxIntCb = CsioMasterRxIntCallback;
    Mfs_Csio_EnableInt(CsioCh1, &stcCsio1IntSel, &stcCsio1IntCb);
    
    stcCsio3IntSel.bTxInt = TRUE;
    stcCsio3IntCb.pfnTxIntCb = CsioSlaveTxIntCallback;
    Mfs_Csio_EnableInt(CsioCh3, &stcCsio3IntSel, &stcCsio3IntCb);
     
    /* Master dummy write */
    while(SendCnt < SAMPLE_CSIO_SLAVE_TX_BUFFSIZE)
    {
        while(TRUE != Mfs_Csio_GetStatus(CsioCh1, CsioTxEmpty));
        Mfs_Csio_SendData(CsioCh1, 0x00u, FALSE);   /* Dummy write */    
    }
    
    while(TRUE != Mfs_Csio_GetStatus(CsioCh1, CsioTxIdle)); // wait until TX bus idle
    
    /* Wait receive finish */
    while(ReceiveCnt < SAMPLE_CSIO_MASTER_RX_BUFFSIZE)
    {
        ;
    }
    
    /* Compare receive data with transfer data */
    if(Error == CompareData(au8CsioMasterRxBuf, au8CsioSlaveTxBuf, SAMPLE_CSIO_MASTER_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }
    
    /* Disable TX function of CSIO3   */
    Mfs_Csio_DisableFunc(CsioCh3, CsioTx);
    
    /* Disable TX and RX function of CSIO1   */
    Mfs_Csio_DisableFunc(CsioCh1, CsioTx);
    Mfs_Csio_DisableFunc(CsioCh1, CsioRx);
    
    Mfs_Csio_DeInit(CsioCh1);
    Mfs_Csio_DeInit(CsioCh3);
    
    while(1); /* Data is normally sent and received */
}


/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
