/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates I2C data transfer and receive in slave mode. 
 ** This example must be used with the example "i2c_master_polling".
 ** Make following connection before using this example:
 ** Master (1st board "i2c_master_polling")   Slave (2nd board with "i2c_slave_polling")
 ** SCK0_0   ---------------------------     SCK1_1
 ** SOT0_0   ---------------------------     SOT1_1
 ** SCK1_1 and SOT1_1 must be pull-up in the slave node.
 **
 ** History:
 **   - 2014-02-18  1.0  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"
#include "string.h"

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/
/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/
#define SAMPLE_I2C_MASTER_TX_BUFFSIZE   sizeof(au8I2cSlaveTxBuf)/sizeof(char)
#define SAMPLE_I2C_MASTER_RX_BUFFSIZE   sizeof(au8O2cSlaveRxBuf)/sizeof(char)

#define I2C_RET_OK                  0
#define I2C_RET_ERROR               1

#define I2C_DEV_ADDR              (0x3Au)
#define I2C_DEV_ADDR_W            (0x3Au | 0u)
#define I2C_DEV_ADDR_R            (0x3Au | 1u)

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/
/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/
volatile stc_mfsn_i2c_t* I2cCh1 = &I2C1;
static uint8_t au8I2cSlaveTxBuf[50];
static uint8_t au8I2cSlaveRxBuf[50];

/*!
 ******************************************************************************
 ** \brief Dummy delay
 ** \param u32Cnt Time count
 ******************************************************************************
 */
void DelayTime(uint32_t u32Cnt)
{
   while(u32Cnt--) ;
}


/*!
 ******************************************************************************
 ** \brief Initialize I2C slave
 ******************************************************************************
 */
void InitI2cSlave(void)
{
    stc_mfs_i2c_config_t stcI2c1Config;
    
    PDL_ZERO_STRUCT(stcI2c1Config);
        
    SetPinFunc_SOT1_1();
    SetPinFunc_SCK1_1();
    
    stcI2c1Config.enMsMode = I2cSlave;
    stcI2c1Config.u32BaudRate = 100000u;
    stcI2c1Config.u8SlaveAddr = I2C_DEV_ADDR;
    stcI2c1Config.u8SlaveMaskAddr = 0x00u;
    stcI2c1Config.bWaitSelection = FALSE;
    stcI2c1Config.bDmaEnable = FALSE;
    stcI2c1Config.bEnableFifo = FALSE;

    Mfs_I2c_Init(I2cCh1, &stcI2c1Config);

}

/*!
 ******************************************************************************
 ** \brief  Transmit the dedicated size data to device
 **
 ** \param  pu8TxData Transmit data pointer
 **
 ** \param  pu32Size the number of transmitted data
 **
 ** \return none
 ******************************************************************************
 */
void I2c_SlaveWriteData(uint8_t *pu8TxData, uint32_t *pu32Size)
{
    uint8_t i=0;
    while(1)
    {
        /* Transmit the data */
        Mfs_I2c_SendData(I2cCh1, pu8TxData[i++]);
        Mfs_I2c_ClrStatus(I2cCh1, I2cRxTxInt);
        
        /* Wait for end of transmission */
        while(1)
        {
            if(Mfs_I2c_GetStatus(I2cCh1, I2cRxTxInt) == TRUE)
            {
                break;
            }
        }
        
        while(1)
        {    
            if(Mfs_I2c_GetStatus(I2cCh1, I2cTxEmpty) == TRUE)
            {
                break;
            }
        }
        
        if(Mfs_I2c_GetAck(I2cCh1) == I2cAck)
        {
            ; /* continue to send */
        }
        else
        {
            /* Master tells that it is last data he wants to read, check the stop signal then */
            Mfs_I2c_ClrStatus(I2cCh1, I2cRxTxInt); /* Release bus  */
            break;
        }
    }
    
    
    /* Check the stop condition */
    while(1)
    {
        if(Mfs_I2c_GetStatus(I2cCh1, I2cStopDetect) == TRUE)
        {            
            Mfs_I2c_ClrStatus(I2cCh1, I2cStopDetect);
            Mfs_I2c_ClrStatus(I2cCh1, I2cRxTxInt);
            break;
        }
    }
    
    *pu32Size = i;
    
}
/*!
 ******************************************************************************
 ** \brief  Receive the dedicated size data to device
 **
 ** \param  pu8RxData The receive buffer pointer
 **
 ** \param  pu32Sizethe number of receiving data
 **
 ** \return none
 ******************************************************************************
 */
void I2c_SlaveReadData(uint8_t *pu8RxData, uint32_t *pu32Size)
{
    uint8_t i = 0;
    while(1)
    {
        /*Stop condition and NACK*/
        if(Mfs_I2c_GetStatus(I2cCh1, I2cStopDetect) == TRUE)
        {            
            Mfs_I2c_ClrStatus(I2cCh1, I2cStopDetect);
            Mfs_I2c_ClrStatus(I2cCh1, I2cRxTxInt);
            
            break;
        }
        
        /*Bus Error*/
        if(Mfs_I2c_GetStatus(I2cCh1, I2cBusErr) == TRUE)
        {
            Mfs_I2c_ClrStatus(I2cCh1, I2cRxTxInt);
            break;
        }
        
        Mfs_I2c_ClrStatus(I2cCh1, I2cRxTxInt);
                
        if(Mfs_I2c_GetStatus(I2cCh1, I2cRxFull) == TRUE)
        {
            pu8RxData[i++] = Mfs_I2c_ReceiveData(I2cCh1);
            Mfs_I2c_ConfigAck(I2cCh1, I2cAck);
            Mfs_I2c_ClrStatus(I2cCh1, I2cRxTxInt);
            
        }
    }
    
    *pu32Size = i;
}

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{   
    uint32_t WriteLength, ReadLength;
    InitI2cSlave();
    
    while(1)
    {
        /* polling flag of first data*/
        if(TRUE == Mfs_I2c_GetStatus(I2cCh1, I2cDevAddrMatch))
        {
            if(i2c_slave_tx_master_rx == Mfs_I2c_GetDataDir(I2cCh1))//Tx
            {
                I2c_SlaveWriteData(au8I2cSlaveTxBuf, &WriteLength);
                DelayTime(100);
            }
            else//Rx
            {
                I2c_SlaveReadData(au8I2cSlaveRxBuf,&ReadLength);
                memset(au8I2cSlaveTxBuf, 0, sizeof(au8I2cSlaveTxBuf));
                memcpy(au8I2cSlaveTxBuf, au8I2cSlaveRxBuf, ReadLength);
            }
        }
    }
}


/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
