/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates communication between UART0 and UART1 with
 ** interrupt mode using DMA.
 ** The SOT0_0 should be connected with SIN1_1 before using this example.
 **
 ** History:
 **   - 2014-03-10  1.0  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/
/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/
#define SAMPLE_UART_RX_BUFFSIZE     (8u)
#define SAMPLE_UART_TX_BUFFSIZE     (8u)

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/
/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/
volatile stc_mfsn_uart_t* UartCh0 = &UART0;
volatile stc_mfsn_uart_t* UartCh1 = &UART1;
stc_uart_int_sel_t stcUart0IntSel, stcUart1IntSel;
stc_uart_int_cb_t  stcUart0IntCb, stcUart1IntCb;
static uint8_t au8UartTxBuf[SAMPLE_UART_TX_BUFFSIZE];
static uint8_t au8UartRxBuf[SAMPLE_UART_RX_BUFFSIZE];
static boolean_t bDma0Finished, bDma1Finished;
/**
 ******************************************************************************
 ** \brief  Compare each data in the input two buffers
 **
 ** \relval Ok     The data in buffer1 are same with that in buffer2
 ** \relval Error  The data in buffer1 are not same with that in buffer2
 ******************************************************************************/
static en_result_t CompareData(uint8_t* pBuf1, uint8_t* pBuf2, uint8_t u8Length)
{
    while(u8Length--)
    {
        if(*pBuf1++ != *pBuf2++)
        {
            return Error;
        }
    }

    return Ok;
}

/**
 ******************************************************************************
 ** \brief  UART0 TX interrupt callback function
 ******************************************************************************/
void Uart0TxIntCallback(void)
{
    // Should never enter
}

/**
 ******************************************************************************
 ** \brief  UART0 RX interrupt callback function
 ******************************************************************************/
void Uart1RxIntCallback(void)
{
   // Should never enter
}

/**
 ******************************************************************************
 ** \brief  DMA Error callback function
 ******************************************************************************/
void Main_dma0_error_callback(uint8_t u8ErrorCode)
{
    // Error handling ... Should never happen.
}

/**
 ******************************************************************************
 ** \brief  DMNA finish callback function
 ******************************************************************************/
void Main_dma0_finish_callback(void)
{
    stcUart0IntSel.bTxInt = TRUE;
    Mfs_Uart_DisableInt(UartCh0, &stcUart0IntSel);

    bDma0Finished = TRUE;         // Set DMA finished notification flag
}

/**
 ******************************************************************************
 ** \brief  DMA Error callback function
 ******************************************************************************/
void Main_dma1_error_callback(uint8_t u8ErrorCode)
{
    // Error handling ... Should never happen.
}

/**
 ******************************************************************************
 ** \brief  DMNA finish callback function
 ******************************************************************************/
void Main_dma1_finish_callback(void)
{
    stcUart1IntSel.bRxInt = TRUE;
    Mfs_Uart_DisableInt(UartCh1, &stcUart1IntSel);

    bDma1Finished = TRUE;         // Set DMA finished notification flag
}


/**
 ******************************************************************************
 ** \brief  DMA configuration
 ******************************************************************************/
void Main_Dma_Init(void)
{
    stc_dma_config_t stcDma0Config, stcDma1Config;
    stc_dma_int_sel_t stcDma0IntSel, stcDma1IntSel;
    stc_dma_int_cb_t  stcDma0IntCb, stcDma1IntCb;

    // Clear local configuration structure to zero.
    PDL_ZERO_STRUCT(stcDma0Config);
    PDL_ZERO_STRUCT(stcDma0IntSel);
    PDL_ZERO_STRUCT(stcDma0IntCb);
    PDL_ZERO_STRUCT(stcDma1Config);
    PDL_ZERO_STRUCT(stcDma1IntSel);
    PDL_ZERO_STRUCT(stcDma1IntCb);

    // DMAC0 configuration
    stcDma0Config.enDmaIdrq = MfsTx0;
    stcDma0Config.u8BlockCount = 1u;
    stcDma0Config.u16TransferCount = SAMPLE_UART_TX_BUFFSIZE ;
    stcDma0Config.enTransferMode = DmaDemandTransfer;
    stcDma0Config.enTransferWdith = Dma8Bit;
    stcDma0Config.u32SourceAddress = (uint32_t)&(au8UartTxBuf[0u]); // UART data address
    stcDma0Config.u32DestinationAddress = MFS0_DATA_REG_ADDR;	   // Destination array's address
    stcDma0Config.bFixedSource = FALSE;
    stcDma0Config.bFixedDestination = TRUE;
    stcDma0Config.bReloadCount = TRUE;       // Reload count for next data package
    stcDma0Config.bReloadSource = TRUE;      // Reload source address for next data package
    stcDma0Config.bReloadDestination = FALSE;
    stcDma0Config.bEnableBitMask = TRUE;    // Don't clear enable bit after transfer completion

    if (Ok == Dma_Init_Channel(0u, &stcDma0Config))      // Initialize DMA channel 0
    {
        /* Configure interrupt */
        stcDma0IntSel.bCompleteInt = 1;
        stcDma0IntSel.bErrorInt = 1;
        stcDma0IntCb.pfnCompleteCallback = Main_dma0_finish_callback;
        stcDma0IntCb.pfnErrorCallback = Main_dma0_error_callback;
        Dma_EnableInt(0u, &stcDma0IntSel, &stcDma0IntCb);

         Dma_SetChannel(0u, TRUE, FALSE, FALSE);	      // Enable channel
    }

    // DMAC1 configuration
    stcDma1Config.enDmaIdrq = MfsRx1;
    stcDma1Config.u8BlockCount = 1u;
    stcDma1Config.u16TransferCount = SAMPLE_UART_RX_BUFFSIZE ;
    stcDma1Config.enTransferMode = DmaDemandTransfer;
    stcDma1Config.enTransferWdith = Dma8Bit;
    stcDma1Config.u32SourceAddress = MFS1_DATA_REG_ADDR ; // UART data address
    stcDma1Config.u32DestinationAddress = (uint32_t)&(au8UartRxBuf[0u]);	 // Destination array's address
    stcDma1Config.bFixedSource = TRUE;
    stcDma1Config.bFixedDestination = FALSE;
    stcDma1Config.bReloadCount = TRUE;       // Reload count for next data package
    stcDma1Config.bReloadSource = FALSE;
    stcDma1Config.bReloadDestination = TRUE; // Reload destination for next data package
    stcDma1Config.bEnableBitMask = TRUE;     // Don't clear enable bit after transfer completion

    if (Ok == Dma_Init_Channel(1u, &stcDma1Config))      // Initialize DMA channel 1
    {
        /* Configure interrupt */
        stcDma1IntSel.bCompleteInt = 1;
        stcDma1IntSel.bErrorInt = 1;
        stcDma1IntCb.pfnCompleteCallback = Main_dma1_finish_callback;
        stcDma1IntCb.pfnErrorCallback = Main_dma1_error_callback;
        Dma_EnableInt(1u, &stcDma1IntSel, &stcDma1IntCb);

        Dma_Enable();            // Overall enable of DMA

        Dma_SetChannel(1u, TRUE, FALSE, FALSE);	      // Enable channel
    }

    Dma_Enable();            // Overall enable of DMA

}

/**
 ******************************************************************************
 ** \brief  Main function of project for MB9BF56X series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
    stc_mfs_uart_config_t stcUartConfig;
    uint8_t u8i;

    /* Initialize UART function I/O */
    SetPinFunc_SIN0_0();
    SetPinFunc_SOT0_0();
    SetPinFunc_SIN1_1();
    SetPinFunc_SOT1_1();

    /* Initialize UART TX and RX channel  */
    stcUartConfig.enMode = UartNormal;
    stcUartConfig.u32BautRate = 115200;
    stcUartConfig.enDataLength = UartEightBits;
    stcUartConfig.enParity = UartParityNone;
    stcUartConfig.enStopBit = UartOneStopBit;
    stcUartConfig.enBitDirection = UartDataLsbFirst;
    stcUartConfig.bInvertData = FALSE;
    stcUartConfig.bHwFlow = FALSE;
    stcUartConfig.bEnableFifo = FALSE;

    Mfs_Uart_Init(UartCh0, &stcUartConfig);
    Mfs_Uart_Init(UartCh1, &stcUartConfig);

    // Initialize and enable DMA
    Main_Dma_Init();

    /* Enable RX function of UART0   */
    Mfs_Uart_EnableFunc(UartCh1, UartRx);
    /* Enable TX function of UART0   */
    Mfs_Uart_EnableFunc(UartCh0, UartTx);

/******************************************************/
/* Test 1st UART transfer/receive data with DMA       */
/******************************************************/
    // Initialize send data
    for(u8i=0; u8i<SAMPLE_UART_TX_BUFFSIZE; u8i++)
    {
        au8UartTxBuf[u8i] = u8i;
    }
    // Clear DMA transfer finish flag
    bDma0Finished = FALSE;
    bDma1Finished = FALSE;

    /* Configure interrupt */
    PDL_ZERO_STRUCT(stcUart0IntSel);
    PDL_ZERO_STRUCT(stcUart1IntSel);

    stcUart0IntSel.bTxInt = TRUE;
    stcUart0IntCb.pfnTxIntCb = Uart0TxIntCallback;

    stcUart1IntSel.bRxInt = TRUE;
    stcUart1IntCb.pfnRxIntCb = Uart1RxIntCallback;

    Mfs_Uart_EnableInt(UartCh1, &stcUart1IntSel, &stcUart1IntCb);
    Mfs_Uart_EnableInt(UartCh0, &stcUart0IntSel, &stcUart0IntCb);

    // Wait for TX finish
    while(bDma0Finished != TRUE)
    {};

    // Wait for RX finish
    while(bDma1Finished != TRUE)
    {};

    /* Compare receive data with transfer data */
    if(Error == CompareData(au8UartTxBuf, au8UartRxBuf, SAMPLE_UART_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }

/******************************************************/
/* Test 2nd UART transfer/receive data with DMA       */
/******************************************************/
    // Initialize send data
    for(u8i=0; u8i<SAMPLE_UART_TX_BUFFSIZE; u8i++)
    {
        au8UartTxBuf[u8i] = ~u8i;
    }
    // Clear DMA transfer finish flag
    bDma0Finished = FALSE;
    bDma1Finished = FALSE;

    // Configure interrupt
    PDL_ZERO_STRUCT(stcUart0IntSel);
    PDL_ZERO_STRUCT(stcUart1IntSel);

    stcUart0IntSel.bTxInt = TRUE;
    stcUart0IntCb.pfnTxIntCb = Uart0TxIntCallback;

    stcUart1IntSel.bRxInt = TRUE;
    stcUart1IntCb.pfnRxIntCb = Uart1RxIntCallback;

    Mfs_Uart_EnableInt(UartCh1, &stcUart1IntSel, &stcUart1IntCb);
    Mfs_Uart_EnableInt(UartCh0, &stcUart0IntSel, &stcUart0IntCb);

    // Wait for TX finish
    while(bDma0Finished != TRUE)
    {};

    // Wait for RX finish
    while(bDma1Finished != TRUE)
    {};

    /* Compare receive data with transfer data */
    if(Error == CompareData(au8UartTxBuf, au8UartRxBuf, SAMPLE_UART_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }

    Mfs_Uart_DeInit(UartCh0);
    Mfs_Uart_DeInit(UartCh1);

    Dma_Disable() ;
    Dma_DeInit_Channel(0u);
    Dma_DeInit_Channel(1u);

    while(1); /* Data is normally sent and received */
}


/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
