/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates communication between UART0 and UART1 with 
 ** interrupt mode using FIFO. 
 ** The SOT0_0 should be connected with SIN1_1 before using this example.
 ** 
 ** History:
 **   - 2014-03-10  1.0  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/
/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/
#define SAMPLE_UART_RX_BUFFSIZE sizeof(au8UartTxBuf)/sizeof(char)
#define SAMPLE_UART_TX_BUFFSIZE SAMPLE_UART_RX_BUFFSIZE
#define SAMPLE_UART_FIFO_MAX_CAPACITY         (128u)
#define SAMPLE_UART_FIFO_RX_CNT               (8u)

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/
/// UART TX FIFO information structure
typedef struct stc_tx_fifo_info
{
    uint32_t u32TxCnt;
    uint8_t* pTxBuf;
    boolean_t bTxFinish;
    
}stc_tx_fifo_info_t;

/// UART RX FIFO information structure
typedef struct stc_rx_fifo_info
{
    uint32_t u32RxCnt;
    uint8_t* pRxBuf;
    boolean_t bRxFinish;
    
}stc_rx_fifo_info_t;
/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/
volatile stc_mfsn_uart_t* UartCh0 = &UART0;
volatile stc_mfsn_uart_t* UartCh1 = &UART1;
stc_uart_int_sel_t stcUart0IntSel, stcUart1IntSel;
stc_uart_int_cb_t  stcUart0IntCb, stcUart1IntCb;
static uint8_t au8UartTxBuf[200] = "Data sent from UART0 to UART1";
static uint8_t au8UartRxBuf[SAMPLE_UART_RX_BUFFSIZE];
stc_tx_fifo_info_t stcTxFifoInfo = {0};
stc_rx_fifo_info_t stcRxFifoInfo = {0};

/**
 ******************************************************************************
 ** \brief  Compare each data in the input two buffers
 **
 ** \relval Ok     The data in buffer1 are same with that in buffer2
 ** \relval Error  The data in buffer1 are not same with that in buffer2 
 ******************************************************************************/
static en_result_t CompareData(uint8_t* pBuf1, uint8_t* pBuf2, uint8_t u8Length)
{
    while(u8Length--)
    {
        if(*pBuf1++ != *pBuf2++)
        {
            return Error;
        }
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  UART0 FIFO TX interrupt callback function
 ******************************************************************************/
void Uart0FifoTxIntCallback(void)
{
    uint8_t u8i = 0;
    if(stcTxFifoInfo.u32TxCnt > SAMPLE_UART_FIFO_MAX_CAPACITY)
    {
        while(u8i < SAMPLE_UART_FIFO_MAX_CAPACITY)
        {
            Mfs_Uart_SendData(UartCh0, *stcTxFifoInfo.pTxBuf++);
            u8i++;
        }
        stcTxFifoInfo.u32TxCnt -= SAMPLE_UART_FIFO_MAX_CAPACITY;
        return;
    }
    
    while(u8i < stcTxFifoInfo.u32TxCnt)
    {
        Mfs_Uart_SendData(UartCh0, *stcTxFifoInfo.pTxBuf++);
        u8i++;
    }
  
    stcUart0IntSel.bTxFifoInt = TRUE;
    Mfs_Uart_DisableInt(UartCh0, &stcUart0IntSel);
    
    stcTxFifoInfo.bTxFinish = TRUE;
}

/**
 ******************************************************************************
 ** \brief  UART0 RX interrupt callback function
 ******************************************************************************/
void Uart1RxIntCallback(void)
{
    uint8_t u8i = 0;
    if(stcRxFifoInfo.u32RxCnt > SAMPLE_UART_FIFO_RX_CNT) 
    {
        /* Receive data when RX FIFO count match with SAMPLE_UART_FIFO_RX_CNT */
        while(u8i < SAMPLE_UART_FIFO_RX_CNT)
        {
            *stcRxFifoInfo.pRxBuf++ = Mfs_Uart_ReceiveData(UartCh1); 
            u8i++;
        }
        stcRxFifoInfo.u32RxCnt -= SAMPLE_UART_FIFO_RX_CNT;
        return;
    }
    
    /* Receive data when RX FIFO is idle */
    /* idle means FIFO count is less than SAMPLE_UART_FIFO_RX_CNT and
       RX FIFO don't receive data from then on for a short time. */
    while(u8i < stcRxFifoInfo.u32RxCnt)
    {
        *stcRxFifoInfo.pRxBuf++ = Mfs_Uart_ReceiveData(UartCh1);
        u8i++;
    }
    
    stcUart1IntSel.bRxInt = TRUE;
    Mfs_Uart_DisableInt(UartCh1, &stcUart1IntSel);
    
    stcRxFifoInfo.bRxFinish = TRUE;
}

/**
 ******************************************************************************
 ** \brief  Main function of project for MB9BF56X series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{   
    stc_mfs_uart_config_t stcUartConfig;
    
    // Initialize the FIFO information   
    stcTxFifoInfo.u32TxCnt = SAMPLE_UART_TX_BUFFSIZE;
    stcTxFifoInfo.pTxBuf = au8UartTxBuf;
    stcTxFifoInfo.bTxFinish = FALSE;
    
    stcRxFifoInfo.u32RxCnt = SAMPLE_UART_RX_BUFFSIZE;
    stcRxFifoInfo.pRxBuf = au8UartRxBuf;
    stcRxFifoInfo.bRxFinish = FALSE;
    
    /* Initialize UART function I/O */
    SetPinFunc_SIN0_0();
    SetPinFunc_SOT0_0();
    SetPinFunc_SIN1_1();
    SetPinFunc_SOT1_1();

    /* Initialize UART TX and RX channel  */
    stcUartConfig.enMode = UartNormal;
    stcUartConfig.u32BautRate = 115200;
    stcUartConfig.enDataLength = UartEightBits;
    stcUartConfig.enParity = UartParityNone;
    stcUartConfig.enStopBit = UartOneStopBit;
    stcUartConfig.enBitDirection = UartDataLsbFirst; 
    stcUartConfig.bInvertData = FALSE;
    stcUartConfig.bHwFlow = FALSE;
    stcUartConfig.bEnableFifo = TRUE;
    stcUartConfig.stcFifoConfig.enFifoSel = MfsTxFifo1RxFifo2;
    stcUartConfig.stcFifoConfig.u8ByteCount1 = 0;
    stcUartConfig.stcFifoConfig.u8ByteCount2 = 8;

    Mfs_Uart_Init(UartCh0, &stcUartConfig);
    Mfs_Uart_Init(UartCh1, &stcUartConfig);
    
    /* Enable RX function of UART0   */
    Mfs_Uart_EnableFunc(UartCh1, UartRx);
    /* Enable TX function of UART0   */
    Mfs_Uart_EnableFunc(UartCh0, UartTx);
    
    /* Configure interrupt */
    PDL_ZERO_STRUCT(stcUart0IntSel);
    PDL_ZERO_STRUCT(stcUart1IntSel);
    
    stcUart0IntSel.bTxFifoInt = TRUE;
    stcUart0IntCb.pfnTxFifoIntCb = Uart0FifoTxIntCallback;
    
    stcUart1IntSel.bRxInt = TRUE;
    stcUart1IntCb.pfnRxIntCb = Uart1RxIntCallback;
    
    Mfs_Uart_EnableInt(UartCh1, &stcUart1IntSel, &stcUart1IntCb);
    Mfs_Uart_EnableInt(UartCh0, &stcUart0IntSel, &stcUart0IntCb);
    
    while(stcRxFifoInfo.bRxFinish != TRUE)
    {
        ;
    }
        
    Mfs_Uart_DeInit(UartCh0);
    Mfs_Uart_DeInit(UartCh1);
    
    /* Compare receive data with transfer data */
    if(Error == CompareData(au8UartTxBuf, au8UartRxBuf, SAMPLE_UART_RX_BUFFSIZE))
    {
        while(1);  /* If code runs here, the communicate error occurs */
    }
    
    while(1); /* Data is normally sent and received */
}


/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
