/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates motor emergency stop function of waveform
 ** generator. WFG will stop output after low level is detected on the
 ** DTTI0X pin.
 **
 ** History:
 **   - 2013-05-23  0.0.1  frank.li        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/* FRT module */
#define USER_FRT                 MFT0_FRT
#define USER_MFT_FRT_CH          MFT_FRT_CH1
#define USER_FRT_MODE            FrtUpCount
#define FRT_CYCLE               (50000)

/* OCU module */
#define USER_OCU                 MFT0_OCU
#define USER_OCU_CH0             MFT_OCU_CH0
#define USER_OCU_CH1             MFT_OCU_CH1
#define USER_OCU_FRT_CH          USER_MFT_FRT_CH

//OCU compare value
#define USER_OCU_CH0_CMPVAL1     (10000)
#define USER_OCU_CH0_CMPVAL2     (30000)
#define USER_OCU_CH1_CMPVAL1     (20000)
#define USER_OCU_CH1_CMPVAL2     (40000)

/* WFG module */
#define USER_WFG                  MFT0_WFG
#define USER_WFG_CH               MFT_WFG_CH10
#define WFG_MODE                  WfgThroughMode
#define WFG_PRE_CLK               WfgPlckDiv64
#define InitWfgOutput()           {SetPinFunc_RTO00_0(); SetPinFunc_RTO01_0();}
#define InitDttixPin()            {SetPinFunc_DTTI0X_0();}

/* Key */
#define GetKeyInput()             Gpio1pin_Get(GPIO1PIN_P0F)

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static uint8_t m_u8Ocu0IntFlag = 0;
static uint8_t m_u8Ocu1IntFlag = 0;

/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/
/**
 ******************************************************************************
 ** \brief  OCU channel 0 interrupt callback function   .
 **
 ** \return none
 ******************************************************************************/
static void Ocu0IntCallback(void)
{
    if(m_u8Ocu0IntFlag)
    {
        Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH0, USER_OCU_CH0_CMPVAL1);
    }
    else
    {
        Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH0, USER_OCU_CH0_CMPVAL2);
    }
    m_u8Ocu0IntFlag = !m_u8Ocu0IntFlag;
}

/**
 ******************************************************************************
 ** \brief  OCU channel 1 interrupt callback function  .
 **
 ** \return none
 ******************************************************************************/
static void Ocu1IntCallback(void)
{
    if(m_u8Ocu1IntFlag)
    {
        Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH1, USER_OCU_CH1_CMPVAL1);
    }
    else
    {
        Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH1, USER_OCU_CH1_CMPVAL2);
    }
    m_u8Ocu1IntFlag = !m_u8Ocu1IntFlag;
}

/**
 ******************************************************************************
 ** \brief  Init user free run timer   .
 **
 ** \return none
 ******************************************************************************/
static void InitFrt(void)
{
    stc_mft_frt_config_t stcFrtConfig;

    //configure the FRT parameter
    stcFrtConfig.enFrtMode = USER_FRT_MODE;
    stcFrtConfig.enFrtClockDiv = FrtPclkDiv64;
    stcFrtConfig.bEnExtClock = FALSE;
    stcFrtConfig.bEnBuffer = FALSE;

    Mft_Frt_Stop(&USER_FRT,USER_MFT_FRT_CH);
    //init frt module
    Mft_Frt_Init(&USER_FRT,USER_MFT_FRT_CH,&stcFrtConfig);
    //set frt cycle value
    Mft_Frt_SetCountCycle(&USER_FRT,USER_MFT_FRT_CH,FRT_CYCLE);
}

/**
 ******************************************************************************
 ** \brief  Init user output compare unit  .
 **
 ** \return none
 ******************************************************************************/
static void InitOcu(void)
{
    stc_mft_ocu_config_t stcOcuConfig;
    stc_even_compare_config_t stcCh0CompareConfig;
    stc_odd_compare_config_t  stcCh1CompareConfig;

    /*configure the OCU parameter*/
    stcOcuConfig.enFrtConnect = (en_mft_ocu_frt_t)USER_OCU_FRT_CH;
    stcOcuConfig.bFm4 = TRUE;
    stcOcuConfig.bCmod = FALSE;
    stcOcuConfig.u8Mod = 0;
    stcOcuConfig.enStatePin = RtLowLevel;
    stcOcuConfig.enOccpBufMode = OccpBufDisable;
    stcOcuConfig.enOcseBufMode = OcseBufDisable;

    //Disable OCU interrupt
    Mft_Ocu_DisableOperation(&USER_OCU,USER_OCU_CH0);
    Mft_Ocu_DisableOperation(&USER_OCU,USER_OCU_CH1);

    //init OCU
    Mft_Ocu_Init(&USER_OCU,USER_OCU_CH0,&stcOcuConfig);
    Mft_Ocu_Init(&USER_OCU,USER_OCU_CH1,&stcOcuConfig);

    // OCU1&OCU0: 1 change mode
    // OCSE0:        0x00000FFF
    // OCSE1:        0x0FF00FFF
    stcCh0CompareConfig.enFrtZeroEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtZeroEvenNotMatchEvenChRtStatus = RtOutputHold;
    stcCh0CompareConfig.enFrtUpCntEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtPeakEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtPeakEvenNotMatchEvenChStatus = RtOutputHold;
    stcCh0CompareConfig.enFrtDownCntEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enIopFlagWhenFrtZeroEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtUpCntEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtPeakEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtDownCntEvenMatch = IopFlagSet;

    stcCh1CompareConfig.enFrtZeroOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtZeroOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtZeroOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtZeroOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtUpCntOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtUpCntOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtUpCntOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtPeakOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtPeakOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtPeakOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtPeakOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtDownOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtDownOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtDownOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enIopFlagWhenFrtZeroOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtUpCntOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtPeakOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtDownCntOddMatch = IopFlagSet;

    Mft_Ocu_SetEvenChCompareMode(&USER_OCU, USER_OCU_CH0, &stcCh0CompareConfig);
    Mft_Ocu_SetOddChCompareMode(&USER_OCU, USER_OCU_CH1, &stcCh1CompareConfig);

    //write OCU compare value
    Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH0,USER_OCU_CH0_CMPVAL1);
    Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH1,USER_OCU_CH1_CMPVAL1);

    // Clear int flag
    Mft_Ocu_ClrIntFlag(&USER_OCU,USER_OCU_CH0);
    Mft_Ocu_ClrIntFlag(&USER_OCU,USER_OCU_CH1);

    //Enable OCU interrupt
    Mft_Ocu_EnableInt(&USER_OCU,USER_OCU_CH0,Ocu0IntCallback);
    Mft_Ocu_EnableInt(&USER_OCU,USER_OCU_CH1,Ocu1IntCallback);
}

/**
 ******************************************************************************
 ** \brief  Init DTIF callback function
 **
 ** \return none
 ******************************************************************************/
static void DtifIntCallback(void)
{
    if(GetKeyInput() == 0)
    {
        Mft_Wfg_ClrDigitalFilterIntFlag(&USER_WFG);

        //Clear Flag
        m_u8Ocu0IntFlag = 0;
        m_u8Ocu1IntFlag = 0;

        //re-initialize FRT, OCU
        InitFrt();
        InitOcu();

        //re-start FRT, OCU
        Mft_Frt_Start(&USER_FRT,USER_MFT_FRT_CH);
        Mft_Ocu_EnableOperation(&USER_OCU,USER_OCU_CH0);
        Mft_Ocu_EnableOperation(&USER_OCU,USER_OCU_CH1);
    }
}

/**
 ******************************************************************************
 ** \brief  Init User Waveform Generator Unit    .
 **
 ** \return none
 ******************************************************************************/
static void InitWfg(void)
{
    stc_wfg_ctrl_bits_t stcWfgCtrl;
    stc_wfg_nzcl_config_t  stcNzclConfig;
    stc_dtif_int_sel_t stcDtifIntSel;
    stc_dtif_int_cb_t  stcDtifIntCb;
    /*config WFG module*/
    InitWfgOutput();
    stcWfgCtrl.enPselBits = PselBits01B;
    stcWfgCtrl.enPgenBits = PgenBits00B;
    stcWfgCtrl.enGtenBits = GtenBits01B;
    stcWfgCtrl.enDmodBit = DmodBit0B;
    Mft_Wfg_ConfigMode(&USER_WFG,USER_WFG_CH,WFG_MODE);
    Mft_Wfg_ConfigCtrlBits(&USER_WFG,USER_WFG_CH,&stcWfgCtrl);

    /* config NZCL */
    InitDttixPin();
    stcNzclConfig.bEnAnalogFilter = FALSE;
    stcNzclConfig.bEnDigitalFilter = TRUE;
    stcNzclConfig.enDigitalFilterWidth = NzlcWidth8Cycle;
    stcNzclConfig.bSwitchToGpio = TRUE;
    Mft_Wfg_ConfigNzcl(&USER_WFG, &stcNzclConfig);
    stcDtifIntSel.bDtifAnalogFilterInt = FALSE;
    stcDtifIntSel.bDtifDigitalFilterInt = TRUE;
    stcDtifIntCb.pfnWfgAnalogFilterCallback = NULL;
    stcDtifIntCb.pfnWfgDigtalFilterCallback = DtifIntCallback;
    Mft_Wfg_EnableDtifInt(&USER_WFG, &stcDtifIntSel, &stcDtifIntCb);

}

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series.
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("          WFG Sample Start \n");
    printf(" WFG running with through mode,\n");
    printf(" please observe the RTO00_0 and RTO01_0 output by \n oscillograph\n");
    printf(".......\n");
#endif
    //init FRT
    InitFrt();
    //init OCU
    InitOcu();
    //init WFG
    InitWfg();

    //enable OCU operation
    Mft_Ocu_EnableOperation(&USER_OCU,USER_OCU_CH0);
    Mft_Ocu_EnableOperation(&USER_OCU,USER_OCU_CH1);

    //start FRT
    Mft_Frt_Start(&USER_FRT,USER_MFT_FRT_CH);

    while(1)
    {
        ;
    }
}
