/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates PPG dead timer mode of  waveform generator
 ** PPG0 connect to WFG10
 ** History:
 **   - 2013-05-23  0.0.1  Frank.li        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/* FRT module */
#define USER_FRT             MFT0_FRT    
#define USER_FRT_CH          MFT_FRT_CH1 
#define FRT_CYCLE            50000UL 

/*  WFG module */
#define USER_WFG             MFT0_WFG
#define USER_WFG_CH          MFT_WFG_CH10
#define WFG_MODE             WfgPpgDeadTimerMode
#define WFG_PRE_CLK          WfgPlckDiv1
#define USER_PSEL_BITS       PselBits00B
#define USER_GTEN_BITS       GtenBits11B
#define USER_PGEN_BITS       PgenBits11B
#define USER_DMOD_BITS       DmodBit0B

//init value of wfg timer
#define WFG_CYCLE            50
#define WFG_WFTA             50
#define WFG_WFTB             50
// init wfg I/O
#define InitWfgOutput()      {SetPinFunc_RTO00_0(); SetPinFunc_RTO01_0();}

/*  PPG module */
#define USER_PPG             PPG_CH0
#define PPG_LOW_WIDTH        200
#define PPG_HIGH_WIDTH       200

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/

/**
 ******************************************************************************
 ** \brief  Init User PPG module    .
 **
 ** \return none
 ******************************************************************************/
static void InitPpg(void)
{
    stc_ppg_config_t stcPpgConfig;

    //set the ppg config value
    stcPpgConfig.enEvenClock = PpgPclkDiv64;
    stcPpgConfig.enOddClock = PpgPclkDiv64;
    stcPpgConfig.enEvenLevel = PpgReverseLevel;
    stcPpgConfig.enOddLevel = PpgReverseLevel;
    stcPpgConfig.enTrig = PpgSoftwareTrig;
    stcPpgConfig.enMode = Ppg8Bit8Bit;

    //init ppg
    Ppg_Init(USER_PPG,&stcPpgConfig);
    Ppg_SetLevelWidth(USER_PPG,PPG_LOW_WIDTH,PPG_HIGH_WIDTH);
    Ppg_StartSoftwareTrig(USER_PPG);
    
}
/**
 ******************************************************************************
 ** \brief  Init User Waveform Generator Unit    .
 **
 ** \return none
 ******************************************************************************/
static void InitWfg(void)
{
    static stc_wfg_ctrl_bits_t StcWfgCtrl;
    
    StcWfgCtrl.enPselBits = USER_PSEL_BITS;
    StcWfgCtrl.enPgenBits = USER_PGEN_BITS;
    StcWfgCtrl.enGtenBits = USER_GTEN_BITS;
    StcWfgCtrl.enDmodBit = USER_DMOD_BITS;

    /*enable wfg GPIO*/
    InitWfgOutput();
    
    /*config WFG module*/
    //config mode
    Mft_Wfg_ConfigMode(&USER_WFG,USER_WFG_CH,WFG_MODE);
    //config control output 
    Mft_Wfg_ConfigCtrlBits(&USER_WFG,USER_WFG_CH,&StcWfgCtrl);
    //config WFG clock
    Mft_Wfg_InitTimerClock(&USER_WFG,USER_WFG_CH,WfgPlckDiv64);
    //set wfg timer value
    Mft_Wfg_WriteTimerCountCycle(&USER_WFG,USER_WFG_CH,WFG_WFTA,WFG_WFTB);
    //start timer
    Mft_Wfg_StartTimer(&USER_WFG,USER_WFG_CH);
}

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series.
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
   
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("          WFG Sample Start \n");
    printf(" WFG running PPG dead timer mode;\n");
    printf(" please observe the RTO00_0 and RTO01_0 output by \n oscillograph\n");
    printf(".......\n");
#endif
    //init PPG, use ppg0
    InitPpg();
    //init WFG
    InitWfg();

    while(1)
    {
        ;
    }
}
