/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates timer PPG mode of  waveform generator
 ** PPG0 connect to WFG10
 ** History:
 **   - 2013-05-23  0.0.1  frank.li        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/* FRT module */
#define USER_FRT             MFT0_FRT
#define USER_FRT_CH          MFT_FRT_CH2
#define FRT_CYCLE            50000

/* OCU module */
#define USER_OCU             MFT0_OCU
#define USER_OCU_CH0         MFT_OCU_CH0
#define USER_OCU_CH1         MFT_OCU_CH1
#define USER_OCU_FRT_CH      USER_FRT_CH
#define USER_OCU_CH0_CMPVAL1 (10000)
#define USER_OCU_CH0_CMPVAL2 (20000)
#define USER_OCU_CH1_CMPVAL1 (30000)
#define USER_OCU_CH1_CMPVAL2 (40000)

/* WFG module */
#define USER_WFG              MFT0_WFG
#define USER_WFG_CH           MFT_WFG_CH10
#define WFG_MODE              WfgTimerPpgMode
#define WFG_GTEN_BITS         GtenBits11B
#define WFG_PSEL_BITS         PselBits00B
#define WFG_PGEN_BITS         PgenBits11B
#define WFG_DMOD_BITS         DmodBit0B
#define WFG_WFTA              2000
#define WFG_WFTB              2000
#define InitWfgOutput()       {SetPinFunc_RTO00_0(); SetPinFunc_RTO01_0();}

/* PPG module */
#define USER_PPG_CH           PPG_CH0

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static uint8_t m_u8Ocu0IntFlag = 0;
static uint8_t m_u8Ocu1IntFlag = 0;
/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/
/**
 ******************************************************************************
 ** \brief  OCU channel 0 interrupt callback function   .
 **
 ** \return none
 ******************************************************************************/
static void Ocu0IntCallback(void)
{
    if(m_u8Ocu0IntFlag)
    {
        Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH0, USER_OCU_CH0_CMPVAL1);
    }
    else
    {
        Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH0, USER_OCU_CH0_CMPVAL2);
    }
    m_u8Ocu0IntFlag = !m_u8Ocu0IntFlag;
}

/**
 ******************************************************************************
 ** \brief  OCU channel 1 interrupt callback function  .
 **
 ** \return none
 ******************************************************************************/
static void Ocu1IntCallback(void)
{
    if(m_u8Ocu1IntFlag)
    {
        Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH1, USER_OCU_CH1_CMPVAL1);
    }
    else
    {
        Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH1, USER_OCU_CH1_CMPVAL2);
    }
    m_u8Ocu1IntFlag = !m_u8Ocu1IntFlag;
}

/**
 ******************************************************************************
 ** \brief  Init user free run timer   .
 **
 ** \return none
 ******************************************************************************/
static void InitFrt(void)
{
    stc_mft_frt_config_t stcFrtConfig;

    //configure the FRT parameter
    stcFrtConfig.enFrtMode = FrtUpCount;
    stcFrtConfig.enFrtClockDiv = FrtPclkDiv64;
    stcFrtConfig.bEnExtClock = FALSE;
    stcFrtConfig.bEnBuffer = TRUE;

    Mft_Frt_Stop(&USER_FRT,USER_FRT_CH);
    //init frt module
    Mft_Frt_Init(&USER_FRT,USER_FRT_CH,&stcFrtConfig);
    //set frt cycle value
    Mft_Frt_SetCountCycle(&USER_FRT,USER_FRT_CH,FRT_CYCLE);
}

/**
 ******************************************************************************
 ** \brief  Init user output compare unit  .
 **
 ** \return none
 ******************************************************************************/
static void InitOcu(void)
{
    stc_mft_ocu_config_t stcOcuConfig;
    stc_even_compare_config_t stcCh0CompareConfig;
    stc_odd_compare_config_t  stcCh1CompareConfig;

    /*configure the OCU parameter*/
    stcOcuConfig.enFrtConnect = (en_mft_ocu_frt_t)USER_OCU_FRT_CH;
    stcOcuConfig.bFm4 = TRUE;
    stcOcuConfig.enStatePin = RtLowLevel;
    stcOcuConfig.bCmod = FALSE;
    stcOcuConfig.u8Mod = 0;
    stcOcuConfig.enOccpBufMode = OccpBufDisable;
    stcOcuConfig.enOcseBufMode = OcseBufDisable;

    //init OCU
    Mft_Ocu_Init(&USER_OCU,USER_OCU_CH0,&stcOcuConfig);
    Mft_Ocu_Init(&USER_OCU,USER_OCU_CH1,&stcOcuConfig);

    // OCU1&OCU0: 1 change mode
    // OCSE0:        0x00000FFF
    // OCSE1:        0x0FF00FFF
    stcCh0CompareConfig.enFrtZeroEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtZeroEvenNotMatchEvenChRtStatus = RtOutputHold;
    stcCh0CompareConfig.enFrtUpCntEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtPeakEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtPeakEvenNotMatchEvenChStatus = RtOutputHold;
    stcCh0CompareConfig.enFrtDownCntEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enIopFlagWhenFrtZeroEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtUpCntEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtPeakEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtDownCntEvenMatch = IopFlagSet;

    stcCh1CompareConfig.enFrtZeroOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtZeroOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtZeroOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtZeroOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtUpCntOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtUpCntOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtUpCntOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtPeakOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtPeakOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtPeakOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtPeakOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtDownOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtDownOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtDownOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enIopFlagWhenFrtZeroOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtUpCntOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtPeakOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtDownCntOddMatch = IopFlagSet;

    Mft_Ocu_SetEvenChCompareMode(&USER_OCU, USER_OCU_CH0, &stcCh0CompareConfig);
    Mft_Ocu_SetOddChCompareMode(&USER_OCU, USER_OCU_CH1, &stcCh1CompareConfig);

    //write OCU compare value
    Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH0,USER_OCU_CH0_CMPVAL1);
    Mft_Ocu_WriteOccp(&USER_OCU,USER_OCU_CH1,USER_OCU_CH1_CMPVAL1);

    //Enable OCU interrupt
    Mft_Ocu_EnableInt(&USER_OCU,USER_OCU_CH0,Ocu0IntCallback);
    Mft_Ocu_EnableInt(&USER_OCU,USER_OCU_CH1,Ocu1IntCallback);
}

/**
 ******************************************************************************
 ** \brief  Init User PPG module    .
 **
 ** \return none
 ******************************************************************************/
static void InitPpg(void)
{
    stc_ppg_config_t stcPpgConfig;

    //set the ppg config value
    stcPpgConfig.enEvenClock = PpgPclkDiv4;
    stcPpgConfig.enOddClock = PpgPclkDiv4;
    stcPpgConfig.enEvenLevel = PpgNormalLevel;
    stcPpgConfig.enOddLevel = PpgNormalLevel;
    stcPpgConfig.enTrig = PpgSoftwareTrig;
    stcPpgConfig.enMode = Ppg8Bit8Bit;

    //init ppg
    Ppg_Init(USER_PPG_CH,&stcPpgConfig);
    Ppg_SetLevelWidth(USER_PPG_CH,100,100);
    Ppg_StartSoftwareTrig(USER_PPG_CH);
}
/**
 ******************************************************************************
 ** \brief  Init User Waveform Generator Unit    .
 **
 ** \return none
 ******************************************************************************/
static void InitWfg(void)
{
    static stc_wfg_ctrl_bits_t stcWfgCtrl;

    stcWfgCtrl.enPselBits = WFG_PSEL_BITS;
    stcWfgCtrl.enPgenBits = WFG_PGEN_BITS;
    stcWfgCtrl.enGtenBits = WFG_GTEN_BITS;
    stcWfgCtrl.enDmodBit = WFG_DMOD_BITS;

    /*enable WFG GPIO*/
    InitWfgOutput();

    /*config WFG module*/
    //config mode
    Mft_Wfg_ConfigMode(&USER_WFG,USER_WFG_CH,WFG_MODE);
    //config control output
    Mft_Wfg_ConfigCtrlBits(&USER_WFG,USER_WFG_CH,&stcWfgCtrl);
    //config WFG clock
    Mft_Wfg_InitTimerClock(&USER_WFG,USER_WFG_CH,WfgPlckDiv64);
    //set wfg timer value
    Mft_Wfg_WriteTimerCountCycle(&USER_WFG,USER_WFG_CH,WFG_WFTA,WFG_WFTB);
    //enable wfg timer interrupt
    Mft_Wfg_StartTimer(&USER_WFG,USER_WFG_CH);
}

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series.
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("          WFG Sample Start \n");
    printf(" WFG running with timer-PPG mode;\n");
    printf(" please observe the RTO00_0 and RTO01_0 output by \n oscillograph\n");
    printf(".......\n");
#endif
    //init FRT
    InitFrt();
    //init OCU
    InitOcu();
    //init PPG, use ppg0
    InitPpg();
    //init WFG
    InitWfg();

    //enable OCU operation
    Mft_Ocu_EnableOperation(&USER_OCU,USER_OCU_CH0);
    Mft_Ocu_EnableOperation(&USER_OCU,USER_OCU_CH1);

    //start FRT
    Mft_Frt_Start(&USER_FRT,USER_FRT_CH);

    while(1)
    {
        ;
    }
}
