/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file mft_adcmp.c
 **
 ** A detailed description is available at 
 ** @link AdcmpGroup ADCMP Module description @endlink
 **
 ** History:
 **   - 2014-01-10  0.1  Edison Zhang  First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mft_adcmp.h"

#if (defined(PDL_PERIPHERAL_MFT_ADCMP_ACTIVE))
/**
 ******************************************************************************
 ** \addtogroup AdcmpGroup
 ******************************************************************************/
//@{
/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/

/**
 ******************************************************************************
 ** \brief Device dependent initialization of Mft adcmp module
 **       
 ** \param [in] pstcMftAdcmp    Pointer to Mft instance
 ** \param [in] u8Ch            Mft adcmp channel
 ** \arg        MFT_ADCMP_CH0 ADCMP channel 0
 ** \arg        MFT_ADCMP_CH1 ADCMP channel 1
 ** \arg        MFT_ADCMP_CH2 ADCMP channel 2
 ** \arg        MFT_ADCMP_CH3 ADCMP channel 3
 ** \arg        MFT_ADCMP_CH4 ADCMP channel 4
 ** \arg        MFT_ADCMP_CH5 ADCMP channel 5
 ** \param [in] pstcConfig      Pointer to Mft adcmp config
 ** \arg        structure of Mft Adcmp configuration
 **
 ** \retval Ok                        Adcmp initialized 
 ** \retval ErrorInvalidParameter     If one of following conditions are met:
 **                                   - pstcMftAdcmp == NULL
 **                                   - pstcConfig == NULL
 **                                   - u8Ch > MFT_ADCMP_CH_MAX
 ******************************************************************************/
en_result_t Mft_Adcmp_Init(volatile stc_mftn_adcmp_t* pstcMftAdcmp, 
                            uint8_t u8Ch, 
                            stc_mft_adcmp_config_t* pstcConfig)
{
    volatile stc_mft_adcmp_acfs10_field_t* pu8ACFS10 ;
    volatile stc_mft_adcmp_acfs32_field_t* pu8ACFS32 ;
    volatile stc_mft_adcmp_acfs54_field_t* pu8ACFS54 ;
    
    volatile stc_mft_adcmp_acsc0_field_t*   pu8ACSCx ;
    volatile stc_mft_adcmp_acsd0_field_t*   pstcACSDx ;
     
    if ((pstcMftAdcmp == NULL)||(pstcConfig == NULL)||(u8Ch > MFT_ADCMP_CH_MAX))
    {
        return ErrorInvalidParameter ;
    }

    // Get actual address of register list of current channel
    pu8ACFS10 = (volatile stc_mft_adcmp_acfs10_field_t*)(&pstcMftAdcmp->ACFS10);
    pu8ACFS32 = (volatile stc_mft_adcmp_acfs32_field_t*)(&pstcMftAdcmp->ACFS32);
    pu8ACFS54 = (volatile stc_mft_adcmp_acfs54_field_t*)(&pstcMftAdcmp->ACFS54);
    
    pu8ACSCx = (volatile stc_mft_adcmp_acsc0_field_t*)(&pstcMftAdcmp->ACSC0 + 0x4*u8Ch);
    pstcACSDx = (volatile stc_mft_adcmp_acsd0_field_t*)(&pstcMftAdcmp->ACSD0 + 0x4*u8Ch);

    // Configure parameter
    pstcACSDx->DE = 0u;
    pstcACSDx->PE = 0u;
    pstcACSDx->UE = 0u;
    pstcACSDx->ZE = 0u;
    
    pstcMftAdcmp->ACSA &= 0x0000u;
	
    // Select FRT channel : FRTx -> ADCMPx
    switch(u8Ch)
    {
        case 0:
            pu8ACFS10->FSA0 = (uint8_t)pstcConfig->enFrt;
            break;
        case 1:
            pu8ACFS10->FSA1 = (uint8_t)pstcConfig->enFrt;
            break;
        case 2:
            pu8ACFS32->FSA0 = (uint8_t)pstcConfig->enFrt;
            break;
        case 3:
            pu8ACFS32->FSA1 = (uint8_t)pstcConfig->enFrt;
            break;
        case 4:
            pu8ACFS54->FSA0 = (uint8_t)pstcConfig->enFrt;
            break;
        case 5:
            pu8ACFS54->FSA1 = (uint8_t)pstcConfig->enFrt;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    // configure buffer function
    if(pstcConfig->enBuf > AdcmpBufFrtZeroPeak)
    {
        return ErrorInvalidParameter;
    }
    
    pu8ACSCx->BUFE = (uint8_t)pstcConfig->enBuf;
	
    // configure start trigger output channel number
    if(pstcConfig->enTrigSel > AdcmpTrigAdc2Prio)
    {
        return ErrorInvalidParameter;
    }
          
    pu8ACSCx->ADSEL = (uint8_t)pstcConfig->enTrigSel;
	
    // Select ADCMP running mode
    if(pstcConfig->enMode > AdcmpOffsetMode)
    {
        return ErrorInvalidParameter;
    }
    pstcACSDx->AMOD = (uint8_t)pstcConfig->enMode;
	
    // select OCU OCCP register: OCCP(x)
    if(pstcConfig->enOccpSel > AdcmpSelOccp1)
    {
        return ErrorInvalidParameter;
    }
    pstcACSDx->OCUS = (uint8_t)pstcConfig->enOccpSel;

    return Ok;
}
/**
 ******************************************************************************
 ** \brief Enable Mft Adcmp operations
 **
 ** \param [in] pstcMftAdcmp  Pointer to Mft instance
 ** \param [in] u8Ch          Mft Adcmp channel
 ** \arg        MFT_ADCMP_CH0 ADCMP channel 0
 ** \arg        MFT_ADCMP_CH1 ADCMP channel 1
 ** \arg        MFT_ADCMP_CH2 ADCMP channel 2
 ** \arg        MFT_ADCMP_CH3 ADCMP channel 3
 ** \arg        MFT_ADCMP_CH4 ADCMP channel 4
 ** \arg        MFT_ADCMP_CH5 ADCMP channel 5
 ** \param [in] pstcFunc      Pointer to Mft Adcmp function structure
 **          
 ** \retval Ok                     Functions enabled 
 ** \retval ErrorInvalidParameter  If one of following conditions are met:
 **                                - pstcMftAdcmp == NULL
 **                                - pstcFunc == NULL
 **                                - u8Ch > MFT_ADCMP_CH_MAX
 ******************************************************************************/
en_result_t Mft_Adcmp_EnableOperation(volatile stc_mftn_adcmp_t *pstcMftAdcmp, 
                                      uint8_t u8Ch, 
                                      stc_mft_adcmp_func_t* pstcFunc)
{
    volatile stc_mft_adcmp_acsd0_field_t*   pstcACSDx ;
	
    // Check for NULL pointer and channel parameter
    if ((pstcMftAdcmp == NULL)||(pstcFunc == NULL)||(u8Ch > MFT_ADCMP_CH_MAX))
    {
        return ErrorInvalidParameter ;
    }  
    // Get actual address of register list of current channel
    pstcACSDx = (volatile stc_mft_adcmp_acsd0_field_t*)(&pstcMftAdcmp->ACSD0 + 0x4*u8Ch);
	
    pstcACSDx->PE = pstcFunc->bPeakEn;
    pstcACSDx->ZE = pstcFunc->bZeroEn;
    pstcACSDx->UE = pstcFunc->bUpEn;
    pstcACSDx->DE = pstcFunc->bDownEn;
	
    return Ok;
}
/**
 ******************************************************************************
 ** \brief Disable Mft Adcmp operations
 **
 ** \param [in] pstcMftAdcmp  Pointer to Mft instance
 ** \param [in] u8Ch          Mft Adcmp channel
 ** \arg        MFT_ADCMP_CH0 ADCMP channel 0
 ** \arg        MFT_ADCMP_CH1 ADCMP channel 1
 ** \arg        MFT_ADCMP_CH2 ADCMP channel 2
 ** \arg        MFT_ADCMP_CH3 ADCMP channel 3
 ** \arg        MFT_ADCMP_CH4 ADCMP channel 4
 ** \arg        MFT_ADCMP_CH5 ADCMP channel 5
 ** \param [in] pstcFunc      Pointer to Mft Adcmp function structure
 ** \retval Ok                     Mft Adcmp operations disabled
 ** \retval ErrorInvalidParameter  If one of following conditions are met:
 **                                - pstcMftAdcmp == NULL
 **                                - pstcFunc == NULL
 **                                - u8Ch > MFT_ADCMP_CH_MAX
 ******************************************************************************/
en_result_t Mft_Adcmp_DisableOperation(volatile stc_mftn_adcmp_t *pstcMftAdcmp, 
                                       uint8_t u8Ch, 
                                       stc_mft_adcmp_func_t* pstcFunc)
{
    volatile stc_mft_adcmp_acsd0_field_t*   pstcACSDx;
	
    // Check for NULL pointer and channel parameter
    if ((pstcMftAdcmp == NULL)||(pstcFunc == NULL)||(u8Ch > MFT_ADCMP_CH_MAX))
    {
        return ErrorInvalidParameter ;
    }  
    // Get actual address of register list of current channel
    pstcACSDx = (volatile stc_mft_adcmp_acsd0_field_t*)(&pstcMftAdcmp->ACSD0 + 0x4*u8Ch);
	
    pstcACSDx->PE = pstcFunc->bPeakEn;
    pstcACSDx->ZE = pstcFunc->bZeroEn;
    pstcACSDx->UE = pstcFunc->bUpEn;
    pstcACSDx->DE = pstcFunc->bDownEn;
	
    return Ok;
}
/**
 ******************************************************************************
 ** \brief Write compare or offset value to ADCMP
 **
 ** \param [in] pstcMftAdcmp  Pointer to Mft instance
 ** \param [in] u8Ch          Mft Adcmp channel
 ** \arg        MFT_ADCMP_CH0 ADCMP channel 0
 ** \arg        MFT_ADCMP_CH1 ADCMP channel 1
 ** \arg        MFT_ADCMP_CH2 ADCMP channel 2
 ** \arg        MFT_ADCMP_CH3 ADCMP channel 3
 ** \arg        MFT_ADCMP_CH4 ADCMP channel 4
 ** \arg        MFT_ADCMP_CH5 ADCMP channel 5
 ** \param [in] u16AdcmpVal ADCMP compare value
 **
 ** \retval Ok                        Compare or offset value to ADCMP is set
 ** \retval ErrorInvalidParameter     If one of following condiitons are met:
 **                                   - pstcMftAdcmp == NULL
 **                                   - u8Ch > MFT_ADCMP_CH_MAX
 ******************************************************************************/
en_result_t Mft_Adcmp_WriteAcmp(volatile stc_mftn_adcmp_t *pstcMftAdcmp, uint8_t u8Ch, 
                                uint16_t u16AdcmpVal)
{
    volatile uint16_t*   pu16AdcmpAcpmReg;
	
    // Check for NULL pointer and channel parameter
    if ((pstcMftAdcmp == NULL)||(u8Ch > MFT_ADCMP_CH_MAX))
    {
        return ErrorInvalidParameter ;
    }
    pu16AdcmpAcpmReg = (volatile uint16_t*)(&pstcMftAdcmp->ACMP0 + 0x4*u8Ch);
    *pu16AdcmpAcpmReg =  u16AdcmpVal;
	
    return Ok;
}
/**
 ******************************************************************************
 ** \brief Read compare and offset value of ACMP
 **
 ** \param [in] pstcMftAdcmp Pointer to Mft instance
 ** \param [in] u8Ch         Mft Adcmp channel
 ** \arg        MFT_ADCMP_CH0 ADCMP channel 0
 ** \arg        MFT_ADCMP_CH1 ADCMP channel 1
 ** \arg        MFT_ADCMP_CH2 ADCMP channel 2
 ** \arg        MFT_ADCMP_CH3 ADCMP channel 3
 ** \arg        MFT_ADCMP_CH4 ADCMP channel 4
 ** \arg        MFT_ADCMP_CH5 ADCMP channel 5
 **
 ** \return  Value of register ACMP
 **
 ******************************************************************************/
uint16_t Mft_Adcmp_ReadAcmp(volatile stc_mftn_adcmp_t *pstcMftAdcmp, uint8_t u8Ch)
{
    volatile uint16_t*   pu16AdcmpAcpmReg ;
    uint16_t u16Data;
	
    // Check for NULL pointer and channel parameter
    if ((pstcMftAdcmp == NULL)||(u8Ch > MFT_ADCMP_CH_MAX))
    {
        return ErrorInvalidParameter ;
    }  
    pu16AdcmpAcpmReg =  (volatile uint16_t*)(&pstcMftAdcmp->ACMP0 + 0x4*u8Ch);
    u16Data = *pu16AdcmpAcpmReg;
    
    // return data value of ACMP register
    return u16Data;
}

/**
 ******************************************************************************
 ** \brief Mft Adcmp fm3 compatible initialization
 **
 ** \param [in] pstcMftAdcmp Pointer to Mft instance
 ** \param [in] u8CoupleCh   Mft Adcmp couple channel
 ** \arg        MFT_ADCMP_CH10  The couple channel of ch.0 and ch.1
 ** \arg        MFT_ADCMP_CH32  The couple channel of ch.2 and ch.3
 ** \arg        MFT_ADCMP_CH54  The couple channel of ch.4 and ch.5
 ** \param [in] pstcConfig   Mft Adcmp configuration parameter
 **
 ** \retval Ok                      Mft Adcmp fm3 compatible mode initialized
 ** \retval ErrorInvalidParameter   Invalid setting of pstcConfig elements
 ******************************************************************************/
en_result_t Mft_Adcmp_Fm3_Init(volatile stc_mftn_adcmp_t *pstcMftAdcmp, 
                               uint8_t u8CoupleCh,
                               stc_mft_adcmp_fm3_config_t *pstcConfig)
{
    volatile stc_mft_adcmp_acsa_field_t*   pu16ACSA ;
    volatile stc_mft_adcmp_acsc0_field_t*   pu16ACSC ;
    volatile stc_mft_adcmp_acfs10_field_t* pu8ACFS10 ;
    volatile stc_mft_adcmp_acfs32_field_t* pu8ACFS32 ;
    volatile stc_mft_adcmp_acfs54_field_t* pu8ACFS54 ;

    // Get actual address of register list of current channel
    pu16ACSA = (volatile stc_mft_adcmp_acsa_field_t*)(&pstcMftAdcmp->ACSA);
    pu16ACSC = (volatile stc_mft_adcmp_acsc0_field_t*)(&pstcMftAdcmp->ACSC0 + 0x4*u8CoupleCh);
    pu8ACFS10 = (volatile stc_mft_adcmp_acfs10_field_t*)(&pstcMftAdcmp->ACFS10);
    pu8ACFS32 = (volatile stc_mft_adcmp_acfs32_field_t*)(&pstcMftAdcmp->ACFS32);
    pu8ACFS54 = (volatile stc_mft_adcmp_acfs54_field_t*)(&pstcMftAdcmp->ACFS54);

    switch(u8CoupleCh)
    {
        case MFT_ADCMP_CH10:            
            pu8ACFS10->FSA0 = (uint8_t)pstcConfig->enFrt;
            pu8ACFS10->FSA1 = (uint8_t)pstcConfig->enFrt;
            break;
        case MFT_ADCMP_CH32:
            pu8ACFS32->FSA0 = (uint8_t)pstcConfig->enFrt;
            pu8ACFS32->FSA1 = (uint8_t)pstcConfig->enFrt;
            break;
        case MFT_ADCMP_CH54:
            pu8ACFS54->FSA0 = (uint8_t)pstcConfig->enFrt;
            pu8ACFS54->FSA1 = (uint8_t)pstcConfig->enFrt;
            break;
        default:
            return ErrorInvalidParameter;
    }

    // configure buffer transfer type
    if(pstcConfig->enBuf > AdcmpBufFrtZeroPeak)
    {
        return ErrorInvalidParameter;
    }
    
    pu16ACSC->BUFE = (uint8_t)pstcConfig->enBuf;

    // configure start trigger output channel number
    if(pstcConfig->enTrigSel > AdcmpTrigAdc2Prio)
    {
        return ErrorInvalidParameter;
    }
    
    pu16ACSC->ADSEL = (uint8_t)pstcConfig->enTrigSel;

    // configure Adcmp Fm3 trig mode
    switch(u8CoupleCh)
    {
        case MFT_ADCMP_CH10:            
            pu16ACSA->CE10 = 0;
            pu16ACSA->SEL10 = (uint8_t)pstcConfig->enMode;
            break;
        case MFT_ADCMP_CH32:
            pu16ACSA->CE32 = 0;
            pu16ACSA->SEL32 = (uint8_t)pstcConfig->enMode;
            break;
        case MFT_ADCMP_CH54:
            pu16ACSA->CE54 = 0;
            pu16ACSA->SEL54 = (uint8_t)pstcConfig->enMode;
            break;
        default:
            return ErrorInvalidParameter;
    }
    return Ok;
}
/**
 ******************************************************************************
 ** \brief Mft Adcmp fm3 compatible mode configuration and enable operation
 **
 ** \param [in] pstcMftAdcmp Pointer to Mft instance
 ** \param [in] u8CoupleCh   Mft Adcmp channel
 ** \arg        MFT_ADCMP_CH10  The couple channel of ch.0 and ch.1
 ** \arg        MFT_ADCMP_CH32  The couple channel of ch.2 and ch.3
 ** \arg        MFT_ADCMP_CH54  The couple channel of ch.4 and ch.5
 ** \retval Ok                      ADCMP FM3 mode funcitons enabled
 ** \retval ErrorInvalidParameter   If one of following conditions are met:
 **                                 - pstcMftAdcmp == NULL
 **                                 - u8CoupleCh > MFT_ADCMP_CPCH_MAX
 ******************************************************************************/
en_result_t 
Mft_Adcmp_Fm3_EnableOperation(volatile stc_mftn_adcmp_t *pstcMftAdcmp, 
                              uint8_t u8CoupleCh)
{
    volatile stc_mft_adcmp_acsa_field_t*   pu16ACSA ;

    // Check for NULL pointer and channel parameter
    if ((pstcMftAdcmp == NULL)||(u8CoupleCh > MFT_ADCMP_CPCH_MAX))
    {
        return ErrorInvalidParameter ;
    }
    // Get actual address of register list of current channel
    pu16ACSA = (volatile stc_mft_adcmp_acsa_field_t*)(&pstcMftAdcmp->ACSA);
    switch(u8CoupleCh)
    {
        case MFT_ADCMP_CH10:
            pu16ACSA->CE10 = 1;
            break;
        case MFT_ADCMP_CH32:
            pu16ACSA->CE32 = 1;
            break;
        case MFT_ADCMP_CH54:
            pu16ACSA->CE54 = 1;
            break;
        default:
            return ErrorInvalidParameter;
    }
    return Ok;
}
/**
 ******************************************************************************
 ** \brief De-configurate Mft Adcmp fm3 compatible function and disable operation
 **
 ** \param [in] pstcMftAdcmp Pointer to Mft instance
 ** \param [in] u8CoupleCh   Mft Adcmp channel
 ** \arg        MFT_ADCMP_CH10  The couple channel of ch.0 and ch.1
 ** \arg        MFT_ADCMP_CH32  The couple channel of ch.2 and ch.3
 ** \arg        MFT_ADCMP_CH54  The couple channel of ch.4 and ch.5
 **
 ** \retval Ok                      ADCMP FM3 operation disabled
 ** \retval ErrorInvalidParameter   If one of followings conditions are met:
 **                                 - pstcMftAdcmp == NULL
 **                                 - u8CoupleCh > MFT_ADCMP_CPCH_MAX
 **
 ******************************************************************************/
en_result_t 
Mft_Adcmp_Fm3_DisableOperation(volatile stc_mftn_adcmp_t *pstcMftAdcmp, 
                               uint8_t u8CoupleCh)
{
    volatile stc_mft_adcmp_acsa_field_t*   pu16ACSA;

    // Check for NULL pointer and channel parameter
    if ((pstcMftAdcmp == NULL)||(u8CoupleCh > MFT_ADCMP_CPCH_MAX))
    {
        return ErrorInvalidParameter ;
    }
    // Get actual address of register list of current channel
    pu16ACSA = (volatile stc_mft_adcmp_acsa_field_t*)(&pstcMftAdcmp->ACSA);
    switch(u8CoupleCh)
    {
        case MFT_ADCMP_CH10:
            pu16ACSA->CE10 = 0;
            break;
        case MFT_ADCMP_CH32:
            pu16ACSA->CE32 = 0;
            break;
        case MFT_ADCMP_CH54:
            pu16ACSA->CE54 = 0;
            break;
        default:
            return ErrorInvalidParameter;
    }
    return Ok;
}
/**
 ******************************************************************************
 ** \brief Mft Adcmp fm3 compatible mode, write Accp register
 **
 ** \param [in] pstcMftAdcmp Pointer to Mft instance
 ** \param [in] u8CoupleCh   Mft Adcmp channel
 ** \arg        MFT_ADCMP_CH10  The couple channel of ch.0 and ch.1
 ** \arg        MFT_ADCMP_CH32  The couple channel of ch.2 and ch.3
 ** \arg        MFT_ADCMP_CH54  The couple channel of ch.4 and ch.5
 ** \param [in] u16AccpVal      Mft Adcmp configuration parameter
 **
 ** \retval  Ok                      Accp is written
 ** \retval  ErrorInvalidParameter   If one of following conditions are met:
 **                                  - pstcMftAdcmp == NULL
 **                                  - u8CoupleCh > MFT_ADCMP_CPCH_MAX
 ******************************************************************************/
en_result_t Mft_Adcmp_Fm3_WriteAccp(volatile stc_mftn_adcmp_t *pstcMftAdcmp, 
                                    uint8_t u8CoupleCh, uint16_t u16AccpVal)
{
    volatile uint16_t*   u16AdcmpAcpmReg;

    // Check for NULL pointer and channel parameter
    if ((pstcMftAdcmp == NULL)||(u8CoupleCh > MFT_ADCMP_CPCH_MAX))
    {
        return ErrorInvalidParameter ;
    }
    u16AdcmpAcpmReg = (volatile uint16_t*)(&pstcMftAdcmp->ACMP0 + 0x4*u8CoupleCh);
    *u16AdcmpAcpmReg =  u16AccpVal;

    return Ok;
}
/**
 ******************************************************************************
 ** \brief Mft Adcmp fm3 compatible mode, read Accp register stored value
 **
 ** \param [in] pstcMftAdcmp Pointer to Mft instance
 ** \arg        structure of Mft_Adcmp
 ** \param [in] u8CoupleCh   Mft Adcmp channel
 ** \arg        MFT_ADCMP_CH10  The couple channel of ch.0 and ch.1
 ** \arg        MFT_ADCMP_CH32  The couple channel of ch.2 and ch.3
 ** \arg        MFT_ADCMP_CH54  The couple channel of ch.4 and ch.5
 ** \return     Data stored in Accp register
 **
 ******************************************************************************/
uint16_t Mft_Adcmp_Fm3_ReadAccp(volatile stc_mftn_adcmp_t *pstcMftAdcmp, 
                                uint8_t u8CoupleCh)
{
    volatile uint16_t*   pu16AdcmpAcpmReg;
    uint16_t u16Data;

    // Check for NULL pointer and channel parameter
    if ((pstcMftAdcmp == NULL)||(u8CoupleCh > MFT_ADCMP_CPCH_MAX))
    {
        return ErrorInvalidParameter ;
    }
    pu16AdcmpAcpmReg = (volatile uint16_t*)(&pstcMftAdcmp->ACMP0 + 0x4*u8CoupleCh);
    u16Data = *pu16AdcmpAcpmReg;
    // return data value of ACMP register
    return u16Data;
}
/**
 ******************************************************************************
 ** \brief Mft Adcmp fm3 compatible mode, write Accpdn register
 **
 ** \param [in] pstcMftAdcmp Pointer to Mft instance
 ** \arg        structure of Mft_Adcmp
 ** \param [in] u8CoupleCh   Mft Adcmp channel
 ** \arg        MFT_ADCMP_CH10  The couple channel of ch.0 and ch.1
 ** \arg        MFT_ADCMP_CH32  The couple channel of ch.2 and ch.3
 ** \arg        MFT_ADCMP_CH54  The couple channel of ch.4 and ch.5
 ** \param [in] u16AccpdnVal    data value of Accpdn
 **
 ** \retval  Ok                      Accpdn is written
 ** \retval  ErrorInvalidParameter   If one of following conditions are met:
 **                                  - pstcMftAdcmp == NULL
 **                                  - u8CoupleCh > MFT_ADCMP_CPCH_MAX
 ******************************************************************************/
en_result_t Mft_Adcmp_Fm3_WriteAccpdn(volatile stc_mftn_adcmp_t *pstcMftAdcmp, 
                                      uint8_t u8CoupleCh, 
                                      uint16_t u16AccpdnVal)
{
    volatile uint16_t*   pu16AdcmpAcpmReg;

    // Check for NULL pointer and channel parameter
    if ((pstcMftAdcmp == NULL)||(u8CoupleCh > MFT_ADCMP_CPCH_MAX))
    {
        return ErrorInvalidParameter ;
    }
    pu16AdcmpAcpmReg = (volatile uint16_t*)(&pstcMftAdcmp->ACMP1 + 0x4*u8CoupleCh);
    *pu16AdcmpAcpmReg =  u16AccpdnVal;

    return Ok;
}
/**
 ******************************************************************************
 ** \brief Mft Adcmp fm3 compatible mode, read Accpdn register stored value
 **
 ** \param [in] pstcMftAdcmp Pointer to Mft instance
 ** \arg        structure of Mft_Adcmp
 ** \param [in] u8CoupleCh   Mft Adcmp channel
 ** \arg        MFT_ADCMP_CH10  The couple channel of ch.0 and ch.1
 ** \arg        MFT_ADCMP_CH32  The couple channel of ch.2 and ch.3
 ** \arg        MFT_ADCMP_CH54  The couple channel of ch.4 and ch.5
 ** \return    Data stored in Accpdn register
 **
 ******************************************************************************/
uint16_t Mft_Adcmp_Fm3_ReadAccpdn(volatile stc_mftn_adcmp_t *pstcMftAdcmp, 
                                  uint8_t u8CoupleCh)
{
    volatile uint16_t*   pu16AdcmpAcpmReg;
    uint16_t u16Data;

    // Check for NULL pointer and channel parameter
    if ((pstcMftAdcmp == NULL)||(u8CoupleCh > MFT_ADCMP_CPCH_MAX))
    {
        return ErrorInvalidParameter ;
    }
    pu16AdcmpAcpmReg = ((volatile uint16_t*)&pstcMftAdcmp->ACMP1 + 0x4*u8CoupleCh);
    u16Data = *pu16AdcmpAcpmReg;
    
    // return data value of ACMP register
    return u16Data;
}

#endif
//@} // Mft_adcmp Group

