/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/

/******************************************************************************/
/** \file pdl_header.h
 **
 ** Peripheral header files for FM0+ PDL.
 **
 ** \note The application program only needs include this file to use FM0+ PDL.
 **
 ** History:
 **   - 2013-12-09  0.1  Edison Zhang  First version.
 **
 ******************************************************************************/

#ifndef __PDL_HEADER_H__
#define __PDL_HEADER_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mcu.h"
#include "stdio.h"
#include "pdl_user.h"

/* C binding of definitions if building with C++ compiler                     */
#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************
 * Peripheral driver header files inclusion
 ******************************************************************************/
// Include adc.h if ADC is active
#if defined(PDL_PERIPHERAL_ADC_ACTIVE)
    #include "adc\adc.h"
#endif

// Include bt.h if BT is active
#if defined(PDL_PERIPHERAL_BT_ACTIVE)
    #include "bt\bt.h"
#endif

// Include clk.h if clock is active
#if defined(PDL_PERIPHERAL_CLK_ACTIVE)
    #include "clk\clk.h"
#endif

// Include cr.h if CR is active
#if defined(PDL_PERIPHERAL_CR_ACTIVE)
    #include "cr\cr.h"
#endif

// Include csv.h if CSV is active
#if defined(PDL_PERIPHERAL_CSV_ACTIVE)
    #include "csv\csv.h"
#endif

// Include dma.h if DMA is active
#if defined(PDL_PERIPHERAL_DMA_ACTIVE)
    #include "dma\dma.h"
#endif

// Include dt.h if DT is active
#if defined(PDL_PERIPHERAL_DT_ACTIVE)
    #include "dt\dt.h"
#endif

// Include exint.h if external interrupt is active
#if defined(PDL_PERIPHERAL_EXINT_ACTIVE) || defined(PDL_PERIPHERAL_NMI_ACTIVE)
    #include "exint\exint.h"
#endif

// Include flash.h if Flash is active
#if defined(PDL_PERIPHERAL_FLASH_ACTIVE)
    #include "flash\flash.h"
#endif

// Include gpio.h and fgpio.h if GPIO is active
#if defined(PDL_PERIPHERAL_GPIO_ACTIVE)
    #include "gpio\gpio.h"
    #include "gpio\fgpio.h"
#endif

// Include lpm.h if LPM is active
#if defined(PDL_PERIPHERAL_LPM_ACTIVE)
    #include "lpm\lpm.h"
#endif

// Include lvd.h if LVD is active
#if defined(PDL_PERIPHERAL_LVD_ACTIVE)
    #include "lvd\lvd.h"
#endif

// Include mfs.h if MFS is active
#if defined(PDL_PERIPHERAL_MFS_ACTIVE)
    #include "mfs\mfs.h"
#endif

// Include mft_frt.h if MFT'FRT is active
#if defined(PDL_PERIPHERAL_MFT_FRT_ACTIVE)
    #include "mft\mft_frt.h"
#endif

// Include mft_ocu.h if MFT'OCU is active
#if defined(PDL_PERIPHERAL_MFT_OCU_ACTIVE)
    #include "mft\mft_ocu.h"
#endif

// Include mft_wfg.h if MFT'WFG is active
#if defined(PDL_PERIPHERAL_MFT_WFG_ACTIVE)
    #include "mft\mft_wfg.h"
#endif

// Include mft_icu.h if MFT'ICU is active
#if defined(PDL_PERIPHERAL_MFT_ICU_ACTIVE)
    #include "mft\mft_icu.h"
#endif

// Include mft_adcmp.h if MFT'ADCMP is active
#if defined(PDL_PERIPHERAL_MFT_ADCMP_ACTIVE)
    #include "mft\mft_adcmp.h"
#endif

// Include ppg.h if MFT'PPG is active
#if defined(PDL_PERIPHERAL_PPG_ACTIVE)
    #include "ppg\ppg.h"
#endif

// Include qprc.h if QPRC is active
#if defined(PDL_PERIPHERAL_QPRC_ACTIVE)
    #include "qprc\qprc.h"
#endif

// Include reset.h if reset is active
#if defined(PDL_PERIPHERAL_RESET_ACTIVE)
    #include "reset\reset.h"
#endif

// Include rtc.h if RTC is active
#if defined(PDL_PERIPHERAL_RTC_ACTIVE)
    #include "rtc\rtc.h"
#endif

// Include uid.h if UID is active
#if defined(PDL_PERIPHERAL_UID_ACTIVE)
    #include "uid\uid.h"
#endif

// Include wc.h if WC is active
#if defined(PDL_PERIPHERAL_WC_ACTIVE)
    #include "wc\wc.h"
#endif

// Include wdg.h if WDG is active
#if defined(PDL_PERIPHERAL_WDG_ACTIVE)
    #include "wdg\wdg.h"
#endif

#if (PDL_UTILITY_ENABLE_I2C_POLLING_AT24CXX == PDL_ON) || \
    (PDL_UTILITY_ENABLE_I2C_INT_AT24CXX == PDL_ON)
    #include "at24cxx\i2c_at24cxx.h"
#endif

#if (PDL_UTILITY_ENABLE_CSIO_INT_S25FL127S == PDL_ON)
    #include "s25fl127s\csio_int_s25fl127s.h"
#endif

#if (PDL_UTILITY_ENABLE_UART_PRINTF == PDL_ON)
    #include "printf\uart_printf.h"
#endif

#ifdef __cplusplus
}
#endif

#endif  /* __PDL_HEADER_H__ */

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
