/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file wc.c
 **
 ** A detailed description is available at 
 ** @link WcGroup WC Module description @endlink
 **
 ** History:
 **   - 2014-02-17  0.1  Edison Zhang  First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "wc.h"

#if (defined(PDL_PERIPHERAL_WC_ACTIVE))

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_WC == PDL_ON)
static func_ptr_t  pWcfnIntCallback = NULL;
#endif

#if (PDL_INTERRUPT_ENABLE_WC == PDL_ON)
/******************************************************************************/
/* Local Functions                                                            */
/******************************************************************************/
/**
 ******************************************************************************
 ** \brief Set NVIC Interrupt depending on ADC instance
 ******************************************************************************/
static void Wc_InitNvic(void)
{
    NVIC_ClearPendingIRQ(OSC_PLL_WC_RTC_IRQn);
    NVIC_EnableIRQ(OSC_PLL_WC_RTC_IRQn);
    NVIC_SetPriority(OSC_PLL_WC_RTC_IRQn, PDL_IRQ_LEVEL_CLK_WC_RTC);   
}

/**
 ******************************************************************************
 ** \brief Clear NVIC Interrupt depending on ADC instance
 ******************************************************************************/
static void Wc_DeInitNvic(void)
{
    NVIC_ClearPendingIRQ(OSC_PLL_WC_RTC_IRQn);
    NVIC_DisableIRQ(OSC_PLL_WC_RTC_IRQn);
    NVIC_SetPriority(OSC_PLL_WC_RTC_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);   
}
/*!
 ******************************************************************************
 ** \brief  Watch counter interrupt function
 ******************************************************************************/
void Wc_IrqHandler(void)
{
    if (PdlSet == Wc_GetIntFlag())
    {
        Wc_ClearIntFlag();
        if (pWcfnIntCallback != NULL)
        {
            pWcfnIntCallback();
        }
    }
}
#endif

/**
 ******************************************************************************
 ** \brief Select the input  clock an and set the division clock to be output.
 **
 ** The Function can set SEL_OUT, SEL_IN of Watch counter prescaler (hereafter WCP)
 **
 ** \param [in]  pstcWcPresClk     WC prescaler clock configuration
 **
 ** \retval Ok                     Write data successfully done or started.
 ** \retval ErrorInvalidParameter  If one of following conditions are met:
 **                                - invalid value of pstcWcPresClk->enInputClk
 **                                - invalid value of pstcWcPresClk->enOutputClk
 **
 ******************************************************************************/
en_result_t Wc_Pres_SelClk(stc_wc_pres_clk_t* pstcWcPresClk)
{
    switch (pstcWcPresClk->enInputClk)
    {
        case WcPresInClkSubOsc:
        case WcPresInClkMainOsc:
        case WcPresInClkHighCr:
        case WcPresInClkLowCr:
            FM0P_WC->CLK_SEL_f.SEL_IN = pstcWcPresClk->enInputClk;
            break;
        default:
            return ErrorInvalidParameter;
    }

    switch(pstcWcPresClk->enOutputClk)
    {
        case WcPresOutClkArray0:
        case WcPresOutClkArray1:
        case WcPresOutClkArray2:
        case WcPresOutClkArray3:
        case WcPresOutClkArray4:
        case WcPresOutClkArray5:
        case WcPresOutClkArray6:
            FM0P_WC->CLK_SEL_f.SEL_OUT = pstcWcPresClk->enOutputClk;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Enable oscillation of the division clock.
 **
 ** The Function can set CLK_EN:CLK_EN to 1 of WCP
 **
 ** \retval Ok                  Enable oscillation of the division clock normally
 **
 ******************************************************************************/
en_result_t Wc_Pres_EnableDiv()
{
    FM0P_WC->CLK_EN_f.CLK_EN = 1;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable oscillation of the division clock.
 **
 ** The Function can clear CLK_EN:CLK_EN to 0 of WCP
 **
 ** \retval Ok             Disable oscillation of the division clock normally
 **
 ******************************************************************************/
en_result_t Wc_Pres_DisableDiv()
{
    FM0P_WC->CLK_EN_f.CLK_EN = 0;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get the operation status of the division counter.
 **
 ** The Function can get CLK_EN:CLK_EN_R to 0 of WCP
 **
 ** \retval PdlClr               CLK_EN_R is 0, oscillation of the division clock is not performed
 ** \retval PdlSet               CLK_EN R is 1, oscillation of the division clock is performed
 **
 ******************************************************************************/
en_stat_flag_t Wc_GetDivStat()
{
    return ((en_stat_flag_t)(FM0P_WC->CLK_EN_f.CLK_EN_R));
}

/**
 ******************************************************************************
 ** \brief Select the input clock and set the division clock to be output.
 **
 ** The Function can set SEL_OUT, SEL_IN of Watch counter prescaler (hereafter WCP)
 **
 ** \param [in]  pstcWcConfig      WC configuration
 **
 ** \retval Ok                     Write data successfully done or started.
 ** \retval ErrorInvalidParameter  If one of following conditions are met:
 **                                - invalid value of pstcWcConfig->enCntClk
 **
 ******************************************************************************/
en_result_t Wc_Init(stc_wc_config_t* pstcWcConfig)
{
    // Check for NULL pointers
    if (NULL == pstcWcConfig)
    {
        return ErrorInvalidParameter;
    }
    
    switch (pstcWcConfig->enCntClk)
    {
        case WcCntClkWcck0:
        case WcCntClkWcck1:
        case WcCntClkWcck2:
        case WcCntClkWcck3:
            FM0P_WC->WCCR_f.CS = pstcWcConfig->enCntClk;
            break;
        default:
           return ErrorInvalidParameter;
    }

    FM0P_WC->WCRL = pstcWcConfig->u8ReloadValue;
    
    return Ok;
}


/**
 ******************************************************************************
 ** \brief Enable WC operation.
 **
 ** The Function can set WCCR:WCEN to 1 of WC
 **
 ** \retval Ok                    Enable WC operation normally
 **
 ******************************************************************************/
en_result_t Wc_EnableCount(void)
{
    FM0P_WC->WCCR_f.WCEN = 1;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable WC operation.
 **
 ** The Function can clear WCCR:WCEN to 0 of WC
 **
 ** \retval Ok                Disable WC operation normally
 **
 ******************************************************************************/
en_result_t Wc_DisableCount(void)
{
    FM0P_WC->WCCR_f.WCEN = 0;
    return Ok;
}

#if (PDL_INTERRUPT_ENABLE_WC == PDL_ON)
/**
 ******************************************************************************
 ** \brief Enable WC underflow interrupt.
 **
 ** The Function can set WCIE of WC
 **
 ** \param [in]  pfnIntCallback     WC interrupt to be enabled
 **
 ** \retval Ok                      Write data successfully done or started.
 ** \retval ErrorInvalidParameter   If one of following conditions are met:
 **                                 - pfnIntCallback == NULL
 **
 ******************************************************************************/
en_result_t Wc_EnableInt(func_ptr_t  pfnIntCallback)
{
    // check for non-null pointers
    if (NULL == pfnIntCallback)
    {
        return ErrorInvalidParameter;
    }
    FM0P_WC->WCCR_f.WCIE = 1;
    pWcfnIntCallback = pfnIntCallback;
    Wc_InitNvic();
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable WC underflow interrupt.
 **
 ** The Function can clear WCIE of WC
 **
 ** \retval Ok                 Disable WC underflow interrupt normally
 **
 ******************************************************************************/
en_result_t Wc_DisableInt(void)
{
    FM0P_WC->WCCR_f.WCIE = 0;
    pWcfnIntCallback = NULL;
    
    /* When RTC interrupt is enabled, don't disable IRQ */
    if((FM0P_RTC->WTCR1 & 0xFF000000u) != 0) 
    {
        return Ok;
    }
    
    Wc_DeInitNvic();
    return Ok;
}
#endif
/**
 ******************************************************************************
 ** \brief Set the counter value of WC
 **
 ** The Function can set value to WCRL of WC
 **
 ** \retval Ok             Set the counter value of WC normally
 **
 ******************************************************************************/
en_result_t Wc_WriteReloadVal(uint8_t u8Val)
{
    FM0P_WC->WCRL = u8Val;
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Read the value in the 6-bit down counter
 **
 ** The Function can read value of WCRD of WC
 **
 ** \retval Current count value of WC
 **
 ******************************************************************************/
uint8_t Wc_ReadCurCnt()
{
    return (FM0P_WC->WCRD);
}

/**
 ******************************************************************************
 ** \brief Clear WC underflow flag
 **
 ** The Function clear WCIF to 0 of WC
 **
 ** \retval Ok               Clear WC underflow flag normally
 **
 ******************************************************************************/
en_result_t Wc_ClearIntFlag()
{
    FM0P_WC->WCCR_f.WCIF = 0;
    return Ok;    
}

/**
 ******************************************************************************
 ** \brief Get WC underflow flag status
 **
 ** The Function get value WCIF of WC
 **
 ** \retval PdlClr               WCIF is 0, WC underflow does not occur
 ** \retval PdlSet               WCIF is 1, WC underflow occurs
 **
 ******************************************************************************/
en_int_flag_t Wc_GetIntFlag()
{
    return ((en_int_flag_t)(FM0P_WC->WCCR_f.WCIF));
}

/**
 ******************************************************************************
 ** \brief Get WC operation state
 **
 ** The Function get value WCOP of WC
 **
 ** \retval PdlClr               WCOP is 0, The WC is stopped
 ** \retval PdlSet               WCOP is 1, The WC is active
 **
 ******************************************************************************/
en_stat_flag_t Wc_GetOperationFlag()
{
    return ((en_stat_flag_t)(FM0P_WC->WCCR_f.WCOP));
}


#endif // #if (defined(PDL_PERIPHERAL_WC_ACTIVE))

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/

