/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file mfs.c
 **
 ** A detailed description is available at 
 ** @link MfsGroup MFS Module description @endlink
 **
 ** History:
 **   - 2014-02-13  1.0  Edison Zhang  First version.
 **   - 2014-03-11  1.1  Edison Zhang  Add instance index variable in the 
 **                                    internal data structure.
 **   - 2014-05-14  1.2  Edison Zhang  Update some comments.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mfs.h"

#if (defined(PDL_PERIPHERAL_MFS_ACTIVE))

/**
 ******************************************************************************
 ** \addtogroup MfsGroup
 ******************************************************************************/
//@{

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/
#define UartInstanceToIndex(Instance) ((uint32_t)Instance - (uint32_t)&UART0)/0x100u
#define CsioInstanceToIndex(Instance) ((uint32_t)Instance - (uint32_t)&CSIO0)/0x100u
#define I2cInstanceToIndex(Instance)  ((uint32_t)Instance - (uint32_t)&I2C0)/0x100u
#define LinInstanceToIndex(Instance)  ((uint32_t)Instance - (uint32_t)&LIN0)/0x100u


/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/// Look-up table for all enabled MFS instances and their internal data
stc_mfs_instance_data_t m_astcMfsInstanceDataLut[MFS_INSTANCE_COUNT] =
{
#if (PDL_PERIPHERAL_ENABLE_MFS0 == PDL_ON)
    {
        {&UART0, &CSIO0, &I2C0, &LIN0}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs0,   /* MFS0 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        }  
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS1 == PDL_ON)
    {
        {&UART1, &CSIO1, &I2C1, &LIN1}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs1,   /* MFS1 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        }  
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS2 == PDL_ON)
    {
        {&UART2, &CSIO2, &I2C2, &LIN2}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs2,   /* MFS2 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS3 == PDL_ON)
    {
        {&UART3, &CSIO3, &I2C3, &LIN3}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs3,   /* MFS3 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
            
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS4 == PDL_ON)
    {
        {&UART4, &CSIO4, &I2C4, &LIN4}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs4,   /* MFS4 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS5 == PDL_ON)
    {
        {&UART5, &CSIO5, &I2C5, &LIN5}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs5,   /* MFS5 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS6 == PDL_ON)
    {
        {&UART6, &CSIO6, &I2C6, &LIN6}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs6,   /* MFS6 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS7 == PDL_ON)
    {
        {&UART7, &CSIO7, &I2C7, &LIN7}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs7,   /* MFS7 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS8 == PDL_ON)
    {
        {&UART8, &CSIO8, &I2C8, &LIN8}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs8,   /* MFS8 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS9 == PDL_ON)
    {
        {&UART9, &CSIO9, &I2C9, &LIN9}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs9,   /* MFS9 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS10 == PDL_ON)
    {
        {&UART10, &CSIO10, &I2C10, &LIN10}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs10,   /* MFS10 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS11 == PDL_ON)
    {
        {&UART11, &CSIO11, &I2C11, &LIN11}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs11,  /* MFS11 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS12 == PDL_ON)
    {
        {&UART12, &CSIO12, &I2C12, &LIN12}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs12,  /* MFS12 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS13 == PDL_ON)
    {
        {&UART13, &CSIO13, &I2C13, &LIN13}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs13,  /* MFS13 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS14 == PDL_ON)
    {
        {&UART14, &CSIO14, &I2C14, &LIN14}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs14,  /* MFS14 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFS15 == PDL_ON)
    {
        {&UART15, &CSIO15, &I2C15, &LIN15}, /* pstcInstance */
        {
            MfsInitMode,            /* MFS initial mode */
            MfsInstanceIndexMfs15,  /* MFS15 instance index */
            NULL,                   /* stcInternData (not initialized yet) */
        } 
    },
#endif
};

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/
static stc_mfs_intern_data_t* MfsUartGetInternDataPtr(volatile stc_mfsn_uart_t* pstcMfs);
static stc_mfs_intern_data_t* MfsCsioGetInternDataPtr(volatile stc_mfsn_csio_t* pstcMfs);
static stc_mfs_intern_data_t* MfsI2cGetInternDataPtr(volatile stc_mfsn_i2c_t* pstcMfs);
static stc_mfs_intern_data_t* MfsLinGetInternDataPtr(volatile stc_mfsn_lin_t* pstcMfs);

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON) ||  \
    (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON) ||  \
    (PDL_INTERRUPT_ENABLE_MFS2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS10 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS3 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS11 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS4 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS12 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS5 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS13 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS6 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS14 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS7 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS15 == PDL_ON) 
/**
 ******************************************************************************
 ** \brief Device dependent initialization of transmit interrupts
 **        according CMSIS with level defined in pdl.h
 **
 ** \param enIndex MFS instance index
 ** \param bRxTx  FALSE: RX IRQ, TRUE: TX IRQ     
 **
 ******************************************************************************/
static void Mfs_InitIrq(en_mfs_instance_index_t enIndex, boolean_t bRxTx)
{
#if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs0)
    {
        if(bRxTx == FALSE)
        {
            /* MFS0/8 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS0_8_RX_IRQn);
            NVIC_EnableIRQ(MFS0_8_RX_IRQn);
            NVIC_SetPriority(MFS0_8_RX_IRQn, PDL_IRQ_LEVEL_MFS0_8_RX);
        }
        else
        {
            /* MFS0/8 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS0_8_TX_IRQn);
            NVIC_EnableIRQ(MFS0_8_TX_IRQn);
            NVIC_SetPriority(MFS0_8_TX_IRQn, PDL_IRQ_LEVEL_MFS0_8_TX);
        }
    }
#endif
    
#if (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs8)
    {
        if(bRxTx == FALSE)
        {
            /* MFS0/8 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS0_8_RX_IRQn);
            NVIC_EnableIRQ(MFS0_8_RX_IRQn);
            NVIC_SetPriority(MFS0_8_RX_IRQn, PDL_IRQ_LEVEL_MFS0_8_RX);
        }
        else
        {
            /* MFS0/8 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS0_8_TX_IRQn);
            NVIC_EnableIRQ(MFS0_8_TX_IRQn);
            NVIC_SetPriority(MFS0_8_TX_IRQn, PDL_IRQ_LEVEL_MFS0_8_TX);
        }
    }
#endif    
    
#if (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs1)
    {
        if(bRxTx == FALSE)
        {
            /* MFS1/9 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS1_9_RX_IRQn);
            NVIC_EnableIRQ(MFS1_9_RX_IRQn);
            NVIC_SetPriority(MFS1_9_RX_IRQn, PDL_IRQ_LEVEL_MFS1_9_RX);
        }
        else
        {
            /* MFS1/9 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS1_9_TX_IRQn);
            NVIC_EnableIRQ(MFS1_9_TX_IRQn);
            NVIC_SetPriority(MFS1_9_TX_IRQn, PDL_IRQ_LEVEL_MFS1_9_TX);
        }
    }
#endif
    
#if (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs9)
    {
        if(bRxTx == FALSE)
        {
            /* MFS1/9 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS1_9_RX_IRQn);
            NVIC_EnableIRQ(MFS1_9_RX_IRQn);
            NVIC_SetPriority(MFS1_9_RX_IRQn, PDL_IRQ_LEVEL_MFS1_9_RX);
        }
        else
        {
            /* MFS1/9 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS1_9_TX_IRQn);
            NVIC_EnableIRQ(MFS1_9_TX_IRQn);
            NVIC_SetPriority(MFS1_9_TX_IRQn, PDL_IRQ_LEVEL_MFS1_9_TX);
        }
    }
#endif    

#if (PDL_INTERRUPT_ENABLE_MFS2  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs2)
    {
        if(bRxTx == FALSE)
        {
            /* MFS2/10 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS2_10_RX_IRQn);
            NVIC_EnableIRQ(MFS2_10_RX_IRQn);
            NVIC_SetPriority(MFS2_10_RX_IRQn, PDL_IRQ_LEVEL_MFS2_10_RX);
        }
        else
        {
            /* MFS2/10 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS2_10_TX_IRQn);
            NVIC_EnableIRQ(MFS2_10_TX_IRQn);
            NVIC_SetPriority(MFS2_10_TX_IRQn, PDL_IRQ_LEVEL_MFS2_10_TX);
        }
    }
#endif
    
#if (PDL_INTERRUPT_ENABLE_MFS10  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs10)
    {
        if(bRxTx == FALSE)
        {
            /* MFS2/10 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS2_10_RX_IRQn);
            NVIC_EnableIRQ(MFS2_10_RX_IRQn);
            NVIC_SetPriority(MFS2_10_RX_IRQn, PDL_IRQ_LEVEL_MFS2_10_RX);
        }
        else
        {
            /* MFS2/10 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS2_10_TX_IRQn);
            NVIC_EnableIRQ(MFS2_10_TX_IRQn);
            NVIC_SetPriority(MFS2_10_TX_IRQn, PDL_IRQ_LEVEL_MFS2_10_TX);
        }
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS3  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs3)
    {
        if(bRxTx == FALSE)
        {
            /* MFS3/11 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS3_11_RX_IRQn);
            NVIC_EnableIRQ(MFS3_11_RX_IRQn);
            NVIC_SetPriority(MFS3_11_RX_IRQn, PDL_IRQ_LEVEL_MFS3_11_RX);
        }
        else
        {
            /* MFS3/11 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS3_11_TX_IRQn);
            NVIC_EnableIRQ(MFS3_11_TX_IRQn);
            NVIC_SetPriority(MFS3_11_TX_IRQn, PDL_IRQ_LEVEL_MFS3_11_TX);
        }
    }
#endif
    
#if (PDL_INTERRUPT_ENABLE_MFS11  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs11)
    {
        if(bRxTx == FALSE)
        {
            /* MFS3/11 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS3_11_RX_IRQn);
            NVIC_EnableIRQ(MFS3_11_RX_IRQn);
            NVIC_SetPriority(MFS3_11_RX_IRQn, PDL_IRQ_LEVEL_MFS3_11_RX);
        }
        else
        {
            /* MFS3/11 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS3_11_TX_IRQn);
            NVIC_EnableIRQ(MFS3_11_TX_IRQn);
            NVIC_SetPriority(MFS3_11_TX_IRQn, PDL_IRQ_LEVEL_MFS3_11_TX);
        }
    }
#endif    

#if (PDL_INTERRUPT_ENABLE_MFS4  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs4)
    {
        if(bRxTx == FALSE)
        {
            /* MFS4/12 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS4_12_RX_IRQn);
            NVIC_EnableIRQ(MFS4_12_RX_IRQn);
            NVIC_SetPriority(MFS4_12_RX_IRQn, PDL_IRQ_LEVEL_MFS4_12_RX);
        }
        else
        {
            /* MFS4/12 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS4_12_TX_IRQn);
            NVIC_EnableIRQ(MFS4_12_TX_IRQn);
            NVIC_SetPriority(MFS4_12_TX_IRQn, PDL_IRQ_LEVEL_MFS4_12_TX);
        }
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS12  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs12)
    {
        if(bRxTx == FALSE)
        {
            /* MFS4/12 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS4_12_RX_IRQn);
            NVIC_EnableIRQ(MFS4_12_RX_IRQn);
            NVIC_SetPriority(MFS4_12_RX_IRQn, PDL_IRQ_LEVEL_MFS4_12_RX);
        }
        else
        {
            /* MFS4/12 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS4_12_TX_IRQn);
            NVIC_EnableIRQ(MFS4_12_TX_IRQn);
            NVIC_SetPriority(MFS4_12_TX_IRQn, PDL_IRQ_LEVEL_MFS4_12_TX);
        }
    }
#endif    
    
#if (PDL_INTERRUPT_ENABLE_MFS5  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs5)
    {
        if(bRxTx == FALSE)
        {
            /* MFS5/13 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS5_13_RX_IRQn);
            NVIC_EnableIRQ(MFS5_13_RX_IRQn);
            NVIC_SetPriority(MFS5_13_RX_IRQn, PDL_IRQ_LEVEL_MFS5_13_RX);
        }
        else
        {
            /* MFS5/13 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS5_13_TX_IRQn);
            NVIC_EnableIRQ(MFS5_13_TX_IRQn);
            NVIC_SetPriority(MFS5_13_TX_IRQn, PDL_IRQ_LEVEL_MFS5_13_TX);
        }
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS13  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs13)
    {
        if(bRxTx == FALSE)
        {
            /* MFS5/13 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS5_13_RX_IRQn);
            NVIC_EnableIRQ(MFS5_13_RX_IRQn);
            NVIC_SetPriority(MFS5_13_RX_IRQn, PDL_IRQ_LEVEL_MFS5_13_RX);
        }
        else
        {
            /* MFS5/13 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS5_13_TX_IRQn);
            NVIC_EnableIRQ(MFS5_13_TX_IRQn);
            NVIC_SetPriority(MFS5_13_TX_IRQn, PDL_IRQ_LEVEL_MFS5_13_TX);
        }
    }
#endif
    
#if (PDL_INTERRUPT_ENABLE_MFS6  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs6)
    {
        if(bRxTx == FALSE)
        {
            /* MFS6/14 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS6_14_RX_DMA0_IRQn);
            NVIC_EnableIRQ(MFS6_14_RX_DMA0_IRQn);
            NVIC_SetPriority(MFS6_14_RX_DMA0_IRQn, PDL_IRQ_LEVEL_MFS6_14_RX_DMA0);
        }
        else
        {
            /* MFS6/14 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS6_14_TX_DMA1_IRQn);
            NVIC_EnableIRQ(MFS6_14_TX_DMA1_IRQn);
            NVIC_SetPriority(MFS6_14_TX_DMA1_IRQn, PDL_IRQ_LEVEL_MFS6_14_TX_DMA1);
        }
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS14  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs14)
    {
        if(bRxTx == FALSE)
        {
            /* MFS6/14 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS6_14_RX_DMA0_IRQn);
            NVIC_EnableIRQ(MFS6_14_RX_DMA0_IRQn);
            NVIC_SetPriority(MFS6_14_RX_DMA0_IRQn, PDL_IRQ_LEVEL_MFS6_14_RX_DMA0);
        }
        else
        {
            /* MFS6/14 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS6_14_TX_DMA1_IRQn);
            NVIC_EnableIRQ(MFS6_14_TX_DMA1_IRQn);
            NVIC_SetPriority(MFS6_14_TX_DMA1_IRQn, PDL_IRQ_LEVEL_MFS6_14_TX_DMA1);
        }
    }
#endif    
    
#if (PDL_INTERRUPT_ENABLE_MFS7  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs7)
    {
        if(bRxTx == FALSE)
        {
            /* MFS7/15 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS7_15_RX_DMA2_IRQn);
            NVIC_EnableIRQ(MFS7_15_RX_DMA2_IRQn);
            NVIC_SetPriority(MFS7_15_RX_DMA2_IRQn, PDL_IRQ_LEVEL_MFS7_15_RX_DMA2);
        }
        else
        {
            /* MFS7/15 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS7_15_TX_DMA3_IRQn);
            NVIC_EnableIRQ(MFS7_15_TX_DMA3_IRQn);
            NVIC_SetPriority(MFS7_15_TX_DMA3_IRQn, PDL_IRQ_LEVEL_MFS7_15_TX_DMA3);
        }
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS15  == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs15)
    {
        if(bRxTx == FALSE)
        {
            /* MFS7/15 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS7_15_RX_DMA2_IRQn);
            NVIC_EnableIRQ(MFS7_15_RX_DMA2_IRQn);
            NVIC_SetPriority(MFS7_15_RX_DMA2_IRQn, PDL_IRQ_LEVEL_MFS7_15_RX_DMA2);
        }
        else
        {
            /* MFS7/15 transmit interrupt settings */
            NVIC_ClearPendingIRQ(MFS7_15_TX_DMA3_IRQn);
            NVIC_EnableIRQ(MFS7_15_TX_DMA3_IRQn);
            NVIC_SetPriority(MFS7_15_TX_DMA3_IRQn, PDL_IRQ_LEVEL_MFS7_15_TX_DMA3);
        }
    }
#endif    
    
} /* Mfs_InitTxIrq */

/**
 ******************************************************************************
 ** \brief Device dependent de-initialization of interrupts according CMSIS
 **
 ** \param enIndex MFS instance index
 ** \param bRxTx  FALSE: RX IRQ, TRUE: TX IRQ     
 **
 ******************************************************************************/
static void Mfs_DeInitIrq(en_mfs_instance_index_t enIndex, boolean_t bRxTx)
{
#if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs0)
    {
        if(bRxTx == FALSE)
        {
            /* MFS0/8 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS0_8_RX_IRQn);
            NVIC_DisableIRQ(MFS0_8_RX_IRQn);
            NVIC_SetPriority(MFS0_8_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS0/8 tranfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS0_8_TX_IRQn);
            NVIC_DisableIRQ(MFS0_8_TX_IRQn);
            NVIC_SetPriority(MFS0_8_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs8)
    {
        if(bRxTx == FALSE)
        {
            /* MFS0/8 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS0_8_RX_IRQn);
            NVIC_DisableIRQ(MFS0_8_RX_IRQn);
            NVIC_SetPriority(MFS0_8_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS0/8 tranfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS0_8_TX_IRQn);
            NVIC_DisableIRQ(MFS0_8_TX_IRQn);
            NVIC_SetPriority(MFS0_8_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        
    }
#endif    
    
#if (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs1)
    {
        if(bRxTx == FALSE)
        {
            /* MFS1/9 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS1_9_RX_IRQn);
            NVIC_DisableIRQ(MFS1_9_RX_IRQn);
            NVIC_SetPriority(MFS1_9_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS1/9 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS1_9_TX_IRQn);
            NVIC_DisableIRQ(MFS1_9_TX_IRQn);
            NVIC_SetPriority(MFS1_9_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        } 
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs9)
    {
        if(bRxTx == FALSE)
        {
            /* MFS1/9 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS1_9_RX_IRQn);
            NVIC_DisableIRQ(MFS1_9_RX_IRQn);
            NVIC_SetPriority(MFS1_9_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS1/9 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS1_9_TX_IRQn);
            NVIC_DisableIRQ(MFS1_9_TX_IRQn);
            NVIC_SetPriority(MFS1_9_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        } 
    }
#endif    
    
#if (PDL_INTERRUPT_ENABLE_MFS2 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs2)
    {
        if(bRxTx == FALSE)
        {
            /* MFS2/10 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS2_10_RX_IRQn);
            NVIC_DisableIRQ(MFS2_10_RX_IRQn);
            NVIC_SetPriority(MFS2_10_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS2/10 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS2_10_TX_IRQn);
            NVIC_DisableIRQ(MFS2_10_TX_IRQn);
            NVIC_SetPriority(MFS2_10_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
    }
#endif
    
#if (PDL_INTERRUPT_ENABLE_MFS10 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs10)
    {
        if(bRxTx == FALSE)
        {
            /* MFS2/10 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS2_10_RX_IRQn);
            NVIC_DisableIRQ(MFS2_10_RX_IRQn);
            NVIC_SetPriority(MFS2_10_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS2/10 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS2_10_TX_IRQn);
            NVIC_DisableIRQ(MFS2_10_TX_IRQn);
            NVIC_SetPriority(MFS2_10_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
    }
#endif    

#if (PDL_INTERRUPT_ENABLE_MFS3 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs3)
    {
        if(bRxTx == FALSE)
        {
            /* MFS3/11 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS3_11_RX_IRQn);
            NVIC_DisableIRQ(MFS3_11_RX_IRQn);
            NVIC_SetPriority(MFS3_11_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS3/11 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS3_11_TX_IRQn);
            NVIC_DisableIRQ(MFS3_11_TX_IRQn);
            NVIC_SetPriority(MFS3_11_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS11 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs11)
    {
        if(bRxTx == FALSE)
        {
            /* MFS3/11 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS3_11_RX_IRQn);
            NVIC_DisableIRQ(MFS3_11_RX_IRQn);
            NVIC_SetPriority(MFS3_11_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS3/11 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS3_11_TX_IRQn);
            NVIC_DisableIRQ(MFS3_11_TX_IRQn);
            NVIC_SetPriority(MFS3_11_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
    }
#endif    
    
#if (PDL_INTERRUPT_ENABLE_MFS4 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs4)
    {
        if(bRxTx == FALSE)
        {
            /* MFS4/12 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS4_12_RX_IRQn);
            NVIC_DisableIRQ(MFS4_12_RX_IRQn);
            NVIC_SetPriority(MFS4_12_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS4/12 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS4_12_TX_IRQn);
            NVIC_DisableIRQ(MFS4_12_TX_IRQn);
            NVIC_SetPriority(MFS4_12_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);          
        }
        
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS12 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs12)
    {
        if(bRxTx == FALSE)
        {
            /* MFS4/12 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS4_12_RX_IRQn);
            NVIC_DisableIRQ(MFS4_12_RX_IRQn);
            NVIC_SetPriority(MFS4_12_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS4/12 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS4_12_TX_IRQn);
            NVIC_DisableIRQ(MFS4_12_TX_IRQn);
            NVIC_SetPriority(MFS4_12_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);          
        }
        
    }
#endif    
    
#if (PDL_INTERRUPT_ENABLE_MFS5 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs5)
    {
        if(bRxTx == FALSE)
        {
            /* MFS5/13 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS5_13_RX_IRQn);
            NVIC_DisableIRQ(MFS5_13_RX_IRQn);
            NVIC_SetPriority(MFS5_13_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS5/13 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS5_13_TX_IRQn);
            NVIC_DisableIRQ(MFS5_13_TX_IRQn);
            NVIC_SetPriority(MFS5_13_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS13 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs13)
    {
        if(bRxTx == FALSE)
        {
            /* MFS5/13 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS5_13_RX_IRQn);
            NVIC_DisableIRQ(MFS5_13_RX_IRQn);
            NVIC_SetPriority(MFS5_13_RX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS5/13 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS5_13_TX_IRQn);
            NVIC_DisableIRQ(MFS5_13_TX_IRQn);
            NVIC_SetPriority(MFS5_13_TX_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
    }
#endif    
    
#if (PDL_INTERRUPT_ENABLE_MFS6 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs6)
    {
        if(bRxTx == FALSE)
        {
            /* MFS6/14 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS6_14_RX_DMA0_IRQn);
            NVIC_DisableIRQ(MFS6_14_RX_DMA0_IRQn);
            NVIC_SetPriority(MFS6_14_RX_DMA0_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS6/14 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS6_14_TX_DMA1_IRQn);
            NVIC_DisableIRQ(MFS6_14_TX_DMA1_IRQn);
            NVIC_SetPriority(MFS6_14_TX_DMA1_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS14 == PDL_ON)
    if(enIndex == MfsInstanceIndexMfs14)
    {
        if(bRxTx == FALSE)
        {
            /* MFS6/14 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS6_14_RX_DMA0_IRQn);
            NVIC_DisableIRQ(MFS6_14_RX_DMA0_IRQn);
            NVIC_SetPriority(MFS6_14_RX_DMA0_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS6/14 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS6_14_TX_DMA1_IRQn);
            NVIC_DisableIRQ(MFS6_14_TX_DMA1_IRQn);
            NVIC_SetPriority(MFS6_14_TX_DMA1_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
    }
#endif    
    
#if (PDL_INTERRUPT_ENABLE_MFS7 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs7)
    {
        if(bRxTx == FALSE)
        {
            /* MFS7/15 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS7_15_RX_DMA2_IRQn);
            NVIC_DisableIRQ(MFS7_15_RX_DMA2_IRQn);
            NVIC_SetPriority(MFS7_15_RX_DMA2_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS7/15 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS7_15_TX_DMA3_IRQn);
            NVIC_DisableIRQ(MFS7_15_TX_DMA3_IRQn);
            NVIC_SetPriority(MFS7_15_TX_DMA3_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
    }
#endif

#if (PDL_INTERRUPT_ENABLE_MFS15 == PDL_ON)
    if (enIndex == MfsInstanceIndexMfs15)
    {
        if(bRxTx == FALSE)
        {
            /* MFS7/15 receive interrupt settings */
            NVIC_ClearPendingIRQ(MFS7_15_RX_DMA2_IRQn);
            NVIC_DisableIRQ(MFS7_15_RX_DMA2_IRQn);
            NVIC_SetPriority(MFS7_15_RX_DMA2_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
        else
        {
            /* MFS7/15 transfer interrupt settings */
            NVIC_ClearPendingIRQ(MFS7_15_TX_DMA3_IRQn);
            NVIC_DisableIRQ(MFS7_15_TX_DMA3_IRQn);
            NVIC_SetPriority(MFS7_15_TX_DMA3_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
        }
    }
#endif    
    
} /* Mfs_DeInitIrq */

#endif

/******************************************************************************
 * UART driver functions
 ******************************************************************************/

/**
 ******************************************************************************
 ** \brief Return the internal data for a certain UART instance.
 **
 ** \param pstcUart Pointer to UART instance
 **
 ** \return Pointer to internal data or NULL if instance is not enabled (or not known)
 **
 ******************************************************************************/
static stc_mfs_intern_data_t* MfsUartGetInternDataPtr(volatile stc_mfsn_uart_t* pstcUart)
{
    stc_mfs_intern_data_t* pstcInternDataPtr = NULL;
    uint32_t u32Instance;

    if (NULL != pstcUart)
    {
        for (u32Instance = 0u; u32Instance < (uint32_t)MfsInstanceIndexMax; u32Instance++)
        {
            if (pstcUart == m_astcMfsInstanceDataLut[u32Instance].stcInstance.pstcUartInstance)
            {
                pstcInternDataPtr = &m_astcMfsInstanceDataLut[u32Instance].stcInternData;
                break;
            }
        }
            
    }

    return (pstcInternDataPtr);
} /* MfsGetInternDataPtr */


#if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON) ||  \
    (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON) ||  \
    (PDL_INTERRUPT_ENABLE_MFS2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS10 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS3 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS11 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS4 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS12 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS5 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS13 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS6 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS14 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS7 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS15 == PDL_ON) 
/**
 ******************************************************************************
 ** \brief Return instance index for a certain UART instance.
 **
 ** \param pstcUart Pointer to UART instance
 **
 ** \return Instance index
 **
 ******************************************************************************/
static en_mfs_instance_index_t MfsUartGetInstanceIndex(volatile stc_mfsn_uart_t* pstcUart)
{
    en_mfs_instance_index_t enInstanceIndex = MfsInstanceIndexUnknown;
    uint32_t u32Instance;

    if (NULL != pstcUart)
    {
        for (u32Instance = 0u; u32Instance < (uint32_t)MfsInstanceIndexMax; u32Instance++)
        {
            if (pstcUart == m_astcMfsInstanceDataLut[u32Instance].stcInstance.pstcUartInstance)
            {
                enInstanceIndex = m_astcMfsInstanceDataLut[u32Instance].stcInternData.enIndex;
                break;
            }
        }
            
    }

    return (enInstanceIndex);
} /* MfsUartGetInstanceIndex */
      
      
/**
 ******************************************************************************
 ** \brief UART receive interrupt service routine.
 ** 
 ** \param pstcUart Pointer to UART instance   
 ** \param pstcMfsInternData Pointer to MFS internal data structure       
 ** 
 ** This function is called on Receive Interrupt set by the MFS. 
 **
 ******************************************************************************/
void MfsUartIrqHandlerRx( volatile stc_mfsn_uart_t*   pstcUart,
                          stc_mfs_intern_data_t* pstcMfsInternData)
{
    if ((NULL != pstcUart) && (NULL != pstcMfsInternData))
    {
        if (NULL != pstcMfsInternData->stcUartInternIntCb.pfnRxIntCb)
        {
            pstcMfsInternData->stcUartInternIntCb.pfnRxIntCb();
        }
    }
} /* MfsIrqHandlerRx */

/**
 ******************************************************************************
 ** \brief MFS transmit interrupt service routine.
 **
 ** \param pstcUart Pointer to UART instance   
 ** \param pstcMfsInternData Pointer to MFS internal data structure     
 **
 ** This function is called on Transmit Interrupt set by the MFS. 
 ** 
 ******************************************************************************/
void MfsUartIrqHandlerTx( volatile stc_mfsn_uart_t*   pstcUart, 
                          stc_mfs_intern_data_t* pstcMfsInternData)
{
    if ((NULL != pstcUart) && (NULL != pstcMfsInternData))
    {
        if(pstcUart->SSR_f.TDRE == 1)
        {
            if (NULL != pstcMfsInternData->stcUartInternIntCb.pfnTxIntCb)
            {
                pstcMfsInternData->stcUartInternIntCb.pfnTxIntCb();
            }
        }
        
        if(pstcUart->SSR_f.TBI == 1)
        {
            if (NULL != pstcMfsInternData->stcUartInternIntCb.pfnTxIdleCb)
            {
                pstcMfsInternData->stcUartInternIntCb.pfnTxIdleCb();
            }
        }
        
        if(pstcUart->FCR1_f.FDRQ == 1)
        {
            if (NULL != pstcMfsInternData->stcUartInternIntCb.pfnTxFifoIntCb)
            {
                pstcMfsInternData->stcUartInternIntCb.pfnTxFifoIntCb();
            }
            
            pstcUart->FCR1_f.FDRQ = 0;
        }
    }
} /* MfsIrqHandlerTx */
            
/**
 ******************************************************************************
 ** \brief Enable UART interrupts
 **
 ** \param [in] pstcUart   Pointer to UART instance   
 ** \param [in] pstcInt    Pointer to the selection structure of UART interrupt
 ** \param [in] pstcIntCb  Pointer to the structure of UART interrupt callback 
 **                        functions 
 ** 
 ** \retval Ok                    Interrupts has been enabled and callback 
 **                               functions are set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcUart == NULL
 **                               - pstcInt == NULL
 **                               - pstcIntCb == NULL
 **                               - Other invalid configuration                   
 **
 ** The NVIC IRQ is also enabled in this function.
 ** 
 ******************************************************************************/
en_result_t Mfs_Uart_EnableInt(volatile stc_mfsn_uart_t* pstcUart, 
                               stc_uart_int_sel_t* pstcInt,
                               stc_uart_int_cb_t* pstcIntCb)
{
    stc_mfs_intern_data_t* pstcMfsInternData;
    en_mfs_instance_index_t enIndex;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsUartGetInternDataPtr(pstcUart);
    enIndex = MfsUartGetInstanceIndex(pstcUart);
  
    if ((NULL == pstcMfsInternData) ||  
        (NULL == pstcInt) ||
        (NULL == pstcIntCb) || 
        (MfsInstanceIndexUnknown == enIndex))
    {
        return (ErrorInvalidParameter);
    }
    
    if(pstcInt->bRxInt == TRUE)
    {
        pstcMfsInternData->stcUartInternIntCb.pfnRxIntCb = pstcIntCb->pfnRxIntCb;
        pstcUart->SCR_f.RIE = 1;
    }
    
    if(pstcInt->bTxInt == TRUE)
    {
        pstcMfsInternData->stcUartInternIntCb.pfnTxIntCb = pstcIntCb->pfnTxIntCb;
        pstcUart->SCR_f.TIE = 1;
    }
    
    if(pstcInt->bTxIdle == TRUE)
    {
        pstcMfsInternData->stcUartInternIntCb.pfnTxIdleCb = pstcIntCb->pfnTxIdleCb;
        pstcUart->SCR_f.TBIE = 1;
    }
    
    if(pstcInt->bTxFifoInt == TRUE)
    {
        pstcMfsInternData->stcUartInternIntCb.pfnTxFifoIntCb = pstcIntCb->pfnTxFifoIntCb;
        pstcUart->FCR1_f.FTIE = 1;
    }
    
    if(pstcInt->bRxInt == TRUE)
    {
        Mfs_InitIrq(enIndex, FALSE);
    }
    
    if((pstcInt->bTxInt == TRUE) || (pstcInt->bTxIdle == TRUE) || (pstcInt->bTxFifoInt == TRUE))
    {
        Mfs_InitIrq(enIndex, TRUE);
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable UART interrupts
 **
 ** \param [in] pstcUart   Pointer to UART instance   
 ** \param [in] pstcInt    Pointer to the selection structure of UART interrupt
 ** 
 ** \retval Ok                    Interrupts has been disabled and callback 
 **                               function pointers are set to NULL 
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcUart == NULL
 **                               - pstcInt == NULL                
 **
 ** The NVIC IRQ is also disabled in this function if all interrupt sources
 ** are disable.
 ** 
 ******************************************************************************/
en_result_t Mfs_Uart_DisableInt(volatile stc_mfsn_uart_t* pstcUart, 
                                stc_uart_int_sel_t* pstcInt)
{
    stc_mfs_intern_data_t* pstcMfsInternData;
    en_mfs_instance_index_t enIndex;
    uint8_t u8Ch;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsUartGetInternDataPtr(pstcUart);
    enIndex = MfsUartGetInstanceIndex(pstcUart);
    
    if ((NULL == pstcMfsInternData) ||  
        (NULL == pstcInt) ||
        (MfsInstanceIndexUnknown == enIndex))
    {
        return (ErrorInvalidParameter);
    }
    
    u8Ch = UartInstanceToIndex(pstcUart);
    
    if(pstcInt->bRxInt == TRUE)
    {
        pstcMfsInternData->stcUartInternIntCb.pfnRxIntCb = NULL;
        pstcUart->SCR_f.RIE = 0;
    }
    
    if(pstcInt->bTxInt == TRUE)
    {
        pstcMfsInternData->stcUartInternIntCb.pfnTxIntCb = NULL;
        pstcUart->SCR_f.TIE = 0;
    }
    
    if(pstcInt->bTxIdle == TRUE)
    {
        pstcMfsInternData->stcUartInternIntCb.pfnTxIdleCb = NULL;
        pstcUart->SCR_f.TBIE = 0;
    }
    
    if(pstcInt->bTxFifoInt == TRUE)
    {
        pstcMfsInternData->stcUartInternIntCb.pfnTxFifoIntCb = NULL;
        pstcUart->FCR1_f.FTIE = 0;
    }
    
    // Only when RX interrupts and DMA interrupt(share same IRQ with MFS ch.14, ch.15) 
    // are all disabled, disable RX IRQ
    do
    {
        if(pstcUart->SCR_f.RIE == 1)
        {
            break;
        }
        
        if(u8Ch == 14u)
        {
            if((FM0P_DMAC->DMACB0_f.CI == 1) || (FM0P_DMAC->DMACB0_f.EI == 1))
            {
                break;
            }
        }

        if(u8Ch == 15u)
        {
            if((FM0P_DMAC->DMACB2_f.CI == 1) || (FM0P_DMAC->DMACB2_f.EI == 1))
            {
                break;
            }
        }
  
        Mfs_DeInitIrq(enIndex, FALSE);
      
    }while(0);
    
    // Only when TX interrupts and DMA interrupt(share same IRQ with MFS ch.14, ch.15) 
    // are all disabled, disable TX IRQ
    do
    {
        if(pstcUart->SCR_f.TIE == 1)
        {
            break;
        }
        
        if(pstcUart->SCR_f.TBIE == 1)
        {
            break;
        }
        
        if(pstcUart->FCR1_f.FTIE == 1)
        {
            break;
        }
  
        if(u8Ch == 14u)
        {
            if((FM0P_DMAC->DMACB1_f.CI == 0) && (FM0P_DMAC->DMACB1_f.EI == 0))
            {
                break;
            }
        }
     
        if(u8Ch == 15u)
        {
            if((FM0P_DMAC->DMACB3_f.CI == 0) && (FM0P_DMAC->DMACB3_f.EI == 0))
            {
                break;
            }
        }
        
        Mfs_DeInitIrq(enIndex, TRUE);
      
    }while(0);
    
    return Ok;
}

#endif     

/**
 ******************************************************************************
 ** \brief Initialization of a MFS module to activate as UART.
 **
 ** Set registers to active MFS as UART.
 **
 ** \param [in]   pstcUart        Pointer to UART instance register area
 ** \param [in]   pstcConfig      MFS UART configuration
 **
 ** \retval Ok                        Process successfully done.
 ** \retval ErrorInvalidParameter     If one of following conditions are met:
 **             - pstcUart == NULL
 **             - pstcConfig == NULL
 **             - pstcMfsInternData == NULL (invalid or disabled MFS unit
 **                                          (PDL_PERIPHERAL_ENABLE_MFS)) 
 **
 ******************************************************************************/
en_result_t Mfs_Uart_Init(volatile stc_mfsn_uart_t*         pstcUart,
                          const stc_mfs_uart_config_t* pstcConfig)
{
    stc_mfs_intern_data_t* pstcMfsInternData;
    
    /* Preset local register variables to zero */
    stc_mfs_uart_smr_field_t  stcSMR  = { 0 };
    stc_mfs_uart_scr_field_t  stcSCR  = { 0 };
    stc_mfs_uart_escr_field_t stcESCR = { 0 };
   
    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsUartGetInternDataPtr(pstcUart);
    
    /* Parameter check and get ptr to internal data struct */
    if ((NULL == pstcMfsInternData)
    ||  (NULL == pstcConfig)
       )
    {
        return (ErrorInvalidParameter);
    }

    /* Set the MFS mode in the internal structure */
    pstcMfsInternData->enMode = MfsUartMode;
    
    /* Set UART mode */
    switch (pstcConfig->enMode)
    {
        case UartNormal:
            stcSMR.MD = 0u;   /* Normal mode */
            break;
        case UartMulti:
            stcSMR.MD = 1u;    /* Multi-processor mode */
            break;
        default:
            return (ErrorInvalidParameter);
    }

    /* Enable SOE */
    stcSMR.SOE = TRUE;
    /* Set Parity */
    switch(pstcConfig->enParity)
    {
        case UartParityNone:
            stcESCR.P   = FALSE; /* Parity type selection (0-even/1-odd) */
            stcESCR.PEN = FALSE; /* Parity disable */
            break;
        case UartParityEven:
            stcESCR.P   = FALSE; /* Parity type selection (0-even/1-odd) */
            stcESCR.PEN = TRUE;  /* Parity Enable */
            break;
        case UartParityOdd:
            stcESCR.P   = TRUE;  /* Parity type selection (0-even/1-odd) */
            stcESCR.PEN = TRUE;  /* Parity Enable */
            break;
        default:
            return (ErrorInvalidParameter);
    }
    /* Set Stop bit length */
    switch (pstcConfig->enStopBit)
    {
        case UartOneStopBit:
            stcSMR.SBL   = FALSE;
            stcESCR.ESBL = FALSE;
            break;
        case UartTwoStopBits:
            stcSMR.SBL   = TRUE;
            stcESCR.ESBL = FALSE;
            break;     
        case UartThreeStopBits:
            stcSMR.SBL   = FALSE;
            stcESCR.ESBL = TRUE;
            break;
        case UartFourStopBits:
            stcSMR.SBL   = TRUE;
            stcESCR.ESBL = TRUE;
            break;
        default:
            return (ErrorInvalidParameter);
    }    
    /* Set Data bit length (5 - 9bits) */
    switch(pstcConfig->enDataLength)
    {
        case UartFiveBits:
            stcESCR.L = UartFiveBits;
            break;
        case UartSixBits:
            stcESCR.L = UartSixBits;
            break;
        case UartSevenBits:
            stcESCR.L = UartSevenBits;
            break;   
        case UartEightBits:
            stcESCR.L = UartEightBits;
            break;
        case UartNineBits:
            stcESCR.L = UartNineBits;
            break;
        default:
            return (ErrorInvalidParameter);
    }
    /* Set Bit direction (LSB/MSB) */    
    switch(pstcConfig->enBitDirection)
    {
        case UartDataLsbFirst:
            stcSMR.BDS = 0;
            break;
        case UartDataMsbFirst:
            stcSMR.BDS = 1;
            break;  
        default:
            return ErrorInvalidParameter;  
    }

    /* HW Flow */
    if (TRUE == pstcConfig->bHwFlow)
    {
        stcESCR.FLWEN = TRUE;
    }
    else
    {
        stcESCR.FLWEN = FALSE;
    }

    /* Set Signal system (NRZ/Int-NRZ) */
    if (TRUE == pstcConfig->bInvertData)
    {
        stcESCR.INV = TRUE;
    }
    else
    {
        stcESCR.INV = FALSE;
    }
    
    /* Clear MFS by setting the Software Reset bit */
    pstcUart->SCR_f.UPCL = TRUE;

    /* Set Baudrate */
    (void)Mfs_Uart_SetBaudRate(pstcUart, pstcConfig->u32BautRate);
    /* Set registers value */
    pstcUart->SMR_f  = stcSMR;
    pstcUart->SCR_f  = stcSCR;
    pstcUart->ESCR_f = stcESCR;
    
    if(pstcConfig->bEnableFifo == TRUE)
    {
        /* Reset FIFO */
        pstcUart->FCR0_f.FCL1 = 1;
        pstcUart->FCR0_f.FCL2 = 1;
        /* Enable FIFO receive Idle detection */
        pstcUart->FCR1_f.FRIIE = 1;
        /* Selection TX and RX FIFO  */
        switch(pstcConfig->stcFifoConfig.enFifoSel)
        {
            case MfsTxFifo1RxFifo2:
                pstcUart->FCR1_f.FSEL = 0;
                break;
            case MfsTxFifo2RxFifo1:
                pstcUart->FCR1_f.FSEL = 1;
                break;
            default:
               return (ErrorInvalidParameter);
        }
        /* Set FIFO count */
        pstcUart->FBYTE1 = pstcConfig->stcFifoConfig.u8ByteCount1;
        pstcUart->FBYTE2 = pstcConfig->stcFifoConfig.u8ByteCount2;
        /* Enable FIFO  */
        pstcUart->FCR0_f.FE1 = 1;
        pstcUart->FCR0_f.FE2 = 1;
    }

    return (Ok);
} /* MFS_Uart_Init  */

/**
 ******************************************************************************
 ** \brief Deinitialisation of a MFS module activating as UART.
 **
 ** All used uart register are reset to their default values.
 **  
 ** \param [in]   pstcUart      Pointer to UART instance register area
 ** 
 ** \retval Ok                        Process successfully done.
 ** \retval ErrorInvalidParameter     If one of following conditions are met:
 **             - pstcUart == NULL
 **             - pstcMfsInternData == NULL (invalid or disabled MFS unit
 **                                          (PDL_PERIPHERAL_ENABLE_MFS)) 
 **
 ******************************************************************************/
en_result_t Mfs_Uart_DeInit(volatile stc_mfsn_uart_t* pstcUart)
{
    en_result_t            enResult;
    stc_mfs_intern_data_t* pstcMfsInternData;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsUartGetInternDataPtr(pstcUart);
    /* ... and check */
    if (NULL == pstcMfsInternData)
    {
        enResult = ErrorInvalidParameter;
    }
    else
    {
        pstcUart->SCR_f.TXE = FALSE;
        pstcUart->SCR_f.RXE = FALSE;

        /* Baud Rate Generation Reload Reset */
        pstcUart->BGR = 0u;
            
        /* Serial Mode Register clear all bits (valid in any mode) */
        pstcUart->SMR = 0u;
           
        /* Reset Mfs receive and transmit bit to default */
        /* and clear all error flags */
        pstcUart->SCR = 0u;

        /* Clear reception Errors */
        pstcUart->SSR_f.REC = TRUE;
           
        /* Reset all other used register to default value */
        pstcUart->SSR  = 0u;
        pstcUart->ESCR = 0u;
        pstcUart->FCR0 = 0u;
        pstcUart->FCR1 = 0u;

        /* Clear MFS by setting the Software Reset bit */
        pstcUart->SCR_f.UPCL = TRUE;

        enResult = Ok;
    }

    return (enResult);
} /* Mfs_Uart_DeInit */

/**
 ******************************************************************************
 ** \brief Set the baudrate of UART
 **
 ** \param [in] pstcUart    Pointer to UART instance   
 ** \param [in] u32BaudRate Baudrate value [bps]
 ** 
 ** \retval Ok                    UART baud rate has been successfully modified
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcUart == NULL
 **                               - u32BaudRate < 4         
 ** \note
 ** The UART baud rate can be initialized in the Mfs_Uart_Init() and be modified
 ** in the funciton.
 ** 
 ******************************************************************************/
en_result_t Mfs_Uart_SetBaudRate(volatile stc_mfsn_uart_t* pstcUart,
                                 uint32_t u32BaudRate)
{
    uint32_t u32Pclk1;
  
    if ((NULL == pstcUart) || (u32BaudRate < 4))
    {
        return ErrorInvalidParameter;
    }
    
    u32Pclk1 = __HCLK/(1ul << (APBC1_PSR_Val & 0x03u)); /* MFS is attached on APB1 bus */
    
    pstcUart->BGR_f.BGR = (u32Pclk1/u32BaudRate) - 1;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Enable UART functions
 **
 ** \param [in] pstcUart   Pointer to UART instance   
 ** \param [in] enFunc     UART function types
 ** \arg   UartTx    UART transfer function
 ** \arg   UartRx    UART receive function
 ** 
 ** \retval Ok                    Function has been enabled normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcUart == NULL
 **                               - Other invalid configuration                   
 ** 
 ******************************************************************************/
en_result_t Mfs_Uart_EnableFunc(volatile stc_mfsn_uart_t* pstcUart, en_uart_func_t enFunc)
{
    if (NULL == pstcUart)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFunc)
    {
        case UartTx:
            pstcUart->SCR_f.TXE = 1;
            break;
        case UartRx:
            pstcUart->SCR_f.RXE = 1;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable UART functions
 **
 ** \param [in] pstcUart   Pointer to UART instance   
 ** \param [in] enFunc     UART function types
 ** \arg   UartTx    UART transfer function
 ** \arg   UartRx    UART receive function
 ** 
 ** \retval Ok                    Function has been disabled normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcUart == NULL
 **                               - Other invalid configuration                   
 ** 
 ******************************************************************************/
en_result_t Mfs_Uart_DisableFunc(volatile stc_mfsn_uart_t* pstcUart, en_uart_func_t enFunc)
{
    if (NULL == pstcUart)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFunc)
    {
        case UartTx:
            pstcUart->SCR_f.TXE = 0;
            break;
        case UartRx:
            pstcUart->SCR_f.RXE = 0;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get status of UART according to status type
 **
 ** \param [in] pstcUart     Pointer to UART instance   
 ** \param [in] enStatus     UART status type
 ** \arg   UartParityError   UART parity error
 ** \arg   UartFrameError    UART frame error
 ** \arg   UartRxFull        UART receive buffer full
 ** \arg   UartTxEmpty       UART tranfer buffer empty
 ** \arg   UartTxIdle        UART tranfer idle status
 ** \arg   UartTxFifoRequest UART transfer FIFO request
 ** 
 ** \retval FALSE                 If one of following conditions are met:
 **                               - No UART parity error [enStatus = UartParityError]
 **                               - No UART frame error  [enStatus = UartFrameError]     
 **                               - UART receive buffer is not full [enStatus = UartRxFull]
 **                               - UART tranfer buffer is not empty [enStatus = UartTxEmpty]
 **                               - UART tranfer is on-going [enStatus = UartTxIdle]
 **                               - No UART transfer FIFO request [enStatus = UartTxFifoRequest] 
 ** \retval TRUE                 If one of following conditions are met:
 **                               - UART parity error occurs [enStatus = UartParityError]
 **                               - UART frame error occurs  [enStatus = UartFrameError]    
 **                               - UART receive buffer is full [enStatus = UartRxFull]
 **                               - UART tranfer buffer is empty [enStatus = UartTxEmpty]
 **                               - UART tranfer is idle [enStatus = UartTxIdle]
 **                               - UART transfer FIFO request issues [enStatus = UartTxFifoRequest] 
 ** 
 ******************************************************************************/
boolean_t Mfs_Uart_GetStatus(volatile stc_mfsn_uart_t* pstcUart, 
                             en_uart_status_t enStatus)
{
    boolean_t bResult = FALSE;
    
    switch(enStatus)
    {
        case UartParityError:
            bResult = (pstcUart->SSR_f.PE == 1) ? TRUE : FALSE;
            break;  
        case UartFrameError:
            bResult = (pstcUart->SSR_f.FRE == 1) ? TRUE : FALSE;
            break;  
        case UartOverrunError:
            bResult = (pstcUart->SSR_f.ORE == 1) ? TRUE : FALSE;
            break;  
        case UartRxFull:
            bResult = (pstcUart->SSR_f.RDRF == 1) ? TRUE : FALSE;
            break;  
        case UartTxEmpty:
            bResult = (pstcUart->SSR_f.TDRE == 1) ? TRUE : FALSE;
            break;  
        case UartTxIdle:
            bResult = (pstcUart->SSR_f.TBI == 1) ? TRUE : FALSE;
            break;  
        case UartTxFifoRequest:
            bResult = (pstcUart->FCR1_f.FDRQ == 1) ? TRUE : FALSE;
            break;  
        default:
            break;  
    }
    
    return bResult;
}

/**
 ******************************************************************************
 ** \brief Clear status of UART according to status type
 **
 ** \param [in] pstcUart     Pointer to UART instance   
 ** \param [in] enStatus     UART status type
 ** \arg   UartParityError   UART parity error
 ** \arg   UartFrameError    UART frame error
 ** \arg   UartRxFull        UART receive buffer full
 ** \arg   UartTxEmpty       UART tranfer buffer empty
 ** \arg   UartTxIdle        UART tranfer idle status
 ** \arg   UartTxFifoRequest UART transfer FIFO request
 ** 
 ** \retval Ok                    Status has been cleared normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcUart == NULL
 ** \note  The following status can only be cleared by hardware behavior:
 **        - UartRxFull
 **        - UartTxEmpty
 **        - UartTxIdle
 ** 
 ******************************************************************************/
en_result_t Mfs_Uart_ClrStatus(volatile stc_mfsn_uart_t* pstcUart,
                               en_uart_status_t enStatus)
{
    if (NULL == pstcUart)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enStatus)
    {
        case UartParityError: 
        case UartFrameError:
        case UartOverrunError:
            pstcUart->SSR_f.REC = 1;
            break;  
        case UartRxFull:
        case UartTxEmpty:
        case UartTxIdle:
            break;  
        case UartTxFifoRequest:
            pstcUart->FCR1_f.FDRQ = 0;
            break;  
        default:
            return ErrorInvalidParameter;  
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Write UART data buffer
 **
 ** \param [in] pstcUart   Pointer to UART instance   
 ** \param [in] u16Data    Send data
 ** 
 ** \retval Ok                    Data has been successfully sent
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcUart == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_Uart_SendData(volatile stc_mfsn_uart_t* pstcUart, uint16_t u16Data)
{
    if (NULL == pstcUart)
    {
        return ErrorInvalidParameter;
    }
    
    pstcUart->TDR = u16Data;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Read UART data buffer
 **
 ** \param [in] pstcUart   Pointer to UART instance   
 ** 
 ** \retval Receive data        
 ** 
 ******************************************************************************/
uint16_t Mfs_Uart_ReceiveData(volatile stc_mfsn_uart_t* pstcUart)
{    
    return (pstcUart->RDR);
}

/**
 ******************************************************************************
 ** \brief Reset UART FIFO
 **
 ** \param [in] pstcUart   Pointer to UART instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** 
 ** \retval Ok                    FIFO has been successfully reset
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcUart == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_Uart_ResetFifo (volatile stc_mfsn_uart_t* pstcUart, 
                                en_mfs_fifo_t enFifo)
{
    if (NULL == pstcUart)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFifo)
    {
        case MfsFifo1:
            pstcUart->FCR0_f.FCL1 = 1;
            break;
        case MfsFifo2:
            pstcUart->FCR0_f.FCL2 = 1;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Set UART FIFO count
 **
 ** \param [in] pstcUart   Pointer to UART instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** \param [in] u8Count    FIFO count
 ** 
 ** \retval Ok                    FIFO count has been successfully set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcUart == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_Uart_SetFifoCount(volatile stc_mfsn_uart_t* pstcUart,
                                  en_mfs_fifo_t enFifo,
                                  uint8_t u8Count)
{
    if (NULL == pstcUart)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFifo)
    {
        case MfsFifo1:
            pstcUart->FBYTE1 = u8Count;
            break;
        case MfsFifo2:
            pstcUart->FBYTE2 = u8Count;
            break;
        default:
           return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get UART FIFO count
 **
 ** \param [in] pstcUart   Pointer to UART instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** 
 ** \retval FIFO count      
 ** 
 ** This function gets the current data count in selected FIFO.
 **
 ** \note 0xFF will be return value if FIFO index is error.
 **
 ******************************************************************************/
uint8_t Mfs_Uart_GetFifoCount(volatile stc_mfsn_uart_t* pstcUart,
                              en_mfs_fifo_t enFifo)
{
    uint8_t u8FifoCnt = 0xFFu;
    
    switch(enFifo)
    {
        case MfsFifo1:
            u8FifoCnt = pstcUart->FBYTE1;
            break;
        case MfsFifo2:
            u8FifoCnt = pstcUart->FBYTE2;
            break;
        default:
           break;
    }
    
    return u8FifoCnt;
}

/******************************************************************************
 * CSIO driver functions
 ******************************************************************************/

/**
 ******************************************************************************
 ** \brief Return the internal data for a certain CSIO instance.
 **
 ** \param pstcCsio Pointer to CSIO instance
 **
 ** \return Pointer to internal data or NULL if instance is not enabled (or not known)
 **
 ******************************************************************************/
static stc_mfs_intern_data_t* MfsCsioGetInternDataPtr(volatile stc_mfsn_csio_t* pstcCsio)
{
    stc_mfs_intern_data_t* pstcInternDataPtr = NULL;
    uint32_t u32Instance;

    if (NULL != pstcCsio)
    {
        for (u32Instance = 0u; u32Instance < (uint32_t)MfsInstanceIndexMax; u32Instance++)
        {
            if (pstcCsio == m_astcMfsInstanceDataLut[u32Instance].stcInstance.pstcCsioInstance)
            {
                pstcInternDataPtr = &m_astcMfsInstanceDataLut[u32Instance].stcInternData;
                break;
            }
        }
            
    }

    return (pstcInternDataPtr);
} /* MfsGetInternDataPtr */

#if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON) ||  \
    (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON) ||  \
    (PDL_INTERRUPT_ENABLE_MFS2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS10 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS3 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS11 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS4 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS12 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS5 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS13 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS6 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS14 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS7 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS15 == PDL_ON) 
/**
 ******************************************************************************
 ** \brief Return instance index for a certain CSIO instance.
 **
 ** \param pstcCsio Pointer to CSIO instance
 **
 ** \return Instance index
 **
 ******************************************************************************/
static en_mfs_instance_index_t MfsCsioGetInstanceIndex(volatile stc_mfsn_csio_t* pstcCsio)
{
    en_mfs_instance_index_t enInstanceIndex = MfsInstanceIndexUnknown;
    uint32_t u32Instance;

    if (NULL != pstcCsio)
    {
        for (u32Instance = 0u; u32Instance < (uint32_t)MfsInstanceIndexMax; u32Instance++)
        {
            if (pstcCsio == m_astcMfsInstanceDataLut[u32Instance].stcInstance.pstcCsioInstance)
            {
                enInstanceIndex = m_astcMfsInstanceDataLut[u32Instance].stcInternData.enIndex;
                break;
            }
        }
            
    }

    return (enInstanceIndex);
} /* MfsCsioGetInstanceIndex */
      
/**
 ******************************************************************************
 ** \brief CSIO receive interrupt service routine.
 ** 
 ** \param pstcCsio Pointer to CSIO instance   
 ** \param pstcMfsInternData Pointer to MFS internal data structure       
 ** 
 ** This function is called on Receive Interrupt set by the MFS. 
 **
 ******************************************************************************/
void MfsCsioIrqHandlerRx( volatile stc_mfsn_csio_t*   pstcCsio,
                          stc_mfs_intern_data_t* pstcMfsInternData)
{
    if ((NULL != pstcCsio) && (NULL != pstcMfsInternData))
    {
        if (NULL != pstcMfsInternData->stcCsioInternIntCb.pfnRxIntCb)
        {
            pstcMfsInternData->stcCsioInternIntCb.pfnRxIntCb();
        }
    }
} /* MfsCsioIrqHandlerRx */

/**
 ******************************************************************************
 ** \brief CSIO transfer interrupt service routine.
 ** 
 ** \param pstcCsio Pointer to CSIO instance   
 ** \param pstcMfsInternData Pointer to MFS internal data structure       
 ** 
 ** This function is called on transfer Interrupt set by the MFS. 
 **
 ******************************************************************************/
void MfsCsioIrqHandlerTx( volatile stc_mfsn_csio_t*   pstcCsio, 
                          stc_mfs_intern_data_t* pstcMfsInternData)
{
    if ((NULL != pstcCsio) && (NULL != pstcMfsInternData))
    {
        if(pstcCsio->SSR_f.TDRE == 1)
        {
            if (NULL != pstcMfsInternData->stcCsioInternIntCb.pfnTxIntCb)
            {
                pstcMfsInternData->stcCsioInternIntCb.pfnTxIntCb();
            }
        }
        
        if(pstcCsio->SSR_f.TBI == 1)
        {
            if (NULL != pstcMfsInternData->stcCsioInternIntCb.pfnTxIdleCb)
            {
                pstcMfsInternData->stcCsioInternIntCb.pfnTxIdleCb();
            }
        }
        
        if(pstcCsio->FCR1_f.FDRQ == 1)
        {
            if (NULL != pstcMfsInternData->stcCsioInternIntCb.pfnTxFifoIntCb)
            {
                pstcMfsInternData->stcCsioInternIntCb.pfnTxFifoIntCb();
            }
            
            pstcCsio->FCR1_f.FDRQ = 0;
        }
        
        if(pstcCsio->SACSR_f.CSE == 1)
        {
            if (NULL != pstcMfsInternData->stcCsioInternIntCb.pfnCsErrIntCb)
            {
                pstcMfsInternData->stcCsioInternIntCb.pfnCsErrIntCb();
            }
            
            pstcCsio->SACSR_f.CSE = 0;
        }
        
    }
} /* MfsIrqHandlerTx */
          
/**
 ******************************************************************************
 ** \brief CSIO status interrupt service routine.
 ** 
 ** \param pstcCsio Pointer to CSIO instance   
 ** \param pstcMfsInternData Pointer to MFS internal data structure       
 ** 
 ** This function is called on transfer Interrupt set by the MFS. 
 **
 ******************************************************************************/
void MfsCsioIrqHandlerStatus( volatile stc_mfsn_csio_t*   pstcCsio, 
                              stc_mfs_intern_data_t* pstcMfsInternData)
{
    if ((NULL != pstcCsio) && (NULL != pstcMfsInternData))
    {
        if(pstcCsio->SACSR_f.TINT == 1)
        {
            if (NULL != pstcMfsInternData->stcCsioInternIntCb.pfnSerialTimerIntCb)
            {
                pstcMfsInternData->stcCsioInternIntCb.pfnSerialTimerIntCb();
            }
            
            pstcCsio->SACSR_f.TINT = 0;
        }
    }
}

/**
 ******************************************************************************
 ** \brief Enable CSIO interrupts
 **
 ** \param [in] pstcCsio   Pointer to CSIO instance   
 ** \param [in] pstcInt    Pointer to the selection structure of CSIO interrupt
 ** \param [in] pstcIntCb  Pointer to the structure of CSIO interrupt callback 
 **                        functions 
 ** 
 ** \retval Ok                    Interrupts has been enabled and callback 
 **                               functions are set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL
 **                               - pstcInt == NULL
 **                               - pstcIntCb == NULL
 **                               - Other invalid configuration                   
 **
 ** The NVIC IRQ is also enabled in this function.
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_EnableInt(volatile stc_mfsn_csio_t* pstcCsio, 
                               stc_csio_int_sel_t* pstcInt,
                               stc_csio_int_cb_t* pstcIntCb)
{
    stc_mfs_intern_data_t* pstcMfsInternData;
    en_mfs_instance_index_t enIndex;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsCsioGetInternDataPtr(pstcCsio);
    enIndex = MfsCsioGetInstanceIndex(pstcCsio);
  
    if ((NULL == pstcMfsInternData) ||  
        (NULL == pstcInt) ||
        (NULL == pstcIntCb) ||
        (MfsInstanceIndexUnknown == enIndex))
    {
        return (ErrorInvalidParameter);
    }
    
    if(pstcInt->bRxInt == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnRxIntCb = pstcIntCb->pfnRxIntCb;
        pstcCsio->SCR_f.RIE = 1;
    }
    
    if(pstcInt->bTxInt == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnTxIntCb = pstcIntCb->pfnTxIntCb;
        pstcCsio->SCR_f.TIE = 1;
    }
    
    if(pstcInt->bTxIdle == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnTxIdleCb = pstcIntCb->pfnTxIdleCb;
        pstcCsio->SCR_f.TBIE = 1;
    }
    
    if(pstcInt->bTxFifoInt == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnTxFifoIntCb = pstcIntCb->pfnTxFifoIntCb;
        pstcCsio->FCR1_f.FTIE = 1;
    }
    
    if(pstcInt->bCsErrInt == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnCsErrIntCb = pstcIntCb->pfnCsErrIntCb;
        pstcCsio->SACSR_f.CSEIE = 1;
    }
    
    if(pstcInt->bSerialTimerInt == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnSerialTimerIntCb = pstcIntCb->pfnSerialTimerIntCb;
        pstcCsio->SACSR_f.TINTE = 1;
    }
    
    if(pstcInt->bRxInt == TRUE)
    {
        Mfs_InitIrq(enIndex, FALSE);
    }
    
    if((pstcInt->bTxInt     == TRUE) || (pstcInt->bTxIdle   == TRUE) || 
       (pstcInt->bTxFifoInt == TRUE) || (pstcInt->bCsErrInt == TRUE) ||
       (pstcInt->bSerialTimerInt == TRUE))
    {
        Mfs_InitIrq(enIndex, TRUE);
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable CSIO interrupts
 **
 ** \param [in] pstcCsio   Pointer to CSIO instance   
 ** \param [in] pstcInt    Pointer to the selection structure of CSIO interrupt
 ** 
 ** \retval Ok                    Interrupts has been disabled and callback 
 **                               function pointers are set to NULL 
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL
 **                               - pstcInt == NULL                
 **
 ** The NVIC IRQ is also disabled in this function if all interrupt sources
 ** are disable.
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_DisableInt(volatile stc_mfsn_csio_t* pstcCsio, 
                                stc_csio_int_sel_t* pstcInt)
{
    stc_mfs_intern_data_t* pstcMfsInternData;
    en_mfs_instance_index_t enIndex;
    uint8_t u8Ch;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsCsioGetInternDataPtr(pstcCsio);
    enIndex = MfsCsioGetInstanceIndex(pstcCsio);
    
    if ((NULL == pstcMfsInternData) ||  
        (NULL == pstcInt) ||
        (MfsInstanceIndexUnknown == enIndex)  )
    {
        return (ErrorInvalidParameter);
    }
    
    u8Ch = CsioInstanceToIndex(pstcCsio);
    
    if(pstcInt->bRxInt == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnRxIntCb = NULL;
        pstcCsio->SCR_f.RIE = 0;
    }
    
    if(pstcInt->bTxInt == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnTxIntCb = NULL;
        pstcCsio->SCR_f.TIE = 0;
    }
    
    if(pstcInt->bTxIdle == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnTxIdleCb = NULL;
        pstcCsio->SCR_f.TBIE = 0;
    }
    
    if(pstcInt->bTxFifoInt == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnTxFifoIntCb = NULL;
        pstcCsio->FCR1_f.FTIE = 0;
    }
    
    if(pstcInt->bCsErrInt == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnCsErrIntCb = NULL;
        pstcCsio->SACSR_f.CSEIE = 0;
    }
    
    if(pstcInt->bSerialTimerInt == TRUE)
    {
        pstcMfsInternData->stcCsioInternIntCb.pfnSerialTimerIntCb = NULL;
        pstcCsio->SACSR_f.TINTE = 0;
    }
    
    // Only when RX interrupts and DMA interrupt(share same IRQ with MFS ch.14, ch.15) 
    // are all disabled, disable RX IRQ
    do
    {
        if(pstcCsio->SCR_f.RIE == 1)
        {
            break;
        }
        
        if(u8Ch == 14u)
        {
            if((FM0P_DMAC->DMACB0_f.CI == 1) || (FM0P_DMAC->DMACB0_f.EI == 1))
            {
                break;
            }
        }
     
        if(u8Ch == 15u)
        {
            if((FM0P_DMAC->DMACB2_f.CI == 1) || (FM0P_DMAC->DMACB2_f.EI == 1))
            {
                break;
            }
        }
   
        Mfs_DeInitIrq(enIndex, FALSE);
      
    }while(0);
    
    // Only when TX interrupts and DMA interrupt(share same IRQ with MFS ch.14, ch.15) 
    // are all disabled, disable TX IRQ
    do
    {
        if(pstcCsio->SCR_f.TIE == 1)
        {
            break;
        }
        
        if(pstcCsio->SCR_f.TBIE == 1)
        {
            break;
        }
        
        if(pstcCsio->FCR1_f.FTIE == 1)
        {
            break;
        }
        
        if(pstcCsio->SACSR_f.CSEIE == 1)
        {
            break;
        }
        
        if(pstcCsio->SACSR_f.TINTE == 1)
        {
            break;
        }
  
        if(u8Ch == 14u)
        {
            if((FM0P_DMAC->DMACB1_f.CI == 0) && (FM0P_DMAC->DMACB1_f.EI == 0))
            {
                break;
            }
        }
 
        if(u8Ch == 15u)
        {
            if((FM0P_DMAC->DMACB3_f.CI == 0) && (FM0P_DMAC->DMACB3_f.EI == 0))
            {
                break;
            }
        }  
        
        Mfs_DeInitIrq(enIndex, TRUE);
      
    }while(0);
    
    return Ok;
}

#endif   

/**
 ******************************************************************************
 ** \brief Initialization of a MFS module to activate as CSIO.
 **
 ** Set registers to active MFS as CSIO.
 **  
 ** \param [in]   pstcCsio        Pointer to CSIO instance register area
 ** \param [in]   pstcConfig      MFS CSIO configuration
 ** 
 ** \retval Ok                        Process successfully done.
 ** \retval ErrorInvalidParameter     If one of following conditions are met:
 **             - pstcCsio == NULL
 **             - pstcConfig == NULL
 **             - pstcMfsInternData == NULL (invalid or disabled MFS unit
 **                                          (PDL_PERIPHERAL_ENABLE_MFS)) 
 **
 ******************************************************************************/
en_result_t Mfs_Csio_Init(volatile stc_mfsn_csio_t*         pstcCsio,
                          const stc_mfs_csio_config_t* pstcConfig)
{
    stc_mfs_intern_data_t* pstcMfsInternData;
    
    /* Preset local register variables to zero */
    stc_mfs_csio_smr_field_t  stcSMR  = { 0 };
    stc_mfs_csio_scr_field_t  stcSCR  = { 0 };
    stc_mfs_csio_ssr_field_t  stcSSR  = { 0 };
    stc_mfs_csio_escr_field_t stcESCR = { 0 };

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsCsioGetInternDataPtr(pstcCsio);
    
    /* Parameter check and get ptr to internal data struct */
    if ((NULL == pstcMfsInternData)
        ||  (NULL == pstcConfig))
    {
        return (ErrorInvalidParameter);
    }

    /* Set MFS mode in the internal structure */
    pstcMfsInternData->enMode = MfsCsioMode;
    
    /* Set CSIO mode */
    stcSMR.MD = 2u;

    /* Set master/slave mode */
    switch(pstcConfig->enMsMode)
    {
        case CsioMaster:
            stcSCR.MS = FALSE;  /* Master mode */
            /* Enable SOE */
            stcSMR.SOE = TRUE;  
            /* Enable SCKE */
            stcSMR.SCKE = TRUE;
            break;
        case CsioSlave:
            stcSCR.MS = TRUE;   /* Slave mode */
            /* Enable SOE */
            stcSMR.SOE = TRUE;  
            /* Enable SCKE */
            stcSMR.SCKE = FALSE; /* Disable clock output in slave mode */
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    

    /* Set normal/SPI mode */
    switch (pstcConfig->enActMode)
    {
        case CsioActNormalMode:
            stcSCR.SPI = FALSE; /* Normal mode */
            break;
        case CsioActSpiMode:
            stcSCR.SPI = TRUE;  /* SPI mode */
            break;
        default:
            return (ErrorInvalidParameter);
    }

    /* Wait time insertion */
    switch(pstcConfig->enSyncWaitTime)
    {
        case CsioSyncWaitZero:
            stcESCR.WT = 0u;    /* 0bit */
            break;
        case CsioSyncWaitOne:
            stcESCR.WT = 1u;    /* 1bit */
            break;
        case CsioSyncWaitTwo:
            stcESCR.WT = 2u;   /* 2bits */
            break;
        case CsioSyncWaitThree:
            stcESCR.WT = 3u;   /* 3bits */
            break;
        default:
            return (ErrorInvalidParameter);
    }

    stcESCR.L3 = 0;  /* Default:less than 13 bits */

    /* Set Data bit length (5-16 bits) */
    switch (pstcConfig->enDataLength)
    {
        case CsioFiveBits:
            stcESCR.L = 1u;
            break;
        case CsioSixBits:
            stcESCR.L = 2u;
            break;
        case CsioSevenBits:
            stcESCR.L = 3u;
            break;   
        case CsioEightBits:
            stcESCR.L = 0u;
            break;
        case CsioNineBits:
            stcESCR.L = 4u;
            break;
        case CsioTenBits:
            stcESCR.L = 5u;
            break;
        case CsioElevenBits:
            stcESCR.L = 6u;
            break;
        case CsioTwelveBits:
            stcESCR.L = 7u;
            break;
        case CsioThirteenBits:
            stcESCR.L = 0u;
            stcESCR.L3 = TRUE;  /* More than 12 bits */
            break;
        case CsioFourteenBits:
            stcESCR.L = 1u;
            stcESCR.L3 = TRUE;  /* More than 12 bits */
            break;
        case CsioFifteenBits:
            stcESCR.L = 2u;
            stcESCR.L3 = TRUE;  /* More than 12 bits */
            break;
        case CsioSixteenBits:
            stcESCR.L = 3u;
            stcESCR.L3 = TRUE;  /* More than 12 bits */
            break;
        default:
            return (ErrorInvalidParameter);
    }
    /* Set Bit direction (LSB/MSB) */    
    switch(pstcConfig->enBitDirection)
    {
        case CsioDataLsbFirst:
            stcSMR.BDS = FALSE;
            break;
        case CsioDataMsbFirst:
            stcSMR.BDS = TRUE;
            break;
        default:
            return ErrorInvalidParameter;
    }

    /* Set Signal system (SCK Mark Level) */
    if (TRUE == pstcConfig->bInvertClk)
    {
        stcSMR.SCINV = TRUE;    /* SCK Make Level : Low */
    }
    else
    {
        stcSMR.SCINV = FALSE;   /* SCK Make Level : High */
    }

    stcSSR.REC = TRUE;          /* Clear received error flag */

    /* Set Baudrate */
    (void)Mfs_Csio_SetBaudRate(pstcCsio, pstcConfig->u32BaudRate);
    /* Set registers value */
    pstcCsio->SMR_f  = stcSMR;
    pstcCsio->SCR_f  = stcSCR;
    pstcCsio->ESCR_f = stcESCR;
    pstcCsio->SSR_f  = stcSSR;

    if(pstcConfig->bEnSyncTransfer == TRUE)  /* Use serial timer? */
    {
        /* Set serial timer clock */
        switch(pstcConfig->stcSerialTimer.enClkDiv)
        {
            case CsioTimerNoDiv:
                pstcCsio->SACSR_f.TDIV = 0u;
                break;
            case CsioTimerDiv2:
                pstcCsio->SACSR_f.TDIV = 1u;
                break;
            case CsioTimerDiv4:
                pstcCsio->SACSR_f.TDIV = 2u;
                break;
            case CsioTimerDiv8: 
                pstcCsio->SACSR_f.TDIV = 3u;
                break;
            case CsioTimerDiv16:
                pstcCsio->SACSR_f.TDIV = 4u;
                break;
            case CsioTimerDiv32: 
                pstcCsio->SACSR_f.TDIV = 5u;
                break;
            case CsioTimerDiv64:
                pstcCsio->SACSR_f.TDIV = 6u;
                break;
            case CsioTimerDiv128: 
                pstcCsio->SACSR_f.TDIV = 7u;
                break;
            case CsioTimerDiv256: 
                pstcCsio->SACSR_f.TDIV = 8u;
                break;    
            default:
                return ErrorInvalidParameter;
        }
        
        /* Transfer count */
        pstcCsio->TBYTE0 = pstcConfig->stcSerialTimer.u8TransferByteCnt;
        
        /* Timer compare value */
        pstcCsio->STMCR = pstcConfig->stcSerialTimer.u16CompareVal;
        
        /* Enable sync transfer */
        pstcCsio->SACSR_f.TSYNE = 1;
    }
    
    if(pstcConfig->bEnChipSelection == TRUE)  /* Use chip selection function? */
    {
        if(pstcConfig->enMsMode == CsioMaster)
        {
            /* Select CS start pin */
            switch(pstcConfig->stcCsConfig.enCsStartPin)
            {
                case CsPinScs0:
                    pstcCsio->SCSCR_f.SST = 0u;
                    break;
                case CsPinScs1:
                    pstcCsio->SCSCR_f.SST = 1u;
                    break;
                case CsPinScs2:
                    pstcCsio->SCSCR_f.SST = 2u;
                    break;
                case CsPinScs3:
                    pstcCsio->SCSCR_f.SST = 3u;
                    break;
                default:
                    return ErrorInvalidParameter;
            }
            
            /* Select CS end pin */
            switch(pstcConfig->stcCsConfig.enCsEndPin)
            {
                case CsPinScs0:
                    pstcCsio->SCSCR_f.SED = 0u;
                    break;
                case CsPinScs1:
                    pstcCsio->SCSCR_f.SED = 1u;
                    break;
                case CsPinScs2:
                    pstcCsio->SCSCR_f.SED = 2u;
                    break;
                case CsPinScs3:
                    pstcCsio->SCSCR_f.SED = 3u;
                    break;
                default:
                    return ErrorInvalidParameter;
            }
            
            /* Set transfer count of each SCS pin */
            pstcCsio->TBYTE0 = pstcConfig->stcCsConfig.u8Scs0TransferByteCnt;
            pstcCsio->TBYTE1 = pstcConfig->stcCsConfig.u8Scs1TransferByteCnt;
            pstcCsio->TBYTE2 = pstcConfig->stcCsConfig.u8Scs2TransferByteCnt;
            pstcCsio->TBYTE3 = pstcConfig->stcCsConfig.u8Scs3TransferByteCnt;
            
            /* Select CS active level (only for SCS0) */
            switch(pstcConfig->stcCsConfig.enLevel)
            {
                case CsHighActive:
                    pstcCsio->SCSCR_f.CSLVL = 0;
                    break;
                case CsLowActive:
                    pstcCsio->SCSCR_f.CSLVL = 1;
                    break;
                default:
                    return ErrorInvalidParameter;
            }
            
            /* Active hold or not */
            pstcCsio->SCSCR_f.SCAM = pstcConfig->stcCsConfig.bActiveHold;
            
            /* Set chip selection clock */
            switch(pstcConfig->stcCsConfig.enClkDiv)
            {
                case CsClkNoDiv:
                    pstcCsio->SCSCR_f.CDIV = 0u;
                    break;
                case CsClkDiv2:
                    pstcCsio->SCSCR_f.CDIV = 1u;
                    break;
                case CsClkDiv4:
                    pstcCsio->SCSCR_f.CDIV = 2u;
                    break;
                case CsClkDiv8:
                    pstcCsio->SCSCR_f.CDIV = 3u;
                    break;
                case CsClkDiv16:
                    pstcCsio->SCSCR_f.CDIV = 4u;
                    break;
                case CsClkDiv32:
                    pstcCsio->SCSCR_f.CDIV = 5u;
                    break;
                case CsClkDiv64:
                    pstcCsio->SCSCR_f.CDIV = 6u;
                    break;
                default:
                    return ErrorInvalidParameter;
            }
            /* Set chip selection timing */
            pstcCsio->SCSTR1 = pstcConfig->stcCsConfig.u8CsSetupDelayTime;
            pstcCsio->SCSTR0 = pstcConfig->stcCsConfig.u8CsHoldDelayTime;
            pstcCsio->SCSTR32 = pstcConfig->stcCsConfig.u16CsDeselectTime;
        }
        
        /* Enable SCS pin */
        pstcCsio->SCSCR_f.CSEN0 = ((pstcConfig->stcCsConfig.bScs0En == TRUE) ? 1 : 0);
        pstcCsio->SCSCR_f.CSEN1 = ((pstcConfig->stcCsConfig.bScs1En == TRUE) ? 1 : 0);
        pstcCsio->SCSCR_f.CSEN2 = ((pstcConfig->stcCsConfig.bScs2En == TRUE) ? 1 : 0);
        pstcCsio->SCSCR_f.CSEN3 = ((pstcConfig->stcCsConfig.bScs3En == TRUE) ? 1 : 0);
        
        if(pstcConfig->enMsMode == CsioMaster)
        {
            /* Enable CS pins output */
            pstcCsio->SCSCR_f.CSOE = 1;
        }
        else
        {
            /* Disable CS pins output */
            pstcCsio->SCSCR_f.CSOE = 0;
        }
    }
    
    if(pstcConfig->bEnableFifo == TRUE) /* Use FIFO function? */
    {
        /* Reset FIFO */
        pstcCsio->FCR0_f.FCL1 = 1;
        pstcCsio->FCR0_f.FCL2 = 1;
        /* Enable FIFO receive Idle detection */
        pstcCsio->FCR1_f.FRIIE = 1;
        /* Selection TX and RX FIFO  */
        switch(pstcConfig->stcFifoConfig.enFifoSel)
        {
            case MfsTxFifo1RxFifo2:
                pstcCsio->FCR1_f.FSEL = 0;
                break;
            case MfsTxFifo2RxFifo1:
                pstcCsio->FCR1_f.FSEL = 1;
                break;
            default:
               return (ErrorInvalidParameter);
        }
        /* Set FIFO count */
        pstcCsio->FBYTE1 = pstcConfig->stcFifoConfig.u8ByteCount1;
        pstcCsio->FBYTE2 = pstcConfig->stcFifoConfig.u8ByteCount2;
        /* Enable FIFO  */
        pstcCsio->FCR0_f.FE1 = 1;
        pstcCsio->FCR0_f.FE2 = 1;
    }
    
    return (Ok);
} /* Mfs_Csio_Init */

/**
 ******************************************************************************
 ** \brief Deinitialisation of a MFS module activating as CSIO.
 **
 ** All used CSIO register are reset to their default values.
 **  
 ** \param [in]   pstcCsio        Pointer to CSIO instance register area
 ** 
 ** \retval Ok                        Process successfully done.
 ** \retval ErrorInvalidParameter     If one of following conditions are met:
 **             - pstcCsio == NULL
 **             - pstcMfsInternData == NULL (invalid or disabled MFS unit
 **                                          (PDL_PERIPHERAL_ENABLE_MFS)) 
 **
 ******************************************************************************/
en_result_t Mfs_Csio_DeInit(volatile stc_mfsn_csio_t* pstcCsio)
{
    en_result_t            enResult;
    stc_mfs_intern_data_t* pstcMfsInternData;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsCsioGetInternDataPtr(pstcCsio);
    /* ... and check */
    if (NULL == pstcMfsInternData)
    {
        enResult = ErrorInvalidParameter;
    }
    else
    {
        /* First of all disable receiver, transmitter and deinit interrupts */
        pstcCsio->SCR_f.TXE = FALSE;
        pstcCsio->SCR_f.RXE = FALSE;

        /* Baud Rate Generation Reload Reset */
        pstcCsio->BGR = 0u;
            
        /* Serial Mode Register clear all bits (valid in any mode) */
        pstcCsio->SMR = 0u;
           
        /* Reset Mfs receive and transmit bit to default */
        /* and clear all error flags */
        pstcCsio->SCR = 0u;

        /* Clear reception Errors */
        pstcCsio->SSR_f.REC = TRUE;
           
        /* Reset all other used register to default value */
        pstcCsio->SSR  = 0u;
        pstcCsio->ESCR = 0u;
        pstcCsio->FCR0 = 0u;
        pstcCsio->FCR1 = 0u;
        pstcCsio->SACSR = 0u;
        pstcCsio->STMCR = 0u;
        pstcCsio->SCSCR = 0u;
        pstcCsio->SCSTR0 = 0;
        pstcCsio->SCSTR1 = 0;
        pstcCsio->SCSTR32 = 0;
        pstcCsio->TBYTE0 = 0u;
        pstcCsio->TBYTE1 = 0u;
        pstcCsio->TBYTE2 = 0u;
        pstcCsio->TBYTE3 = 0u;

        enResult = Ok;
    }

    return (enResult);
} /* Mfs_Csio_DeInit */

/**
 ******************************************************************************
 ** \brief Set the baudrate of CSIO
 **
 ** \param [in] pstcCsio    Pointer to CSIO instance   
 ** \param [in] u32BaudRate Baudrate value [bps]
 ** 
 ** \retval Ok                    CSIO baud rate has been successfully modified
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL
 **                               - u32BaudRate < 3         
 ** \note
 ** The CSIO baud rate can be initialized in the Mfs_Csio_Init() and be modified
 ** in the funciton.
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_SetBaudRate(volatile stc_mfsn_csio_t* pstcCsio,
                                 uint32_t u32BaudRate)
{
    uint32_t u32Pclk1;
  
    if ((NULL == pstcCsio) || (u32BaudRate < 3))
    {
        return ErrorInvalidParameter;
    }
    
    u32Pclk1 = __HCLK/(1ul << (APBC1_PSR_Val & 0x03u)); /* MFS is attached on APB1 bus */
    
    pstcCsio->BGR_f.BGR = (u32Pclk1/u32BaudRate) - 1;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Set the compare value of CSIO serial timer
 **
 ** \param [in] pstcCsio        Pointer to CSIO instance   
 ** \param [in] u16CompareValue Compare value
 ** 
 ** \retval Ok                    Compare value has been successfully modified
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL
 **
 ** \note
 ** The compare value of CSIO serial timer can be initialized in 
 ** the Mfs_Csio_Init() and be modified in this function.
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_SetTimerCompareValue(volatile stc_mfsn_csio_t* pstcCsio,
                                          uint16_t u16CompareValue)
{
    if (NULL == pstcCsio)
    {
        return ErrorInvalidParameter;
    }
    
    pstcCsio->STMCR = u16CompareValue;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Set the transfer byte count of a selected chip selection pin
 **
 ** \param [in] pstcCsio        Pointer to CSIO instance   
 ** \param [in] enCsPin         CS pin index
 ** \param [in] u8ByteCnt       Transfer byte count of a CS pin 
 ** 
 ** \retval Ok                    Transfer byte count has been successfully modified
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL
 **
 ** \note
 ** The transfer byte count of chip selection pin can be initialized in 
 ** the Mfs_Csio_Init() and be modified in this funciton.
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_SetCsTransferByteCount(volatile stc_mfsn_csio_t* pstcCsio,
                                            en_cs_pin_sel_t enCsPin,
                                            uint8_t u8ByteCnt)
{
    if (NULL == pstcCsio)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enCsPin)
    {
        case CsPinScs0:
            pstcCsio->TBYTE0 = u8ByteCnt;
            break;
        case CsPinScs1:
            pstcCsio->TBYTE1 = u8ByteCnt;
            break;
        case CsPinScs2:
            pstcCsio->TBYTE2 = u8ByteCnt;
            break;
        case CsPinScs3:
            pstcCsio->TBYTE3 = u8ByteCnt;
            break;
        default:  
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Set the hold status of chip selection pin
 **
 ** \param [in] pstcCsio        Pointer to CSIO instance   
 ** \param [in] bHold 
 ** \arg  FALSE    Make the CS pin inactive if specified count bytes are 
 **                transferred
 ** \arg  TRUE     Hold the CS pin status even if specified count bytes are 
 **                transferred
 ** 
 ** \retval Ok                    Hold status of chip selection pin is set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_SetCsHoldStatus(volatile stc_mfsn_csio_t* pstcCsio, 
                                     boolean_t bHold)
{
    if (NULL == pstcCsio)
    {
        return ErrorInvalidParameter;
    }
    
    pstcCsio->SCSCR_f.SCAM = (bHold == TRUE) ? 1 : 0;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Set the transfer byte count of CSIO serial timer
 **
 ** \param [in] pstcCsio        Pointer to CSIO instance   
 ** \param [in] u8ByteCnt       Transfer byte count
 ** 
 ** \retval Ok                    Transfer byte count has been successfully modified
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL
 **
 ** \note
 ** The transfer byte count of CSIO serial timer can be initialized in 
 ** the Mfs_Csio_Init() and be modified in this function.
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_SetTimerTransferByteCount(volatile stc_mfsn_csio_t* pstcCsio,
                                               uint8_t u8ByteCnt)
{
    if (NULL == pstcCsio)
    {
        return ErrorInvalidParameter;
    }
    
    pstcCsio->TBYTE0 = u8ByteCnt;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Enable CSIO functions
 **
 ** \param [in] pstcCsio   Pointer to CSIO instance   
 ** \param [in] enFunc     CSIO function types
 ** \arg   CsioTx            CSIO transfer function
 ** \arg   CsioRx            CSIO receive function
 ** \arg   CsioSerialTimer   CSIO serial timer function
 ** \arg   CsioCsErrOccur    CSIO chip selection error detection function 
 ** 
 ** \retval Ok                    Function has been enabled normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL                
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_EnableFunc(volatile stc_mfsn_csio_t* pstcCsio, en_csio_func_t enFunc)
{
    if (NULL == pstcCsio)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFunc)
    {
        case CsioTx:
            pstcCsio->SCR_f.TXE = 1;
            break;
        case CsioRx:
            pstcCsio->SCR_f.RXE = 1;
            break;
        case CsioSerialTimer:
            pstcCsio->SACSR_f.TMRE = 1;
            break;
        case CsioCsErrOccur:
            pstcCsio->SACSR_f.TBEEN = 1;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable CSIO functions
 **
 ** \param [in] pstcCsio   Pointer to CSIO instance   
 ** \param [in] enFunc     CSIO function types
 ** \arg   CsioTx            CSIO transfer function
 ** \arg   CsioRx            CSIO receive function
 ** \arg   CsioSerialTimer   CSIO transfer function
 ** \arg   CsioCsErrOccur    CSIO receive function 
 ** 
 ** \retval Ok                    Function has been disabled normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL              
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_DisableFunc(volatile stc_mfsn_csio_t* pstcCsio, en_csio_func_t enFunc)
{
    if (NULL == pstcCsio)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFunc)
    {
        case CsioTx:
            pstcCsio->SCR_f.TXE = 0;
            break;
        case CsioRx:
            pstcCsio->SCR_f.RXE = 0;
            break;
        case CsioSerialTimer:
            pstcCsio->SACSR_f.TMRE = 0;
            break;
        case CsioCsErrOccur:
            pstcCsio->SACSR_f.TBEEN = 0;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get status of CSIO according to status type
 **
 ** \param [in] pstcCsio     Pointer to CSIO instance   
 ** \param [in] enStatus     CSIO status type
 ** \arg   CsioOverrunError  CSIO parity error
 ** \arg   CsioRxFull        CSIO receive buffer full
 ** \arg   CsioTxEmpty       CSIO tranfer buffer empty
 ** \arg   CsioTxIdle        CSIO tranfer idle status
 ** \arg   CsioTxFifoRequest CSIO transfer FIFO request
 ** \arg   CsioCsErrIntFlag  CSIO chip selection error occurrance
 ** \arg   CsioTimerIntFlag  CSIO serial timer interrupt flag
 ** 
 ** \retval FALSE                 If one of following conditions are met:
 **                               - No CSIO overrun error [enStatus = CsioOverrunError]
 **                               - CSIO receive buffer is not full [enStatus = CsioRxFull]
 **                               - CSIO tranfer buffer is not empty [enStatus = CsioTxEmpty]
 **                               - CSIO tranfer is on-going [enStatus = CsioTxIdle]
 **                               - No CSIO transfer FIFO request [enStatus = CsioTxFifoRequest]
 **                               - CSIO chip selection error doesn't occur [enStatus = CsioCsErrIntFlag]
 **                               - CSIO serial timer interrupt doesn't occur [enStatus = CsioTimerIntFlag]
 ** \retval TRUE                 If one of following conditions are met:
 **                               - CSIO overrun error occurs [enStatus = CsioOverrunError]
 **                               - CSIO receive buffer is full [enStatus = CsioRxFull]
 **                               - CSIO tranfer buffer is empty [enStatus = CsioTxEmpty]
 **                               - CSIO tranfer is idle [enStatus = CsioTxIdle]
 **                               - CSIO transfer FIFO request issues [enStatus = CsioTxFifoRequest]
 **                               - CSIO chip selection error occurs  [enStatus = CsioCsErrIntFlag]
 **                               - CSIO serial timer interrupt occurs [enStatus = CsioTimerIntFlag]
 ** 
 ******************************************************************************/
boolean_t Mfs_Csio_GetStatus(volatile stc_mfsn_csio_t* pstcCsio, 
                               en_csio_status_t enStatus)
{
    boolean_t bRet = FALSE;
      
    switch(enStatus)
    {
        case CsioOverrunError:
            bRet = (pstcCsio->SSR_f.ORE == 1) ? TRUE : FALSE;
            break;
        case CsioRxFull:
            bRet = (pstcCsio->SSR_f.RDRF == 1) ? TRUE : FALSE;
            break;
        case CsioTxEmpty:
            bRet = (pstcCsio->SSR_f.TDRE == 1) ? TRUE : FALSE;
            break;
        case CsioTxIdle:
            bRet = (pstcCsio->SSR_f.TBI == 1) ? TRUE : FALSE;
            break;
        case CsioTxFifoRequest:
            bRet = (pstcCsio->FCR1_f.FDRQ == 1) ? TRUE : FALSE;
            break;
        case CsioCsErrIntFlag:
            bRet = (pstcCsio->SACSR_f.CSE == 1) ? TRUE : FALSE;
            break;
        case CsioTimerIntFlag:
            bRet = (pstcCsio->SACSR_f.TINT == 1) ? TRUE : FALSE;
            break;
        default:
            break;
    }
    
    return bRet;
}

/**
 ******************************************************************************
 ** \brief Clear status of CSIO according to status type
 **
 ** \param [in] pstcCsio     Pointer to CSIO instance   
 ** \param [in] enStatus     UART status type
 ** \arg   CsioOverrunError  CSIO overrun error
 ** \arg   CsioRxFull        CSIO receive buffer full
 ** \arg   CsioTxEmpty       CSIO tranfer buffer empty
 ** \arg   CsioTxIdle        CSIO tranfer idle status
 ** \arg   CsioTxFifoRequest CSIO transfer FIFO request
 ** \arg   CsioCsErrIntFlag  CSIO chip selection error occurrance
 ** 
 ** \retval Ok                    Status has been cleared normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcUart == NULL
 ** \note  The following status can only be cleared by hardware behavior:
 **        - CsioRxFull
 **        - CsioTxEmpty
 **        - CsioTxIdle
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_ClrStatus(volatile stc_mfsn_csio_t* pstcCsio,
                               en_csio_status_t enStatus)
{
    if (NULL == pstcCsio)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enStatus)
    {
        case CsioOverrunError:
            pstcCsio->SSR_f.REC = 1;
            break;
        case CsioRxFull:
        case CsioTxEmpty:
        case CsioTxIdle:
            break;
        case CsioTxFifoRequest:
            pstcCsio->FCR1_f.FDRQ = 0;
            break;
        case CsioCsErrIntFlag:
            pstcCsio->SACSR_f.CSE = 0;;
            break;
        case CsioTimerIntFlag:
            pstcCsio->SACSR_f.TINT = 0;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Write CSIO data buffer
 **
 ** \param [in] pstcCsio   Pointer to CSIO instance   
 ** \param [in] u16Data    Send data
 ** \param [in] bSotEn     SOT output enable 
 ** \arg   FALSE  Mask SOT output
 ** \arg   TRUE   Enable SOT output
 ** 
 ** \retval Ok                    Data has been successfully sent
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_SendData(volatile stc_mfsn_csio_t* pstcCsio, 
                              uint16_t u16Data,
                              boolean_t bSotEn)
{
    if (NULL == pstcCsio)
    {
        return ErrorInvalidParameter;
    }
    
    (bSotEn == TRUE) ? (pstcCsio->SMR_f.SOE = 1) : (pstcCsio->SMR_f.SOE = 0);
    
    pstcCsio->TDR = u16Data;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Read CSIO data buffer
 **
 ** \param [in] pstcCsio   Pointer to CSIO instance   
 ** 
 ** \retval Receive data        
 ** 
 ******************************************************************************/
uint16_t Mfs_Csio_ReceiveData(volatile stc_mfsn_csio_t* pstcCsio)
{
    return pstcCsio->RDR;
}

/**
 ******************************************************************************
 ** \brief Reset CSIO FIFO
 **
 ** \param [in] pstcCsio   Pointer to CSIO instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** 
 ** \retval Ok                    FIFO has been successfully reset
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_ResetFifo (volatile stc_mfsn_csio_t* pstcCsio, 
                                en_mfs_fifo_t enFifo)
{
    if (NULL == pstcCsio)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFifo)
    {
        case MfsFifo1:
            pstcCsio->FCR0_f.FCL1 = 1;
            break;
        case MfsFifo2:
            pstcCsio->FCR0_f.FCL2 = 1;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Set CSIO FIFO count
 **
 ** \param [in] pstcCsio   Pointer to CSIO instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** \param [in] u8Count    FIFO count
 ** 
 ** \retval Ok                    FIFO count has been successfully set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcCsio == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_Csio_SetFifoCount(volatile stc_mfsn_csio_t* pstcCsio,
                                  en_mfs_fifo_t enFifo,
                                  uint8_t u8Count)
{
    if (NULL == pstcCsio)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFifo)
    {
        case MfsFifo1:
            pstcCsio->FBYTE1 = u8Count;
            break;
        case MfsFifo2:
            pstcCsio->FBYTE2 = u8Count;
            break;
        default:
           return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get CSIO FIFO count
 **
 ** \param [in] pstcCsio   Pointer to CSIO instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** 
 ** \retval FIFO count      
 ** 
 ** This function gets the current data count in selected FIFO.
 **
 ** \note 0xFF will be return value if FIFO index is error.
 **
 ******************************************************************************/
uint8_t Mfs_Csio_GetFifoCount(volatile stc_mfsn_csio_t* pstcCsio,
                              en_mfs_fifo_t enFifo)
{
    uint8_t u8FifoCnt = 0xFFu;
    
    switch(enFifo)
    {
        case MfsFifo1:
            u8FifoCnt = pstcCsio->FBYTE1;
            break;
        case MfsFifo2:
            u8FifoCnt = pstcCsio->FBYTE2;
            break;
        default:
           break;
    }
    
    return u8FifoCnt;
}

/******************************************************************************
 * I2C driver functions
 ******************************************************************************/

/**
 ******************************************************************************
 ** \brief Return the internal data for a certain I2C instance.
 **
 ** \param pstcI2c Pointer to I2C instance
 **
 ** \return Pointer to internal data or NULL if instance is not enabled (or not known)
 **
 ******************************************************************************/
static stc_mfs_intern_data_t* MfsI2cGetInternDataPtr(volatile stc_mfsn_i2c_t* pstcI2c)
{
    stc_mfs_intern_data_t* pstcInternDataPtr = NULL;
    uint32_t u32Instance;

    if (NULL != pstcI2c)
    {
        for (u32Instance = 0u; u32Instance < (uint32_t)MfsInstanceIndexMax; u32Instance++)
        {
            if (pstcI2c == m_astcMfsInstanceDataLut[u32Instance].stcInstance.pstcI2cInstance)
            {
                pstcInternDataPtr = &m_astcMfsInstanceDataLut[u32Instance].stcInternData;
                break;
            }
        }
            
    }

    return (pstcInternDataPtr);
} /* MfsGetInternDataPtr */


#if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON) ||  \
    (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON) ||  \
    (PDL_INTERRUPT_ENABLE_MFS2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS10 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS3 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS11 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS4 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS12 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS5 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS13 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS6 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS14 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS7 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS15 == PDL_ON) 
/**
 ******************************************************************************
 ** \brief Return instance index for a certain I2C instance.
 **
 ** \param pstcI2c Pointer to I2C instance
 **
 ** \return Instance index
 **
 ******************************************************************************/
static en_mfs_instance_index_t MfsI2cGetInstanceIndex(volatile stc_mfsn_i2c_t* pstcI2c)
{
    en_mfs_instance_index_t enInstanceIndex = MfsInstanceIndexUnknown;
    uint32_t u32Instance;

    if (NULL != pstcI2c)
    {
        for (u32Instance = 0u; u32Instance < (uint32_t)MfsInstanceIndexMax; u32Instance++)
        {
            if (pstcI2c == m_astcMfsInstanceDataLut[u32Instance].stcInstance.pstcI2cInstance)
            {
                enInstanceIndex = m_astcMfsInstanceDataLut[u32Instance].stcInternData.enIndex;
                break;
            }
        }
            
    }

    return (enInstanceIndex);
} /* MfsI2cGetInstanceIndex */
      
      
/**
 ******************************************************************************
 ** \brief I2C receive interrupt service routine.
 ** 
 ** \param pstcI2c Pointer to I2C instance   
 ** \param pstcMfsInternData Pointer to MFS internal data structure       
 ** 
 ** This function is called on Receive Interrupt set by the MFS. 
 **
 ******************************************************************************/
void MfsI2cIrqHandlerRx( volatile stc_mfsn_i2c_t*   pstcI2c,
                         stc_mfs_intern_data_t* pstcMfsInternData)
{
    if ((NULL != pstcI2c) && (NULL != pstcMfsInternData))
    {
        if (NULL != pstcMfsInternData->stcI2cInternIntCb.pfnRxIntCb)
        {
            pstcMfsInternData->stcI2cInternIntCb.pfnRxIntCb();
        }
    }
} /* MfsIrqHandlerRx */

/**
 ******************************************************************************
 ** \brief I2C transfer interrupt service routine.
 ** 
 ** \param pstcI2c           Pointer to I2C instance   
 ** \param pstcMfsInternData Pointer to MFS internal data structure       
 ** 
 ** This function is called on transfer Interrupt set by the MFS. 
 **
 ******************************************************************************/
void MfsI2cIrqHandlerTx( volatile stc_mfsn_i2c_t*   pstcI2c, 
                         stc_mfs_intern_data_t* pstcMfsInternData)
{
    if(pstcI2c->SSR_f.TDRE == 1)
    {
        if (NULL != pstcMfsInternData->stcI2cInternIntCb.pfnTxIntCb)
        {
            pstcMfsInternData->stcI2cInternIntCb.pfnTxIntCb();
        }
    }
    
    if(pstcI2c->SSR_f.TBI == 1)
    {
        if (NULL != pstcMfsInternData->stcI2cInternIntCb.pfnTxIdleCb)
        {
            pstcMfsInternData->stcI2cInternIntCb.pfnTxIdleCb();
        }
    }
    
    if(pstcI2c->FCR1_f.FDRQ == 1)
    {
        if (NULL != pstcMfsInternData->stcI2cInternIntCb.pfnTxFifoIntCb)
        {
            pstcMfsInternData->stcI2cInternIntCb.pfnTxFifoIntCb();
        }
        
        pstcI2c->FCR1_f.FDRQ = 0;
    }
} /* MfsIrqHandlerTx */
          
/**
 ******************************************************************************
 ** \brief I2C status interrupt service routine.
 ** 
 ** \param pstcI2c           Pointer to I2C instance   
 ** \param pstcMfsInternData Pointer to MFS internal data structure       
 ** 
 ** This function is called on transfer Interrupt set by the MFS. 
 **
 ******************************************************************************/
void MfsI2cIrqHandlerStatus( volatile stc_mfsn_i2c_t*   pstcI2c, 
                              stc_mfs_intern_data_t* pstcMfsInternData)
{
    if ((NULL != pstcI2c) && (NULL != pstcMfsInternData))
    {
        if(pstcI2c->IBCR_f.INT == 1)
        {
            if (NULL != pstcMfsInternData->stcI2cInternIntCb.pfnTxRxCb)
            {
                pstcMfsInternData->stcI2cInternIntCb.pfnTxRxCb();
            }
            
            pstcI2c->IBCR_f.INT = 0;
        }
        
        if(pstcI2c->IBSR_f.SPC == 1)
        {
            if (NULL != pstcMfsInternData->stcI2cInternIntCb.pfnStopDetectCb)
            {
                pstcMfsInternData->stcI2cInternIntCb.pfnStopDetectCb();
            }
            
            pstcI2c->IBSR_f.SPC = 0;
        }
    }
}

/**
 ******************************************************************************
 ** \brief Enable I2C interrupts
 **
 ** \param [in] pstcI2c    Pointer to I2C instance   
 ** \param [in] pstcInt    Pointer to the selection structure of I2C interrupt
 ** \param [in] pstcIntCb  Pointer to the structure of I2C interrupt callback 
 **                        functions 
 ** 
 ** \retval Ok                    Interrupts has been enabled and callback 
 **                               functions are set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcI2c == NULL
 **                               - pstcInt == NULL
 **                               - pstcIntCb == NULL
 **                               - Other invalid configuration                   
 **
 ** The NVIC IRQ is also enabled in this function.
 ** 
 ******************************************************************************/
en_result_t Mfs_I2c_EnableInt(volatile stc_mfsn_i2c_t* pstcI2c, 
                              stc_i2c_int_sel_t* pstcInt,
                              stc_i2c_int_cb_t* pstcIntCb)
{
    stc_mfs_intern_data_t* pstcMfsInternData;
    en_mfs_instance_index_t enIndex;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsI2cGetInternDataPtr(pstcI2c);
    enIndex = MfsI2cGetInstanceIndex(pstcI2c);
  
    if ((NULL == pstcMfsInternData) ||  
        (NULL == pstcInt) ||
        (NULL == pstcIntCb) ||
        (MfsInstanceIndexUnknown == enIndex)  )
    {
        return (ErrorInvalidParameter);
    }
    
    if(pstcInt->bTxInt == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnTxIntCb = pstcIntCb->pfnTxIntCb;
        pstcI2c->SMR_f.TIE = 1;
    }
    
    if(pstcInt->bRxInt == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnRxIntCb = pstcIntCb->pfnRxIntCb;
        pstcI2c->SMR_f.RIE = 1;
    }
    
    if(pstcInt->bTxFifoInt == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnTxFifoIntCb = pstcIntCb->pfnTxFifoIntCb;
        pstcI2c->FCR1_f.FTIE = 1;
    }
    
    if(pstcInt->bTxIdle == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnTxIdleCb = pstcIntCb->pfnTxIdleCb;
        pstcI2c->SSR_f.TBIE = 1;
    }
    
    if(pstcInt->bTxRxInt == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnTxRxCb = pstcIntCb->pfnTxRxCb;
        pstcI2c->IBCR_f.INTE = 1;
    }
    
    if(pstcInt->bStopDetectInt == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnStopDetectCb = pstcIntCb->pfnStopDetectCb;
        pstcI2c->IBCR_f.CNDE = 1;
    }
        
    if((pstcInt->bTxRxInt     == TRUE) || (pstcInt->bStopDetectInt   == TRUE) ||
       (pstcInt->bTxInt       == TRUE) || (pstcInt->bTxFifoInt       == TRUE) || 
       (pstcInt->bTxIdle      == TRUE)  )
    {
        Mfs_InitIrq(enIndex, TRUE);
    }
    
    if(pstcInt->bRxInt == TRUE)
    {
        Mfs_InitIrq(enIndex, FALSE);
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable I2C interrupts
 **
 ** \param [in] pstcI2c    Pointer to I2C instance   
 ** \param [in] pstcInt    Pointer to the selection structure of I2C interrupt
 **
 ** \retval Ok                    Interrupts has been disabled and callback 
 **                               functions are set to NULL
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcI2c == NULL
 **                               - pstcInt == NULL
 **                               - pstcIntCb == NULL
 **                               - Other invalid configuration                   
 **
 ** The NVIC IRQ is also disabled in this function if all interrupt sources
 ** are disable.
 ** 
 ******************************************************************************/
en_result_t Mfs_I2c_DisableInt(volatile stc_mfsn_i2c_t* pstcI2c, 
                               stc_i2c_int_sel_t* pstcInt)
{
    stc_mfs_intern_data_t* pstcMfsInternData;
    en_mfs_instance_index_t enIndex;
    uint8_t u8Ch;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsI2cGetInternDataPtr(pstcI2c);
    enIndex = MfsI2cGetInstanceIndex(pstcI2c);
    
    if ((NULL == pstcMfsInternData) ||  
        (NULL == pstcInt) ||
        (MfsInstanceIndexUnknown == enIndex)  )
    {
        return (ErrorInvalidParameter);
    }
    
    u8Ch = I2cInstanceToIndex(pstcI2c);
    
    if(pstcInt->bTxInt == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnTxIntCb = NULL;
        pstcI2c->SMR_f.TIE = 0;
    }
    
    if(pstcInt->bRxInt == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnRxIntCb = NULL;
        pstcI2c->SMR_f.RIE = 0;
    }
    
    if(pstcInt->bTxFifoInt == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnTxFifoIntCb = NULL;
        pstcI2c->FCR1_f.FTIE = 0;
    }
    
    if(pstcInt->bTxIdle == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnTxIdleCb = NULL;
        pstcI2c->SSR_f.TBIE = 0;
    }
    
    if(pstcInt->bTxRxInt == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnTxRxCb = NULL;
        pstcI2c->IBCR_f.INTE = 0;
    }
    
    if(pstcInt->bStopDetectInt == TRUE)
    {
        pstcMfsInternData->stcI2cInternIntCb.pfnStopDetectCb = NULL;
        pstcI2c->IBCR_f.CNDE = 0;
    }
    
    // Only when TX and RX interrupts and DMA interrupt(share same IRQ with MFS ch.14, ch.15) 
    // are all disabled, disable TX IRQ
    do
    {
        if(pstcI2c->IBCR_f.INTE == 1)
        {
            break;
        }
        
        if(pstcI2c->IBCR_f.CNDE == 1)
        {
            break;
        }
        
        if(pstcI2c->SMR_f.TIE == 1)
        {
            break;
        }
        
        if(pstcI2c->SMR_f.RIE == 1)
        {
            break;
        }
        
        if(pstcI2c->SSR_f.TBIE == 1)
        {
            break;
        }
        
        if(pstcI2c->FCR1_f.FTIE == 1)
        {
            break;
        }
  
        if(u8Ch == 14u)
        {
            if((FM0P_DMAC->DMACB1_f.CI == 1) || (FM0P_DMAC->DMACB1_f.EI == 1))
            {
                break;
            }
        }
   
        if(u8Ch == 15u)
        {
            if((FM0P_DMAC->DMACB3_f.CI == 1) || (FM0P_DMAC->DMACB3_f.EI == 1))
            {
                break;
            }
        }

        Mfs_DeInitIrq(enIndex, TRUE);
      
    }while(0);
    
    return Ok;
}

#endif   

/**
 ******************************************************************************
 ** \brief Initialization of a MFS module to activate as I2C.
 **
 ** Set registers to active MFS as I2C.
 **  
 ** \param [in]   pstcI2c         Pointer to I2C instance register area
 ** \param [in]   pstcConfig      MFS I2C configuration
 ** 
 ** \retval Ok                        Process successfully done.
 ** \retval ErrorInvalidParameter     If one of following conditions are met:
 **             - pstcI2c == NULL
 **             - pstcConfig == NULL
 **             - pstcMfsInternData == NULL (invalid or disabled MFS unit
 **                                          (PDL_PERIPHERAL_ENABLE_MFS)) 
 **
 ******************************************************************************/
en_result_t Mfs_I2c_Init(volatile stc_mfsn_i2c_t*        pstcI2c, 
                         const stc_mfs_i2c_config_t* pstcConfig)
{
    stc_mfs_intern_data_t* pstcMfsInternData;

    /* Preset local register variables to zero */
    stc_mfs_i2c_smr_field_t       stcSMR   = { 0 };
    stc_mfs_i2c_ibcr_field_t  stcIBCR  = { 0 };
    stc_mfs_i2c_isba_field_t  stcISBA  = { 0 };
    stc_mfs_i2c_ismk_field_t  stcISMK  = { 0 };

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsI2cGetInternDataPtr(pstcI2c);
    
    /* Parameter check and get ptr to internal data struct */
    if ((NULL == pstcMfsInternData)
    ||  (NULL == pstcConfig)
       )
    {
        return (ErrorInvalidParameter);
    }
    
    /* Set the MFS mode in the internal structure */
    pstcMfsInternData->enMode = MfsI2cMode;

    /* Set I2C mode */
    switch (pstcConfig->enMsMode)
    {
        case I2cMaster:
        case I2cSlave:
            stcSMR.MD = 4u;
            break;
        default:
            return (ErrorInvalidParameter);
    }

    /* Ack enable */
    stcIBCR.ACKE = TRUE;

    /* Slave mode */
    if (I2cSlave == pstcConfig->enMsMode)
    {
        /* Set slave address */
        stcISBA.SA = pstcConfig->u8SlaveAddr;
        /* Enable slave address detection */
        stcISBA.SAEN = TRUE;
        /* Set slave address bit mask */
        stcISMK.SM = pstcConfig->u8SlaveMaskAddr;
    }
    
    /* Enable I2C*/
    stcISMK.EN = TRUE;

    /* now setup hardware with correct mode first and then go on with */
    /*   bit settings */
    pstcI2c->SMR_f = stcSMR;

    /* I2C disable before other registers are set. */
    pstcI2c->ISMK  = 0u;

    /* Set baud rate generation */
    (void)Mfs_I2c_SetBaudRate(pstcI2c, pstcConfig->u32BaudRate);

    /* Set registers value */
    pstcI2c->IBCR_f  = stcIBCR;
    pstcI2c->ISBA_f  = stcISBA;
    pstcI2c->ISMK_f  = stcISMK;
    
    /* Use DMA or not */
    if(pstcConfig->bDmaEnable == TRUE)
    {
        pstcI2c->SSR_f.DMA = 1;
    }
    else
    {
        pstcI2c->SSR_f.DMA = 0;
    }

    if(pstcConfig->bEnableFifo == TRUE) /* Use FIFO function? */
    {
        /* Reset FIFO */
        pstcI2c->FCR0_f.FCL1 = 1;
        pstcI2c->FCR0_f.FCL2 = 1;
        /* Enable FIFO receive Idle detection */
        pstcI2c->FCR1_f.FRIIE = 1;
        /* Selection TX and RX FIFO  */
        switch(pstcConfig->stcFifoConfig.enFifoSel)
        {
            case MfsTxFifo1RxFifo2:
                pstcI2c->FCR1_f.FSEL = 0;
                break;
            case MfsTxFifo2RxFifo1:
                pstcI2c->FCR1_f.FSEL = 1;
                break;
            default:
               return (ErrorInvalidParameter);
        }
        /* Set FIFO count */
        pstcI2c->FBYTE1 = pstcConfig->stcFifoConfig.u8ByteCount1;
        pstcI2c->FBYTE2 = pstcConfig->stcFifoConfig.u8ByteCount2;
        /* Enable FIFO  */
        pstcI2c->FCR0_f.FE1 = 1;
        pstcI2c->FCR0_f.FE2 = 1;
    }
    
    return (Ok);
} /* Mfs_I2c_Init */

/**
 ******************************************************************************
 ** \brief Deinitialisation of a MFS module activating as I2C.
 **
 ** All used I2C register are reset to their default values.
 **  
 ** \param [in]   pstcI2c         Pointer to I2C instance register area
 ** 
 ** \retval Ok                        Process successfully done.
 ** \retval ErrorInvalidParameter     If one of following conditions are met:
 **             - pstcI2c == NULL
 **             - pstcMfsInternData == NULL (invalid or disabled MFS unit
 **                                          (PDL_PERIPHERAL_ENABLE_MFS)) 
 **
 ******************************************************************************/
en_result_t Mfs_I2c_DeInit( volatile stc_mfsn_i2c_t* pstcI2c)
{
    en_result_t            enResult;
    stc_mfs_intern_data_t* pstcMfsInternData;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsI2cGetInternDataPtr(pstcI2c);
    /* ... and check */
    if (NULL == pstcMfsInternData)
    {
        enResult = ErrorInvalidParameter;
    }
    else
    {
        /* Disable I2C */
        pstcI2c->ISMK_f.EN = FALSE;

        /* Serial Mode Register clear all bits (valid in any mode) */
        pstcI2c->SMR = 0u;
           
        /* Reset Mfs receive and transmit bit to default */
        /* and clear all error flags */
        pstcI2c->IBCR = 0u;

        /* Clear reception Errors */
        pstcI2c->SSR_f.REC = TRUE;

        /* Reset all other used register to default value */
        pstcI2c->SSR  = 0u;
        pstcI2c->IBSR = 0u;
        pstcI2c->FCR0 = 0u;
        pstcI2c->FCR1 = 0u;

        enResult = Ok;
    }

    return (enResult);
} /* Mfs_I2c_DeInit */

/**
 ******************************************************************************
 ** \brief Set the baudrate of I2C
 **
 ** \param [in] pstcI2c     Pointer to I2C instance   
 ** \param [in] u32BaudRate Baudrate value [bps]
 ** 
 ** \retval Ok                    I2C baud rate has been successfully modified
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcI2c == NULL
 **                               - u32BaudRate < 3         
 **
 ** The I2C baud rate can be initialized in the Mfs_I2c_Init() and be modified
 ** with this function.
 ** 
 ******************************************************************************/
en_result_t Mfs_I2c_SetBaudRate(volatile stc_mfsn_i2c_t* pstcI2c,
                                uint32_t u32BaudRate)
{
    uint32_t u32Pclk1;
    boolean_t bTempBit = FALSE;
  
    if ((NULL == pstcI2c) || (u32BaudRate > 400000))
    {
        return ErrorInvalidParameter;
    }
    
    bTempBit = pstcI2c->ISMK_f.EN;
    pstcI2c->ISMK_f.EN = 1;
    u32Pclk1 = __HCLK/(1ul << (APBC1_PSR_Val & 0x03u)); /* MFS is attached on APB1 bus */ 
    pstcI2c->BGR_f.BGR = (u32Pclk1/u32BaudRate) - 1;
    pstcI2c->ISMK_f.EN = bTempBit;
   
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Generate start signal of I2C
 **
 ** \param [in] pstcI2c    Pointer to I2C instance   
 ** 
 ** \retval Ok                    Generate start signal successfully
 ** \retval ErrorInvalidParameter pstcI2c == NULL
 ** \retval ErrorTimeout          Timeout when generating a start signal
 ** 
 ******************************************************************************/
en_result_t Mfs_I2c_GenerateStart(volatile stc_mfsn_i2c_t*        pstcI2c)
{
    uint32_t u32TimeOut = SystemCoreClock;
      
    if (NULL == pstcI2c)
    {
        return ErrorInvalidParameter;
    }
    
    pstcI2c->IBCR_f.MSS = 1;
    while(u32TimeOut--)
    {
        if((pstcI2c->IBCR_f.MSS == 1) && (pstcI2c->IBCR_f.ACT_SCC == 1))
        {
            return Ok;
        }
    }
    
    return  ErrorTimeout;
}

/**
 ******************************************************************************
 ** \brief Generate restart signal of I2C
 **
 ** \param [in] pstcI2c    Pointer to I2C instance   
 ** 
 ** \retval Ok                    Generate restart signal successfully
 ** \retval ErrorInvalidParameter pstcI2c == NULL
 ** \retval ErrorTimeout          Timeout when generating a restart signal
 ** 
 ******************************************************************************/
en_result_t Mfs_I2c_GenerateRestart(volatile stc_mfsn_i2c_t*      pstcI2c)
{
    uint32_t u32TimeOut = SystemCoreClock;
    
    if (NULL == pstcI2c)
    {
        return ErrorInvalidParameter;
    }
    
    pstcI2c->IBCR_f.ACT_SCC = 1;
    
    while(u32TimeOut--)
    {
        if(pstcI2c->IBSR_f.RSC == 1)
        {
            return Ok;
        }
    }
    
    return ErrorTimeout;
}

/**
 ******************************************************************************
 ** \brief Generate stop signal of I2C
 **
 ** \param [in] pstcI2c    Pointer to I2C instance   
 ** 
 ** \retval Ok                    Generate stop signal successfully
 ** \retval ErrorInvalidParameter pstcI2c == NULL
 ** \retval ErrorTimeout          Timeout when generating a stop signal
 ** 
 ******************************************************************************/
en_result_t Mfs_I2c_GenerateStop(volatile stc_mfsn_i2c_t*      pstcI2c)
{
    uint32_t u32TimeOut = SystemCoreClock;
    
    if (NULL == pstcI2c)
    {
        return ErrorInvalidParameter;
    }
    
    while(u32TimeOut--)
    {
        pstcI2c->IBCR &= ~0xc1u;
        if((pstcI2c->IBCR_f.MSS == 0) && (pstcI2c->IBCR_f.ACT_SCC == 0))
        {
            return Ok;
        }
    }
    
    return ErrorTimeout;
}

/**
 ******************************************************************************
 ** \brief Write I2C data buffer
 **
 ** \param [in] pstcI2c    Pointer to I2C instance   
 ** \param [in] u8Data     Data to be sent
 ** 
 ** \retval Ok                    Data has been successfully sent
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcI2c == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_I2c_SendData(volatile stc_mfsn_i2c_t*      pstcI2c, uint8_t u8Data)
{
    if (NULL == pstcI2c)
    {
        return ErrorInvalidParameter;
    }
    
    pstcI2c->TDR = u8Data;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Read I2C data buffer
 **
 ** \param [in] pstcI2c   Pointer to I2C instance   
 ** 
 ** \retval Receive data        
 ** 
 ******************************************************************************/
uint8_t Mfs_I2c_ReceiveData(volatile stc_mfsn_i2c_t*      pstcI2c)
{
    return pstcI2c->RDR;
}

/**
 ******************************************************************************
 ** \brief Configure ACK signal sent to master
 **
 ** \param [in] pstcI2c    Pointer to I2C instance   
 ** \param [in] enAck      ACK to be sent
 ** \arg  I2cAck ACK will be sent as response signal
 ** \arg  I2cNAck NACK will be sent as response signal
 ** 
 ** \retval Ok                    ACK signal has been successfully configured
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcI2c == NULL           
 ** 
 ** This function is only meaningful when I2C is in slave mode
 **
 ******************************************************************************/
en_result_t Mfs_I2c_ConfigAck(volatile stc_mfsn_i2c_t* pstcI2c, en_i2c_ack_t enAck)
{
    stc_mfs_i2c_ibcr_field_t stcIbcr =  pstcI2c->IBCR_f; 
  
    if (NULL == pstcI2c)
    {
        return ErrorInvalidParameter;
    }
    
    (enAck == I2cAck) ?  (stcIbcr.ACKE = 1u) :  (stcIbcr.ACKE = 0u);
    
    stcIbcr.ACT_SCC = 0; /* Set SCC = 0 to avoid generating a restart */
    
    pstcI2c->IBCR_f = stcIbcr;
    
    return Ok;
    
}

/**
 ******************************************************************************
 ** \brief Get the ACK signal from slave
 **
 ** \param [in] pstcI2c    Pointer to I2C instance   
 ** 
 ** \retval I2cAck Receive the ACK  from I2C Slave
 ** \retval I2cNAck Receive the NACK from I2C Slave
 ** 
 ** This function applies in the I2C master mode.
 **
 ******************************************************************************/
en_i2c_ack_t   Mfs_I2c_GetAck(volatile stc_mfsn_i2c_t*        pstcI2c)
{
    en_i2c_ack_t enRet;
    
    enRet = (pstcI2c->IBSR_f.RACK == 0u) ? I2cAck : I2cNAck;
    
    return enRet; 
}

/**
 ******************************************************************************
 ** \brief Get status of I2C according to status type
 **
 ** \param [in] pstcI2c      Pointer to I2C instance   
 ** \param [in] enStatus     I2C status type
 ** \arg   I2cOverrunError          I2C overrun error
 ** \arg   I2cRxFull                I2C receive buffer full
 ** \arg   I2cTxEmpty               I2C tranfer buffer empty
 ** \arg   I2cTxFifoRequest         I2C transfer FIFO request
 ** \arg   I2cTxIdle                I2C tranfer idle status
 ** \arg   I2cFirstByteDetect       I2C first byte detection
 ** \arg   I2cReservedByteDetect    I2C reserved address detection
 ** \arg   I2cStopDetect            I2C stop condition detection
 ** \arg   I2cBusStatus             I2C bus status 
 ** \arg   I2cBusErr                I2C bus error
 ** \arg   I2cRxTxInt               I2C transfer and receive interrupt flag
 ** 
 ** \retval FALSE                 If one of following conditions are met:
 **                               - No I2C overrun error [enStatus = I2cOverrunError] 
 **                               - I2C receive buffer is not full [enStatus = I2cRxFull] 
 **                               - I2C tranfer buffer is not empty [enStatus = I2cTxEmpty] 
 **                               - I2C tranfer FIFO request doesn't issue [enStatus = I2cTxFifoRequest] 
 **                               - I2C tranfer is on-going [enStatus = I2cTxIdle] 
 **                               - I2C first byte is not detected [enStatus = I2cFirstByteDetect] 
 **                               - I2C reserved address is not detected [enStatus = I2cReservedByteDetect] 
 **                               - I2C stop condition is not detected [enStatus = I2cStopDetect] 
 **                               - I2C bus is idle [enStatus = I2cBusStatus] 
 **                               - I2C bus error doesn't occur [enStatus = I2cBusErr] 
 **                               - I2C transfer or receive is not completed. [enStatus = I2cRxTxInt] 
 **
 ** \retval TRUE                 If one of following conditions are met:
 **                               - I2C overrun error occurs  [enStatus = I2cOverrunError] 
 **                               - I2C receive buffer is full [enStatus = I2cRxFull] 
 **                               - I2C tranfer buffer is empty [enStatus = I2cTxEmpty] 
 **                               - I2C tranfer FIFO request issues [enStatus = I2cTxFifoRequest] 
 **                               - I2C tranfer is idle [enStatus = I2cTxIdle] 
 **                               - I2C first byte is detected [enStatus = I2cFirstByteDetect] 
 **                               - I2C reserved address is detected [enStatus = I2cReservedByteDetect]
 **                               - I2C stop condition is detected [enStatus = I2cStopDetect]
 **                               - I2C bus is busy [enStatus = I2cBusStatus] 
 **                               - I2C bus error occurs [enStatus = I2cBusErr] 
 **                               - I2C transfer or receive is completed. [enStatus = I2cRxTxInt] 
 ** 
 ******************************************************************************/
boolean_t Mfs_I2c_GetStatus(volatile stc_mfsn_i2c_t* pstcI2c, 
                            en_i2c_status_t enStatus)
{
    boolean_t bRet;
    
    switch(enStatus)
    {
        case I2cOverrunError:
            bRet = (pstcI2c->SSR_f.ORE == 1) ? TRUE : FALSE;
            break;  
        case I2cRxFull:
            bRet = (pstcI2c->SSR_f.RDRF == 1) ? TRUE : FALSE;
            break;  
        case I2cTxEmpty:
            bRet = (pstcI2c->SSR_f.TDRE == 1) ? TRUE : FALSE;
            break;  
        case I2cTxIdle:
            bRet = (pstcI2c->SSR_f.TBI == 1) ? TRUE : FALSE;
            break;  
        case I2cTxFifoRequest:
            bRet = (pstcI2c->FCR1_f.FDRQ == 1) ? TRUE : FALSE;
            break;  
        case I2cFirstByteDetect:
            bRet = (pstcI2c->IBSR_f.FBT == 1) ? TRUE : FALSE;
            break;  
        case I2cReservedByteDetect:
            bRet = (pstcI2c->IBSR_f.RSA == 1) ? TRUE : FALSE;
            break;  
        case I2cStopDetect:
            bRet = (pstcI2c->IBSR_f.SPC == 1) ? TRUE : FALSE;
            break;  
        case I2cBusStatus:
            bRet = (pstcI2c->IBSR_f.BB == 1) ? TRUE : FALSE;
            break;  
        case I2cBusErr:
            bRet = (pstcI2c->IBCR_f.BER == 1) ? TRUE : FALSE;
            break; 
        case I2cRxTxInt:
            bRet = (pstcI2c->IBCR_f.INT == 1) ? TRUE : FALSE;
            break;
        case I2cDevAddrMatch:
            if((pstcI2c->IBCR_f.MSS == 0) && (pstcI2c->IBCR_f.ACT_SCC == 1))
            {
                bRet = TRUE;
            }
            else
            {
                bRet = FALSE;
            }
            break;
        default:
            break;  
    }
    
    return bRet;
}


/**
 ******************************************************************************
 ** \brief Clear status of I2C according to status type
 **
 ** \param [in] pstcI2c      Pointer to I2C instance   
 ** \param [in] enStatus     I2C status type
 ** \arg   I2cOverrunError          I2C overrun error
 ** \arg   I2cRxFull                I2C receive buffer full
 ** \arg   I2cTxEmpty               I2C tranfer buffer empty
 ** \arg   I2cTxFifoRequest         I2C transfer FIFO request
 ** \arg   I2cTxIdle                I2C tranfer idle status
 ** \arg   I2cFirstByteDetect       I2C first byte detection
 ** \arg   I2cReservedByteDetect    I2C reserved address detection
 ** \arg   I2cStopDetect            I2C stop condition detection
 ** \arg   I2cBusStatus             I2C bus status 
 ** \arg   I2cBusErr                I2C bus error
 ** \arg   I2cRxTxInt               I2C transfer and receive interrupt flag
 ** 
 ** \retval Ok                    Status has been cleared normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcI2c == NULL
 ** \note  The following status can only be cleared by hardware behavior:
 **        - I2cRxFull
 **        - I2cTxEmpty
 **        - I2cTxIdle
 **        - I2cFirstByteDetect
 **        - I2cReservedByteDetect
 **        - I2cBusStatus
 **        - I2cBusErr
 ** 
 ******************************************************************************/
en_result_t Mfs_I2c_ClrStatus(volatile stc_mfsn_i2c_t* pstcI2c,
                               en_i2c_status_t enStatus)
{
    if (NULL == pstcI2c)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enStatus)
    {
        case I2cOverrunError:
            pstcI2c->SSR_f.REC = 1;
            break;  
        case I2cRxFull:
        case I2cTxEmpty:
        case I2cTxIdle:
        case I2cDevAddrMatch:  
            break;  
        case I2cTxFifoRequest:
            pstcI2c->FCR1_f.FDRQ = 0u;
            break;  
        case I2cFirstByteDetect:
            break;  
        case I2cReservedByteDetect:
            break;  
        case I2cStopDetect:
            pstcI2c->IBSR_f.SPC = 0u;
            break;  
        case I2cBusStatus:
            break;  
        case I2cBusErr:
            break;  
        case I2cRxTxInt:
            //pstcI2c->IBCR_f.INT = 0;
            pstcI2c->IBCR &= ~0x41u;
            break;
        default:
            return ErrorInvalidParameter;  
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get direction of I2C data in slave mode
 **
 ** \param [in] pstcI2c      Pointer to I2C instance   
 ** 
 ** \retval i2c_master_tx_slave_rx  Master will send data to slave 
 ** \retval i2c_slave_tx_master_rx  Master will receive data from slave
 ** 
 ** This function can be called after receiving the device address from master
 ** in the slave mode.
 **
 ******************************************************************************/
en_i2c_data_dir_t Mfs_I2c_GetDataDir(volatile stc_mfsn_i2c_t* pstcI2c)
{
    en_i2c_data_dir_t enDir;
    
    (pstcI2c->IBSR_f.TRX == 0) ? (enDir = i2c_master_tx_slave_rx) : (enDir = i2c_slave_tx_master_rx);
    
    return enDir;
}

/**
 ******************************************************************************
 ** \brief Reset I2C FIFO
 **
 ** \param [in] pstcI2c    Pointer to I2C instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** 
 ** \retval Ok                    FIFO has been successfully reset
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcI2c == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_I2c_ResetFifo (volatile stc_mfsn_i2c_t* pstcI2c, 
                               en_mfs_fifo_t enFifo)
{
    if (NULL == pstcI2c)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFifo)
    {
        case MfsFifo1:
            pstcI2c->FCR0_f.FCL1 = 1;
            break;
        case MfsFifo2:
            pstcI2c->FCR0_f.FCL2 = 1;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Set I2C FIFO count
 **
 ** \param [in] pstcI2c    Pointer to I2C instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** \param [in] u8Count    FIFO count
 ** 
 ** \retval Ok                    FIFO count has been successfully set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcI2c == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_I2c_SetFifoCount(volatile stc_mfsn_i2c_t* pstcI2c,
                                  en_mfs_fifo_t enFifo,
                                  uint8_t u8Count)
{
    if (NULL == pstcI2c)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFifo)
    {
        case MfsFifo1:
            pstcI2c->FBYTE1 = u8Count;
            break;
        case MfsFifo2:
            pstcI2c->FBYTE2 = u8Count;
            break;
        default:
           return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get I2C FIFO count
 **
 ** \param [in] pstcI2c    Pointer to I2C instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** 
 ** \retval FIFO count      
 ** 
 ** This function gets the current data count in selected FIFO.
 **
 ** \note 0xFF will be return value if FIFO index is error.
 **
 ******************************************************************************/
uint8_t Mfs_I2c_GetFifoCount(volatile stc_mfsn_i2c_t* pstcI2c,
                             en_mfs_fifo_t enFifo)
{
    uint8_t u8FifoCnt = 0xFFu;
    
    switch(enFifo)
    {
        case MfsFifo1:
            u8FifoCnt = pstcI2c->FBYTE1;
            break;
        case MfsFifo2:
            u8FifoCnt = pstcI2c->FBYTE2;
            break;
        default:
            break;
    }
    
    return u8FifoCnt;
}

/******************************************************************************
 * LIN driver functions
 ******************************************************************************/

/**
 ******************************************************************************
 ** \brief Return the internal data for a certain MFS instance.
 **
 ** \param pstcLin Pointer to LIN instance
 **
 ** \return Pointer to internal data or NULL if instance is not enabled (or not known)
 **
 ******************************************************************************/
static stc_mfs_intern_data_t* MfsLinGetInternDataPtr(volatile stc_mfsn_lin_t* pstcLin)
{
    stc_mfs_intern_data_t* pstcInternDataPtr = NULL;
    uint32_t u32Instance;

    if (NULL != pstcLin)
    {
        for (u32Instance = 0u; u32Instance < (uint32_t)MfsInstanceIndexMax; u32Instance++)
        {
            if (pstcLin == m_astcMfsInstanceDataLut[u32Instance].stcInstance.pstcLinInstance)
            {
                pstcInternDataPtr = &m_astcMfsInstanceDataLut[u32Instance].stcInternData;
                break;
            }
        }
            
    }

    return (pstcInternDataPtr);
} /* MfsGetInternDataPtr */

#if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON) ||  \
    (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON) ||  \
    (PDL_INTERRUPT_ENABLE_MFS2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS10 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS3 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS11 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS4 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS12 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS5 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS13 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS6 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS14 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFS7 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS15 == PDL_ON)
/**
 ******************************************************************************
 ** \brief Return instance index for a certain LIN instance.
 **
 ** \param pstcLin Pointer to LIN instance
 **
 ** \return Instance index
 **
 ******************************************************************************/
static en_mfs_instance_index_t MfsLinGetInstanceIndex(volatile stc_mfsn_lin_t* pstcLin)
{
    en_mfs_instance_index_t enInstanceIndex = MfsInstanceIndexUnknown;
    uint32_t u32Instance;

    if (NULL != pstcLin)
    {
        for (u32Instance = 0u; u32Instance < (uint32_t)MfsInstanceIndexMax; u32Instance++)
        {
            if (pstcLin == m_astcMfsInstanceDataLut[u32Instance].stcInstance.pstcLinInstance)
            {
                enInstanceIndex = m_astcMfsInstanceDataLut[u32Instance].stcInternData.enIndex;
                break;
            }
        }
            
    }

    return (enInstanceIndex);
} /* MfsLinGetInstanceIndex */      
      
/**
 ******************************************************************************
 ** \brief LIN receive interrupt service routine.
 **
 ** \param pstcLin Pointer to LIN instance   
 ** \param pstcMfsInternData Pointer to MFS internal data structure     
 **
 ** This function is called on Transmit Interrupt set by the MFS. 
 ** 
 ******************************************************************************/
void MfsLinIrqHandlerRx( volatile stc_mfsn_lin_t*   pstcLin,
                         stc_mfs_intern_data_t* pstcMfsInternData)
{
    if ((NULL != pstcLin) && (NULL != pstcMfsInternData))
    {
        if (NULL != pstcMfsInternData->stcLinInternIntCb.pfnRxIntCb)
        {
            pstcMfsInternData->stcLinInternIntCb.pfnRxIntCb();
        }
    }
} /* MfsIrqHandlerRx */

/**
 ******************************************************************************
 ** \brief LIN transmit interrupt service routine.
 **
 ** \param pstcLin Pointer to LIN instance   
 ** \param pstcMfsInternData Pointer to MFS internal data structure     
 **
 ** This function is called on Transmit Interrupt set by the MFS. 
 ** 
 ******************************************************************************/
void MfsLinIrqHandlerTx( volatile stc_mfsn_lin_t*   pstcLin, 
                         stc_mfs_intern_data_t* pstcMfsInternData)
{
    if ((NULL != pstcLin) && (NULL != pstcMfsInternData))
    {
        if(pstcLin->SSR_f.TDRE == 1)
        {
            if (NULL != pstcMfsInternData->stcLinInternIntCb.pfnTxIntCb)
            {
                pstcMfsInternData->stcLinInternIntCb.pfnTxIntCb();
            }
        }
        
        if(pstcLin->SSR_f.TBI == 1)
        {
            if (NULL != pstcMfsInternData->stcLinInternIntCb.pfnTxIdleIntCb)
            {
                pstcMfsInternData->stcLinInternIntCb.pfnTxIdleIntCb();
            }
        }
        
        if(pstcLin->FCR1_f.FDRQ == 1)
        {
            if (NULL != pstcMfsInternData->stcLinInternIntCb.pfnTxFifoIntCb)
            {
                pstcMfsInternData->stcLinInternIntCb.pfnTxFifoIntCb();
            }
            
            pstcLin->FCR1_f.FDRQ = 0;
        }
        
        
    }
} /* MfsIrqHandlerTx */

/**
 ******************************************************************************
 ** \brief LIN status interrupt service routine.
 **
 ** \param pstcLin Pointer to LIN instance   
 ** \param pstcMfsInternData Pointer to MFS internal data structure     
 **
 ** This function is called on Transmit Interrupt set by the MFS. 
 ** 
 ******************************************************************************/
void MfsLinIrqHandlerStatus( volatile stc_mfsn_lin_t*   pstcLin, 
                             stc_mfs_intern_data_t* pstcMfsInternData)
{
    if(pstcLin->SSR_f.LBD == 1)
    {
        pstcLin->SSR_f.LBD = 0;
        if (NULL != pstcMfsInternData->stcLinInternIntCb.pfnLinBreakIntCb)
        {
            pstcMfsInternData->stcLinInternIntCb.pfnLinBreakIntCb();
        }
    }
}

/**
 ******************************************************************************
 ** \brief Enable LIN interrupts
 **
 ** \param [in] pstcLin    Pointer to LIN instance   
 ** \param [in] pstcInt    Pointer to the selection structure of LIN interrupt
 ** \param [in] pstcIntCb  Pointer to the structure of LIN interrupt callback 
 **                        functions 
 ** 
 ** \retval Ok                    Interrupts has been enabled and callback 
 **                               functions are set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcLin == NULL
 **                               - pstcInt == NULL
 **                               - pstcIntCb == NULL
 **                               - Other invalid configuration                   
 **
 ** The NVIC IRQ is also disabled in this function if all interrupt sources
 ** are disable.
 ** 
 ******************************************************************************/            
en_result_t Mfs_Lin_EnableInt(volatile stc_mfsn_lin_t* pstcLin, 
                              stc_lin_int_sel_t* pstcInt,
                              stc_lin_int_cb_t* pstcIntCb)
{
    stc_mfs_intern_data_t* pstcMfsInternData;
    en_mfs_instance_index_t enIndex;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsLinGetInternDataPtr(pstcLin);
    enIndex = MfsLinGetInstanceIndex(pstcLin);
  
    if ((NULL == pstcMfsInternData) ||  
        (NULL == pstcInt) ||
        (NULL == pstcIntCb) || 
        (MfsInstanceIndexUnknown == enIndex)  )
    {
        return (ErrorInvalidParameter);
    }
    
    if(pstcInt->bRxInt == TRUE)
    {
        pstcMfsInternData->stcLinInternIntCb.pfnRxIntCb = pstcIntCb->pfnRxIntCb;
        pstcLin->SCR_f.RIE = 1;
    }
    
    if(pstcInt->bTxInt == TRUE)
    {
        pstcMfsInternData->stcLinInternIntCb.pfnTxIntCb = pstcIntCb->pfnTxIntCb;
        pstcLin->SCR_f.TIE = 1;
    }
    
    if(pstcInt->bTxIdle == TRUE)
    {
        pstcMfsInternData->stcLinInternIntCb.pfnTxIdleIntCb = pstcIntCb->pfnTxIdleIntCb;
        pstcLin->SCR_f.TBIE = 1;
    }
    
    if(pstcInt->bTxFifoInt == TRUE)
    {
        pstcMfsInternData->stcLinInternIntCb.pfnTxFifoIntCb = pstcIntCb->pfnTxFifoIntCb;
        pstcLin->FCR1_f.FTIE = 1;
    }
    
    if(pstcInt->bLinBreakInt == TRUE)
    {
        pstcMfsInternData->stcLinInternIntCb.pfnLinBreakIntCb = pstcIntCb->pfnLinBreakIntCb;
        pstcLin->ESCR_f.LBIE = 1;
    }
    
    if(pstcInt->bRxInt == TRUE)
    {
        Mfs_InitIrq(enIndex, FALSE);
    }
    
    if((pstcInt->bTxInt == TRUE) || (pstcInt->bTxIdle == TRUE) || 
       (pstcInt->bTxFifoInt == TRUE) || (pstcInt->bLinBreakInt == TRUE))
    {
        Mfs_InitIrq(enIndex, TRUE);
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable LIN interrupts
 **
 ** \param [in] pstcLin    Pointer to LIN instance   
 ** \param [in] pstcInt    Pointer to the selection structure of LIN interrupt
 ** 
 ** \retval Ok                    Interrupts has been disabled and callback 
 **                               fucntion pointers are set to NULL 
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcLin == NULL
 **                               - pstcInt == NULL                
 **
 ** The NVIC IRQ is also disabled in this function.
 ** 
 ******************************************************************************/
en_result_t Mfs_Lin_DisableInt(volatile stc_mfsn_lin_t* pstcLin, 
                               stc_lin_int_sel_t* pstcInt)
{
    stc_mfs_intern_data_t* pstcMfsInternData;
    en_mfs_instance_index_t enIndex;
    uint8_t u8Ch;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsLinGetInternDataPtr(pstcLin);
    enIndex = MfsLinGetInstanceIndex(pstcLin);
    
    if ((NULL == pstcMfsInternData) ||  
        (NULL == pstcInt))
    {
        return (ErrorInvalidParameter);
    }
    
    u8Ch = LinInstanceToIndex(pstcLin);
    
    if(pstcInt->bRxInt == TRUE)
    {
        pstcMfsInternData->stcLinInternIntCb.pfnRxIntCb = NULL;
        pstcLin->SCR_f.RIE = 0;
    }
    
    if(pstcInt->bTxInt == TRUE)
    {
        pstcMfsInternData->stcLinInternIntCb.pfnTxIntCb = NULL;
        pstcLin->SCR_f.TIE = 0;
    }
    
    if(pstcInt->bTxIdle == TRUE)
    {
        pstcMfsInternData->stcLinInternIntCb.pfnTxIdleIntCb = NULL;
        pstcLin->SCR_f.TBIE = 0;
    }
    
    if(pstcInt->bTxFifoInt == TRUE)
    {
        pstcMfsInternData->stcLinInternIntCb.pfnTxFifoIntCb = NULL;
        pstcLin->FCR1_f.FTIE = 0;
    }
    
    if(pstcInt->bLinBreakInt == TRUE)
    {
        pstcMfsInternData->stcLinInternIntCb.pfnLinBreakIntCb = NULL;
        pstcLin->ESCR_f.LBIE = 0;
    }
    
    // Only when RX interrupts and DMA interrupt(share same IRQ with MFS ch.14, ch.15) 
    // are all disabled, disable RX IRQ
    do
    {
        if(pstcLin->SCR_f.RIE == 1)
        {
            break;
        }
         
        if(u8Ch == 14u)
        {
            if((FM0P_DMAC->DMACB0_f.CI == 1) || (FM0P_DMAC->DMACB0_f.EI == 1))
            {
                break;
            }
        }
  
        if(u8Ch == 15u)
        {
            if((FM0P_DMAC->DMACB2_f.CI == 1) || (FM0P_DMAC->DMACB2_f.EI == 1))
            {
                break;
            }
        }
   
        Mfs_DeInitIrq(enIndex, FALSE);
      
    }while(0);
    
    // Only when TX interrupts and DMA interrupt(share same IRQ with MFS ch.14, ch.15) 
    // are all disabled, disable TX IRQ
    do
    {
        if(pstcLin->SCR_f.TIE == 1)
        {
            break;
        }
        
        if(pstcLin->SCR_f.TBIE == 1)
        {
            break;
        }
        
        if(pstcLin->FCR1_f.FTIE == 1)
        {
            break;
        }
        
        if(pstcLin->SCR_f.LBR == 1)
        {
            break;
        }
     
        if(u8Ch == 14u)
        {
            if((FM0P_DMAC->DMACB1_f.CI == 0) && (FM0P_DMAC->DMACB1_f.EI == 0))
            {
                break;
            }
        }
   
        if(u8Ch == 15u)
        {
            if((FM0P_DMAC->DMACB3_f.CI == 0) && (FM0P_DMAC->DMACB3_f.EI == 0))
            {
                break;
            }
        }
        
        Mfs_DeInitIrq(enIndex, TRUE);
      
    }while(0);
    
    return Ok;
}



#endif     

/**
 ******************************************************************************
 ** \brief Initialization of a MFS module to activate as LIN.
 **
 ** Set registers to active MFS as LIN.
 **  
 ** \param [in]   pstcLin         Pointer to LIN instance register area
 ** \param [in]   pstcConfig      MFS LIN configuration
 ** 
 ** \retval Ok                        Process successfully done.
 ** \retval ErrorInvalidParameter     If one of following conditions are met:
 **             - pstcLin == NULL
 **             - pstcConfig == NULL
 **             - pstcMfsInternData == NULL (invalid or disabled MFS unit
 **                                          (PDL_PERIPHERAL_ENABLE_MFS)) 
 **
 ******************************************************************************/
en_result_t Mfs_Lin_Init( volatile stc_mfsn_lin_t*        pstcLin, 
                          const stc_mfs_lin_config_t* pstcConfig)
{
    stc_mfs_intern_data_t* pstcMfsInternData;
    uint32_t u32DummyCnt = 100;
  
    /* Preset local register variables to zero */
    stc_mfs_lin_smr_field_t  stcSMR  = { 0 };
    stc_mfs_lin_scr_field_t  stcSCR  = { 0 };
    stc_mfs_lin_escr_field_t stcESCR = { 0 };

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsLinGetInternDataPtr(pstcLin);
    
    /* Parameter check and get ptr to internal data struct */
    if ((NULL == pstcMfsInternData)
    ||  (NULL == pstcConfig)
       )
    {
        return (ErrorInvalidParameter);
    }
    
    /* Set MFS mode in the internal structure */
    pstcMfsInternData->enMode = MfsLinMode;

    /* First of all set MFS to Asynchronous mode 0 */
    pstcLin->SMR = 0u;

    /* Then we disable TX and RX for safe operation */
    pstcLin->SCR = 0u;

    /* Clear MFS by setting the Software Reset bit */
    pstcLin->SCR_f.UPCL = TRUE;

    /* Set LIN mode to HW and preset variable */
    stcSMR.MD = 3u;

    /* Enable serial output */
    stcSMR.SOE = TRUE;

    /* Operation Mode Selection */
    switch (pstcConfig->enMsMode)
    {
        /* Master */
        case LinMasterMode:
            stcSCR.MS = FALSE;
            break;
        /* Slave */
        case LinSlaveMode:
            stcSCR.MS = TRUE;
            break;
        default:
            return (ErrorInvalidParameter);
    }

    /* Set LIN master configuration */
    if (LinMasterMode == pstcConfig->enMsMode)
    {
        /* Stop bits configuration */
        switch (pstcConfig->enStopBits)
        {
            case LinOneStopBit:
                stcSMR.SBL   = FALSE;
                stcESCR.ESBL = FALSE;
                break;
            case LinTwoStopBits:
                stcSMR.SBL   = TRUE;
                stcESCR.ESBL = FALSE;
                break;
            case LinThreeStopBits:
                stcSMR.SBL   = FALSE;
                stcESCR.ESBL = TRUE;
                break;
            case LinFourStopBits:
                stcSMR.SBL   = TRUE;
                stcESCR.ESBL = TRUE;
                break;
            default:
                return (ErrorInvalidParameter);
        }
        /* Break length configuration */
        switch (pstcConfig->enBreakLength)
        {
            case LinBreakLength13:
                stcESCR.LBL = 0u;
                break;
            case LinBreakLength14:
                stcESCR.LBL = 1u;
                break;
            case LinBreakLength15:
                stcESCR.LBL = 2u;
                break;
            case LinBreakLength16:
                stcESCR.LBL = 3u;
                break;
            default:
                return (ErrorInvalidParameter);
        }
        /* Delimiter length configuration */
        switch (pstcConfig->enDelimiterLength)
        {
            case LinDelimiterLength1:
                stcESCR.DEL = 0u;
                break;
            case LinDelimiterLength2:
                stcESCR.DEL = 1u;
                break;
            case LinDelimiterLength3:
                stcESCR.DEL = 2u;
                break;
            case LinDelimiterLength4:
                stcESCR.DEL = 3u;
                break;
            default:
                return (ErrorInvalidParameter);
        }
    }

    /* Set Baudrate */
    (void)Mfs_Lin_SetBaudRate(pstcLin, pstcConfig->u32BaudRate);

    /* Set registers value */
    pstcLin->SMR_f  = stcSMR;
    pstcLin->SCR_f  = stcSCR;
    pstcLin->ESCR_f = stcESCR;
    
    while(u32DummyCnt--)
    {
        pstcLin->SSR_f.LBD = 0;
    }
    
    if(pstcConfig->bEnableFifo == TRUE)
    {
        /* Reset FIFO */
        pstcLin->FCR0_f.FCL1 = 1;
        pstcLin->FCR0_f.FCL2 = 1;
        /* Enable FIFO receive Idle detection */
        pstcLin->FCR1_f.FRIIE = 1;
        /* Selection TX and RX FIFO  */
        switch(pstcConfig->stcFifoConfig.enFifoSel)
        {
            case MfsTxFifo1RxFifo2:
                pstcLin->FCR1_f.FSEL = 0;
                break;
            case MfsTxFifo2RxFifo1:
                pstcLin->FCR1_f.FSEL = 1;
                break;
            default:
               return (ErrorInvalidParameter);
        }
        /* Set FIFO count */
        pstcLin->FBYTE1 = pstcConfig->stcFifoConfig.u8ByteCount1;
        pstcLin->FBYTE2 = pstcConfig->stcFifoConfig.u8ByteCount2;
        /* Enable FIFO  */
        pstcLin->FCR0_f.FE1 = 1;
        pstcLin->FCR0_f.FE2 = 1;
    }
    
    return (Ok);
} /* Mfs_Lin_Init */

/**
 ******************************************************************************
 ** \brief Deinitialisation of a MFS module activating as LIN.
 **
 ** All used LIN register are reset to their default values.
 **  
 ** \param [in]   pstcLin      Pointer to LIN instance register area
 ** 
 ** \retval Ok                        Process successfully done.
 ** \retval ErrorInvalidParameter     If one of following conditions are met:
 **             - pstcLin == NULL
 **             - pstcMfsInternData == NULL (invalid or disabled MFS unit
 **                                          (PDL_PERIPHERAL_ENABLE_MFS)) 
 **
 ******************************************************************************/
en_result_t Mfs_Lin_DeInit(volatile stc_mfsn_lin_t* pstcLin)
{
    en_result_t            enResult;
    stc_mfs_intern_data_t* pstcMfsInternData;

    /* Check for valid pointer and get pointer to internal data struct ... */
    pstcMfsInternData = MfsLinGetInternDataPtr(pstcLin);
    /* ... and check */
    if (NULL == pstcMfsInternData)
    {
        enResult = ErrorInvalidParameter;
    }
    else
    {
        pstcLin->SCR_f.TXE = FALSE;
        pstcLin->SCR_f.RXE = FALSE;

        /* Baud Rate Generation Reload Reset */
        pstcLin->BGR = 0u;
            
        /* Serial Mode Register clear all bits (valid in any mode) */
        pstcLin->SMR = 0u;
           
        /* Reset Mfs receive and transmit bit to default */
        /* and clear all error flags */
        pstcLin->SCR = 0u;

        /* Clear reception Errors */
        pstcLin->SSR_f.REC = TRUE;
           
        /* Reset all other used register to default value */
        pstcLin->SSR  = 0u;
        pstcLin->ESCR = 0u;
        pstcLin->FCR0 = 0u;
        pstcLin->FCR1 = 0u;

        /* Clear MFS by setting the Software Reset bit */
        pstcLin->SCR_f.UPCL = TRUE;

        enResult = Ok;
    }

    return (enResult);
} /* Mfs_Lin_DeInit */


/**
 ******************************************************************************
 ** \brief Set the baudrate of LIN
 **
 ** \param [in] pstcLin     Pointer to LIN instance   
 ** \param [in] u32BaudRate Baudrate value [bps]
 ** 
 ** \retval Ok                    LIN baud rate has been successfully modified
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcLin == NULL
 **                               - u32BaudRate < 3         
 ** \note
 ** The LIN baud rate can be initialized in the Mfs_Lin_Init() and be modified
 ** with the function.
 ** 
 ******************************************************************************/
en_result_t Mfs_Lin_SetBaudRate(volatile stc_mfsn_lin_t* pstcLin,
                                uint32_t u32BaudRate)
{
    uint32_t u32Pclk1;
  
    if ((NULL == pstcLin) || (u32BaudRate < 3))
    {
        return ErrorInvalidParameter;
    }
    
    u32Pclk1 = __HCLK/(1ul << (APBC1_PSR_Val & 0x03u)); /* MFS is attached on APB1 bus */
    
    pstcLin->BGR_f.BGR = (u32Pclk1/u32BaudRate) - 1;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Generate LIN break filed 
 **
 ** \param [in] pstcLin     Pointer to LIN instance   
 ** 
 ** \retval Ok                    Break filed has been successfully generated
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcLin == NULL  
 ** 
 ******************************************************************************/
en_result_t Mfs_Lin_GenerateBreakField(volatile stc_mfsn_lin_t* pstcLin)
{
    if (NULL == pstcLin)
    {
        return ErrorInvalidParameter;
    }
    
    pstcLin->SCR_f.LBR = 1;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Enable LIN functions
 **
 ** \param [in] pstcLin    Pointer to LIN instance   
 ** \param [in] enFunc     LIN function types
 ** \arg   LinTx    LIN transfer function
 ** \arg   LinRx    LIN receive function
 ** 
 ** \retval Ok                    Function has been enabled normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcLin == NULL                
 ** 
 ******************************************************************************/
en_result_t Mfs_Lin_EnableFunc(volatile stc_mfsn_lin_t* pstcLin, en_lin_func_t enFunc)
{
    if (NULL == pstcLin)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFunc)
    {
        case LinTx:
            pstcLin->SCR_f.TXE = 1;
            break;
        case LinRx:
            pstcLin->SCR_f.RXE = 1;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable LIN functions
 **
 ** \param [in] pstcLin    Pointer to LIN instance   
 ** \param [in] enFunc     LIN function types
 ** \arg   LinTx    LIN transfer function
 ** \arg   LinRx    LIN receive function
 ** 
 ** \retval Ok                    Function has been disabled normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcLin == NULL                  
 ** 
 ******************************************************************************/
en_result_t Mfs_Lin_DisableFunc(volatile stc_mfsn_lin_t* pstcLin, en_lin_func_t enFunc)
{
    if (NULL == pstcLin)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFunc)
    {
        case LinTx:
            pstcLin->SCR_f.TXE = 0;
            break;
        case LinRx:
            pstcLin->SCR_f.RXE = 0;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get status of LIN according to status type
 **
 ** \param [in] pstcLin      Pointer to LIN instance   
 ** \param [in] enStatus     LIN status type
 ** \arg   LinParityError   LIN parity error
 ** \arg   LinFrameError    LIN frame error
 ** \arg   LinRxFull        LIN receive buffer full
 ** \arg   LinTxEmpty       LIN tranfer buffer empty
 ** \arg   LinTxIdle        LIN tranfer idle status
 ** \arg   LinBreakFlag     LIN break field detection flag
 ** \arg   LinTxFifoRequest LIN transfer FIFO request
 ** 
 ** \retval FALSE                 If one of following conditions are met:
 **                               - No LIN parity error [enStatus = LinParityError]
 **                               - No LIN frame error  [enStatus = LinFrameError]    
 **                               - LIN receive buffer is not full [enStatus = LinRxFull]
 **                               - LIN tranfer buffer is not empty [enStatus = LinTxEmpty]
 **                               - LIN tranfer is on-going [enStatus = LinTxIdle]
 **                               - LIN break field is not detected [enStatus = LinBreakFlag]
 **                               - No LIN transfer FIFO request [enStatus = LinTxFifoRequest] 
 ** \retval TRUE                 If one of following conditions are met:
 **                               - LIN parity error occurs [enStatus = LinParityError]
 **                               - LIN frame error occurs  [enStatus = LinFrameError]   
 **                               - LIN receive buffer is full [enStatus = LinRxFull]
 **                               - LIN tranfer buffer is empty [enStatus = LinTxEmpty]
 **                               - LIN tranfer is idle [enStatus = LinTxIdle]
 **                               - LIN break field is detected [enStatus = LinBreakFlag]
 **                               - LIN transfer FIFO request issues [enStatus = LinTxFifoRequest] 
 ** 
 ******************************************************************************/
boolean_t Mfs_Lin_GetStatus(volatile stc_mfsn_lin_t* pstcLin, 
                            en_lin_status_t enStatus)
{
    boolean_t bResult = FALSE;
    
    switch(enStatus)
    {
        case LinFrameError:
            bResult = (pstcLin->SSR_f.FRE == 1) ? TRUE : FALSE;
            break;  
        case LinOverrunError:
            bResult = (pstcLin->SSR_f.ORE == 1) ? TRUE : FALSE;
            break;  
        case LinRxFull:
            bResult = (pstcLin->SSR_f.RDRF == 1) ? TRUE : FALSE;
            break;  
        case LinTxEmpty:
            bResult = (pstcLin->SSR_f.TDRE == 1) ? TRUE : FALSE;
            break;  
        case LinTxIdle:
            bResult = (pstcLin->SSR_f.TBI == 1) ? TRUE : FALSE;
            break;  
        case LinBreakFlag:
            bResult = (pstcLin->SSR_f.LBD == 1) ? TRUE : FALSE;
            break;
        case LinTxFifoRequest:
            bResult = (pstcLin->FCR1_f.FDRQ == 1) ? TRUE : FALSE;
            break;  
        default:
            break;  
    }
    
    return bResult;
}

/**
 ******************************************************************************
 ** \brief Clear status of LIN according to status type
 **
 ** \param [in] pstcLin      Pointer to LIN instance   
 ** \param [in] enStatus     LIN status type
 ** \arg   LinParityError   LIN parity error
 ** \arg   LinFrameError    LIN frame error
 ** \arg   LinRxFull        LIN receive buffer full
 ** \arg   LinTxEmpty       LIN tranfer buffer empty
 ** \arg   LinTxIdle        LIN tranfer idle status
 ** \arg   LinBreakFlag     LIN break field detection flag
 ** \arg   LinTxFifoRequest LIN transfer FIFO request
 ** 
 ** \retval Ok                    Status has been cleared normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcLin == NULL
 ** \note  The following status can only be cleared by hardware behavior:
 **        - LinRxFull
 **        - LinTxEmpty
 **        - LinTxIdle
 ** 
 ******************************************************************************/
en_result_t Mfs_Lin_ClrStatus(volatile stc_mfsn_lin_t* pstcLin,
                              en_lin_status_t enStatus)
{
    if (NULL == pstcLin)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enStatus)
    {
        case LinFrameError:
        case LinOverrunError:
            pstcLin->SSR_f.REC = 1;
            break;  
        case LinRxFull:
        case LinTxEmpty:
        case LinTxIdle:
            break;  
        case LinBreakFlag:
            pstcLin->SSR_f.LBD = 0;
            break;
        case LinTxFifoRequest:
            pstcLin->FCR1_f.FDRQ = 0;
            break;  
        default:
            return ErrorInvalidParameter;  
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Write LIN data buffer
 **
 ** \param [in] pstcLin   Pointer to LIN instance   
 ** \param [in] Data      Data to be sent
 ** 
 ** \retval Ok                    Data has been successfully sent
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcLin == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_Lin_SendData(volatile stc_mfsn_lin_t* pstcLin, uint8_t Data)
{
    if (NULL == pstcLin)
    {
        return ErrorInvalidParameter;
    }
    
    pstcLin->TDR = Data;
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Read LIN data buffer
 **
 ** \param [in] pstcLin   Pointer to LIN instance   
 ** 
 ** \retval Receive data        
 ** 
 ******************************************************************************/
uint8_t Mfs_Lin_ReceiveData(volatile stc_mfsn_lin_t* pstcLin)
{    
    return (pstcLin->RDR);
}

/**
 ******************************************************************************
 ** \brief Reset LIN FIFO
 **
 ** \param [in] pstcLin    Pointer to LIN instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** 
 ** \retval Ok                    FIFO has been successfully reset
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcLin == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_Lin_ResetFifo (volatile stc_mfsn_lin_t* pstcLin, 
                               en_mfs_fifo_t enFifo)
{
    if (NULL == pstcLin)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFifo)
    {
        case MfsFifo1:
            pstcLin->FCR0_f.FCL1 = 1;
            break;
        case MfsFifo2:
            pstcLin->FCR0_f.FCL2 = 1;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Set LIN FIFO count
 **
 ** \param [in] pstcLin    Pointer to LIN instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** \param [in] u8Count    FIFO count
 ** 
 ** \retval Ok                    FIFO count has been successfully set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcLin == NULL           
 ** 
 ******************************************************************************/
en_result_t Mfs_Lin_SetFifoCount(volatile stc_mfsn_lin_t* pstcLin,
                                 en_mfs_fifo_t enFifo,
                                 uint8_t u8Count)
{
    if (NULL == pstcLin)
    {
        return ErrorInvalidParameter;
    }
    
    switch(enFifo)
    {
        case MfsFifo1:
            pstcLin->FBYTE1 = u8Count;
            break;
        case MfsFifo2:
            pstcLin->FBYTE2 = u8Count;
            break;
        default:
           return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get LIN FIFO count
 **
 ** \param [in] pstcLin    Pointer to LIN instance   
 ** \param [in] enFifo     FIFO1 or FIFO2
 ** 
 ** \retval FIFO count      
 ** 
 ** This function gets the current data count in selected FIFO.
 **
 ** \note 0xFF will be return value if FIFO index is error.
 **
 ******************************************************************************/
uint8_t Mfs_Lin_GetFifoCount(volatile stc_mfsn_lin_t* pstcLin,
                             en_mfs_fifo_t enFifo)
{
    uint8_t u8FifoCnt = 0xFFu;
    
    switch(enFifo)
    {
        case MfsFifo1:
            u8FifoCnt = pstcLin->FBYTE1;
            break;
        case MfsFifo2:
            u8FifoCnt = pstcLin->FBYTE2;
            break;
        default:
           break;
    }
    
    return u8FifoCnt;
}

//@}

#endif /* #if (defined(PDL_PERIPHERAL_MFS_ACTIVE)) */
/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
